@@ -1,4 +1,4 @@
-/* im_abs()
+/* absolute value
  *
  * Copyright: 1990, N. Dessipris, based on im_powtra()
  * Author: Nicos Dessipris
@@ -24,6 +24,9 @@
  * 	- remove liboil
  * 6/11/11
  * 	- redone as a class
+ * 3/12/13
+ * 	- add orc, though the speed improvement vs. gcc's auto-vectorizer
+ * 	  seems very marginal 
  */
 
 /*
@@ -93,25 +96,19 @@ vips_abs_build( VipsObject *object )
 /* Integer abs operation: just test and negate.
  */
 #define ABS_INT( TYPE ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict q = (TYPE *) out; \
 	int x; \
 	\
-	for( x = 0; x < sz; x++ ) { \
-		TYPE v = p[x]; \
-		\
-		if( v < 0 ) \
-			q[x] = 0 - v; \
-		else \
-			q[x] = v; \
-	} \
+	for( x = 0; x < sz; x++ ) \
+		q[x] = p[x] < 0 ? 0 - p[x] : p[x]; \
 }
 
 /* Float abs operation: call fabs().
  */
 #define ABS_FLOAT( TYPE ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict q = (TYPE *) out; \
 	int x; \
 	\
 	for( x = 0; x < sz; x++ ) \
@@ -124,8 +121,8 @@ vips_abs_build( VipsObject *object )
 #ifdef HAVE_HYPOT
 
 #define ABS_COMPLEX( TYPE ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict q = (TYPE *) out; \
 	int x; \
 	\
 	for( x = 0; x < sz; x++ ) { \
@@ -137,8 +134,8 @@ vips_abs_build( VipsObject *object )
 #else /*HAVE_HYPOT*/
 
 #define ABS_COMPLEX( TYPE ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict q = (TYPE *) out; \
 	int x; \
 	\
 	for( x = 0; x < sz; x++ ) { \
@@ -168,21 +165,36 @@ static void
 vips_abs_buffer( VipsArithmetic *arithmetic, 
 	VipsPel *out, VipsPel **in, int width )
 {
-	VipsUnary *unary = VIPS_UNARY( arithmetic );
-	const int bands = vips_image_get_bands( unary->in );
+	VipsArithmeticClass *class = VIPS_ARITHMETIC_GET_CLASS( arithmetic );
+	VipsImage *im = arithmetic->ready[0];
+	const int bands = vips_image_get_bands( im );
 	int sz = width * bands;
 
-	switch( vips_image_get_format( unary->in ) ) {
-        case VIPS_FORMAT_CHAR: 		ABS_INT( signed char ); break; 
-        case VIPS_FORMAT_SHORT: 	ABS_INT( signed short ); break; 
-        case VIPS_FORMAT_INT: 		ABS_INT( signed int ); break; 
-        case VIPS_FORMAT_FLOAT: 	ABS_FLOAT( float ); break; 
-        case VIPS_FORMAT_DOUBLE:	ABS_FLOAT( double ); break; 
-        case VIPS_FORMAT_COMPLEX:	ABS_COMPLEX( float ); break;
-        case VIPS_FORMAT_DPCOMPLEX:	ABS_COMPLEX( double ); break;
+	VipsVector *v;
 
-	default:
-		g_assert( 0 );
+	if( (v = vips_arithmetic_get_vector( class, 
+		vips_image_get_format( im ) )) ) {
+		VipsExecutor ex;
+
+		vips_executor_set_program( &ex, v, sz );
+		vips_executor_set_array( &ex, v->s[0], in[0] );
+		vips_executor_set_destination( &ex, out );
+
+		vips_executor_run( &ex );
+	}
+	else {
+		switch( vips_image_get_format( im ) ) {
+		case VIPS_FORMAT_CHAR: 		ABS_INT( signed char ); break; 
+		case VIPS_FORMAT_SHORT: 	ABS_INT( signed short ); break; 
+		case VIPS_FORMAT_INT: 		ABS_INT( signed int ); break; 
+		case VIPS_FORMAT_FLOAT: 	ABS_FLOAT( float ); break; 
+		case VIPS_FORMAT_DOUBLE:	ABS_FLOAT( double ); break; 
+		case VIPS_FORMAT_COMPLEX:	ABS_COMPLEX( float ); break;
+		case VIPS_FORMAT_DPCOMPLEX:	ABS_COMPLEX( double ); break;
+
+		default:
+			g_assert( 0 );
+		}
 	}
 }
 
@@ -201,7 +213,7 @@ vips_abs_buffer( VipsArithmetic *arithmetic,
 
 /* Format doesn't change with abs, other than complex -> real.
  */
-static const VipsBandFormat vips_bandfmt_abs[10] = {
+static const VipsBandFormat vips_abs_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    UC, C,  US, S,  UI, I,  F,  F,  D,  D 
 };
@@ -212,13 +224,26 @@ vips_abs_class_init( VipsAbsClass *class )
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
 	VipsArithmeticClass *aclass = VIPS_ARITHMETIC_CLASS( class );
 
+	VipsVector *v;
+
 	object_class->nickname = "abs";
 	object_class->description = _( "absolute value of an image" );
 	object_class->build = vips_abs_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_abs );
-
 	aclass->process_line = vips_abs_buffer;
+
+	vips_arithmetic_set_format_table( aclass, vips_abs_format_table ); 
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_CHAR );
+	vips_vector_asm2( v, "absb", "d1", "s1" ); 
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_SHORT );
+	vips_vector_asm2( v, "absw", "d1", "s1" ); 
+
+	v = vips_arithmetic_get_program( aclass, VIPS_FORMAT_INT );
+	vips_vector_asm2( v, "absl", "d1", "s1" ); 
+
+	vips_arithmetic_compile( aclass );
 }
 
 static void
