@@ -478,6 +478,8 @@ vips_arithmetic_gen( VipsRegion *or,
 	p[i] = NULL;
 	q = (VipsPel *) VIPS_REGION_ADDR( or, r->left, r->top );
 
+	VIPS_GATE_START( "vips_arithmetic_gen: work" );
+
 	for( y = 0; y < r->height; y++ ) {
 		class->process_line( arithmetic, q, p, r->width );
 
@@ -486,6 +488,8 @@ vips_arithmetic_gen( VipsRegion *or,
 		q += VIPS_REGION_LSKIP( or );
 	}
 
+	VIPS_GATE_STOP( "vips_arithmetic_gen: work" );
+
 	return( 0 );
 }
 
@@ -545,15 +549,16 @@ vips_arithmetic_build( VipsObject *object )
 	 */
 	arithmetic->ready = size;
 
-	if( vips_image_copy_fields_array( arithmetic->out, 
-		arithmetic->ready ) ) 
+	if( vips_image_pipeline_array( arithmetic->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, arithmetic->ready ) ) 
 		return( -1 );
-        vips_demand_hint_array( arithmetic->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, arithmetic->ready );
 
 	arithmetic->out->Bands = arithmetic->ready[0]->Bands;
-	arithmetic->out->BandFmt = 
-		aclass->format_table[arithmetic->ready[0]->BandFmt];
+	if( arithmetic->format != VIPS_FORMAT_NOTSET )
+		arithmetic->out->BandFmt = arithmetic->format;
+	else
+		arithmetic->out->BandFmt = 
+			aclass->format_table[arithmetic->ready[0]->BandFmt];
 
 	if( vips_image_generate( arithmetic->out,
 		vips_start_many, vips_arithmetic_gen, vips_stop_many, 
@@ -590,6 +595,7 @@ static void
 vips_arithmetic_init( VipsArithmetic *arithmetic )
 {
 	arithmetic->base_bands = 1;
+	arithmetic->format = VIPS_FORMAT_NOTSET;
 }
 
 void 
