@@ -108,18 +108,18 @@ vips_boolean_build( VipsObject *object )
 }
 
 #define LOOP( TYPE, OP ) { \
-	TYPE *left = (TYPE *) in[0]; \
-	TYPE *right = (TYPE *) in[1]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict left = (TYPE *) in[0]; \
+	TYPE * restrict right = (TYPE *) in[1]; \
+	TYPE * restrict q = (TYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = left[x] OP right[x]; \
 }
 
 #define FLOOP( TYPE, OP ) { \
-	TYPE *left = (TYPE *) in[0]; \
-	TYPE *right = (TYPE *) in[1]; \
-	int *q = (int *) out; \
+	TYPE * restrict left = (TYPE *) in[0]; \
+	TYPE * restrict right = (TYPE *) in[1]; \
+	int * restrict q = (int *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = ((int) left[x]) OP ((int) right[x]); \
@@ -191,7 +191,7 @@ vips_boolean_buffer( VipsArithmetic *arithmetic,
 
 /* Type conversions for boolean. 
  */
-static const VipsBandFormat vips_bandfmt_boolean[10] = {
+static const VipsBandFormat vips_boolean_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    UC, C,  US, S,  UI, I,  I,  I,  I,  I,
 };
@@ -210,10 +210,10 @@ vips_boolean_class_init( VipsBooleanClass *class )
 	object_class->description = _( "boolean operation on two images" );
 	object_class->build = vips_boolean_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_boolean );
-
 	aclass->process_line = vips_boolean_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_boolean_format_table ); 
+
 	VIPS_ARG_ENUM( class, "boolean", 200, 
 		_( "Operation" ), 
 		_( "boolean to perform" ),
@@ -258,7 +258,7 @@ vips_booleanv( VipsImage *left, VipsImage *right, VipsImage **out,
  * one-band image by joining n copies of the one-band image together, and then
  * the two n-band images are operated upon.
  *
- * The two input images are cast up to the smallest common type (see table 
+ * The two input images are cast up to the smallest common format (see table 
  * Smallest common format in 
  * <link linkend="VIPS-arithmetic">arithmetic</link>).
  *
@@ -513,10 +513,10 @@ vips_boolean_const_class_init( VipsBooleanConstClass *class )
 		_( "boolean operations against a constant" );
 	object_class->build = vips_boolean_const_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_boolean );
-
 	aclass->process_line = vips_boolean_const_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_boolean_format_table ); 
+
 	VIPS_ARG_ENUM( class, "boolean", 200, 
 		_( "Operation" ), 
 		_( "boolean to perform" ),
