@@ -80,8 +80,8 @@ typedef VipsUnaryClass VipsComplexClass;
 G_DEFINE_TYPE( VipsComplex, vips_complex, VIPS_TYPE_UNARY );
 
 #define LOOP( IN, OUT, OP ) { \
-	IN *p = (IN *) in[0]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		OP( q, p[x], 0.0 ); \
@@ -91,8 +91,8 @@ G_DEFINE_TYPE( VipsComplex, vips_complex, VIPS_TYPE_UNARY );
 }
 
 #define CLOOP( IN, OUT, OP ) { \
-	IN *p = (IN *) in[0]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		OP( q, p[0], p[1] ); \
@@ -196,7 +196,7 @@ vips_complex_buffer( VipsArithmetic *arithmetic,
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static const VipsBandFormat vips_bandfmt_complex[10] = {
+static const VipsBandFormat vips_complex_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    X,  X,  X,  X,  X,  X,  X,  X,  DX, DX 
 };
@@ -215,10 +215,10 @@ vips_complex_class_init( VipsComplexClass *class )
 	object_class->description = 
 		_( "perform a complex operation on an image" );
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_complex );
-
 	aclass->process_line = vips_complex_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_complex_format_table ); 
+
 	VIPS_ARG_ENUM( class, "cmplx", 200, 
 		_( "Operation" ), 
 		_( "complex to perform" ),
@@ -336,13 +336,13 @@ vips_conj( VipsImage *in, VipsImage **out, ... )
 }
 
 typedef struct _VipsComplex2 {
-	VipsUnary parent_instance;
+	VipsBinary parent_instance;
 
 	VipsOperationComplex2 cmplx;
 
 } VipsComplex2;
 
-typedef VipsUnaryClass VipsComplex2Class;
+typedef VipsBinaryClass VipsComplex2Class;
 
 G_DEFINE_TYPE( VipsComplex2, vips_complex2, VIPS_TYPE_BINARY );
 
@@ -475,7 +475,7 @@ vips_complex2_buffer( VipsArithmetic *arithmetic,
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static const VipsBandFormat vips_bandfmt_complex2[10] = {
+static const VipsBandFormat vips_complex2_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    X,  X,  X,  X,  X,  X,  X,  X,  DX, DX 
 };
@@ -494,16 +494,16 @@ vips_complex2_class_init( VipsComplex2Class *class )
 	object_class->description = 
 		_( "perform a binary complex operation on two images" );
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_complex2 );
-
 	aclass->process_line = vips_complex2_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_complex2_format_table ); 
+
 	VIPS_ARG_ENUM( class, "cmplx", 200, 
 		_( "Operation" ), 
 		_( "binary complex operation to perform" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsComplex2, cmplx ),
-		VIPS_TYPE_OPERATION_COMPLEX, 
+		VIPS_TYPE_OPERATION_COMPLEX2, 
 			VIPS_OPERATION_COMPLEX2_CROSS_PHASE ); 
 }
 
@@ -689,7 +689,7 @@ vips_complexget_buffer( VipsArithmetic *arithmetic,
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static const VipsBandFormat vips_bandfmt_complexget[10] = {
+static const VipsBandFormat vips_complexget_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    UC, C,  US, S,  UI, I,  F,  F,  D,  D
 };
@@ -708,10 +708,11 @@ vips_complexget_class_init( VipsComplexgetClass *class )
 	object_class->description = _( "get a component from a complex image" );
 	object_class->build = vips_complexget_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_complexget );
-
 	aclass->process_line = vips_complexget_buffer;
 
+	vips_arithmetic_set_format_table( aclass, 
+		vips_complexget_format_table ); 
+
 	VIPS_ARG_ENUM( class, "get", 200, 
 		_( "Operation" ), 
 		_( "complex to perform" ),
@@ -891,7 +892,7 @@ vips_complexform_buffer( VipsArithmetic *arithmetic,
 /* Type promotion for division. Sign and value preserving. Make sure 
  * these match the case statement in complexform_buffer() above.
  */
-static int vips_bandfmt_complexform[10] = {
+static int vips_complexform_format_table[10] = {
 /* UC  C   US  S   UI  I  F  X  D  DX */
    X,  X,  X,  X,  X,  X, X, X, DX,DX
 };
@@ -907,9 +908,10 @@ vips_complexform_class_init( VipsComplexformClass *class )
 		_( "form a complex image from two real images" );
 	object_class->build = vips_complexform_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_complexform );
-
 	aclass->process_line = vips_complexform_buffer;
+
+	vips_arithmetic_set_format_table( aclass, 
+		vips_complexform_format_table ); 
 }
 
 static void
