@@ -38,6 +38,12 @@
  * 31/10/11
  * 	- rework as a class
  * 	- removed the 1-ary constant path, no faster
+ * 30/11/13
+ * 	- 1ary is back, faster with gcc 4.8
+ * 3/12/13
+ * 	- try an ORC path with the band loop unrolled
+ * 14/1/14
+ * 	- add uchar output option
  */
 
 /*
@@ -92,6 +98,10 @@ typedef struct _VipsLinear {
 	VipsArea *a;
 	VipsArea *b;
 
+	/* uchar output.
+	 */
+	gboolean uchar;
+
 	/* Our constants expanded to match arith->ready in size.
 	 */
 	int n;
@@ -111,6 +121,7 @@ vips_linear_build( VipsObject *object )
 	VipsArithmetic *arithmetic = VIPS_ARITHMETIC( object );
 	VipsUnary *unary = (VipsUnary *) object;
 	VipsLinear *linear = (VipsLinear *) object;
+
 	int i;
 
 	/* If we have a three-element vector we need to bandup the image to
@@ -154,38 +165,118 @@ vips_linear_build( VipsObject *object )
 		}
 	}
 
+	if( linear->uchar )
+		arithmetic->format = VIPS_FORMAT_UCHAR;
+
 	if( VIPS_OBJECT_CLASS( vips_linear_parent_class )->build( object ) )
 		return( -1 );
 
 	return( 0 );
 }
 
+/* Non-complex input, any output, all bands of the constant equal.
+ */
+#define LOOP1( IN, OUT ) { \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
+	OUT a1 = a[0]; \
+	OUT b1 = b[0]; \
+	int sz = width * nb; \
+	\
+	for( x = 0; x < sz; x++ ) \
+		q[x] = a1 * (OUT) p[x] + b1; \
+}
+
 /* Non-complex input, any output.
  */
 #define LOOPN( IN, OUT ) { \
-	IN *p = (IN *) in[0]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( i = 0, x = 0; x < width; x++ ) \
 		for( k = 0; k < nb; k++, i++ ) \
 			q[i] = a[k] * (OUT) p[i] + b[k]; \
 }
 
+#define LOOP( IN, OUT ) { \
+	if( linear->a->n == 1 && linear->b->n == 1 ) { \
+		LOOP1( IN, OUT ); \
+	} \
+	else { \
+		LOOPN( IN, OUT ); \
+	} \
+}
+
 /* Complex input, complex output. 
  */
 #define LOOPCMPLXN( IN, OUT ) { \
-	IN *p = (IN *) in[0]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < width; x++ ) \
 		for( k = 0; k < nb; k++ ) { \
 			q[0] = a[k] * p[0] + b[k]; \
-			q[1] = a[k] * p[1]; \
+			q[1] = p[1]; \
 			q += 2; \
 			p += 2; \
 		} \
 }
 
+/* Non-complex input, any output, all bands of the constant equal, uchar
+ * output.
+ */
+#define LOOP1uc( IN ) { \
+	IN * restrict p = (IN *) in[0]; \
+	VipsPel * restrict q = (VipsPel *) out; \
+	float a1 = a[0]; \
+	float b1 = b[0]; \
+	int sz = width * nb; \
+	\
+	for( x = 0; x < sz; x++ ) { \
+		float t = a1 * p[x] + b1; \
+		\
+		q[x] = VIPS_CLIP( 0, t, 255 ); \
+	} \
+}
+
+/* Non-complex input, uchar output.
+ */
+#define LOOPNuc( IN ) { \
+	IN * restrict p = (IN *) in[0]; \
+	VipsPel * restrict q = (VipsPel *) out; \
+	\
+	for( i = 0, x = 0; x < width; x++ ) \
+		for( k = 0; k < nb; k++, i++ ) { \
+			double t = a[k] * p[i] + b[k]; \
+			\
+			q[i] = VIPS_CLIP( 0, t, 255 ); \
+		} \
+}
+
+#define LOOPuc( IN ) { \
+	if( linear->a->n == 1 && linear->b->n == 1 ) { \
+		LOOP1uc( IN ); \
+	} \
+	else { \
+		LOOPNuc( IN ); \
+	} \
+}
+
+/* Complex input, uchar output. 
+ */
+#define LOOPCMPLXNuc( IN ) { \
+	IN * restrict p = (IN *) in[0]; \
+	VipsPel * restrict q = (VipsPel *) out; \
+	\
+	for( i = 0, x = 0; x < width; x++ ) \
+		for( k = 0; k < nb; k++, i++ ) { \
+			double t = a[k] * p[0] + b[k]; \
+			\
+			q[i] = VIPS_CLIP( 0, t, 255 ); \
+			p += 2; \
+		} \
+}
+
 /* Lintra a buffer, n set of scale/offset.
  */
 static void
@@ -194,27 +285,65 @@ vips_linear_buffer( VipsArithmetic *arithmetic,
 {
 	VipsImage *im = arithmetic->ready[0];
 	VipsLinear *linear = (VipsLinear *) arithmetic;
-	double *a = linear->a_ready;
-	double *b = linear->b_ready;
+	double * restrict a = linear->a_ready;
+	double * restrict b = linear->b_ready;
 	int nb = im->Bands;
 
 	int i, x, k;
 
-	switch( vips_image_get_format( im ) ) {
-        case VIPS_FORMAT_UCHAR: 	LOOPN( unsigned char, float ); break;
-        case VIPS_FORMAT_CHAR: 		LOOPN( signed char, float ); break; 
-        case VIPS_FORMAT_USHORT: 	LOOPN( unsigned short, float ); break; 
-        case VIPS_FORMAT_SHORT: 	LOOPN( signed short, float ); break; 
-        case VIPS_FORMAT_UINT: 		LOOPN( unsigned int, float ); break; 
-        case VIPS_FORMAT_INT: 		LOOPN( signed int, float );  break; 
-        case VIPS_FORMAT_FLOAT: 	LOOPN( float, float ); break; 
-        case VIPS_FORMAT_DOUBLE:	LOOPN( double, double ); break; 
-        case VIPS_FORMAT_COMPLEX:	LOOPCMPLXN( float, float ); break; 
-        case VIPS_FORMAT_DPCOMPLEX:	LOOPCMPLXN( double, double ); break;
+	if( linear->uchar )
+		switch( vips_image_get_format( im ) ) {
+		case VIPS_FORMAT_UCHAR: 	
+			LOOPuc( unsigned char ); break;
+		case VIPS_FORMAT_CHAR: 		
+			LOOPuc( signed char ); break; 
+		case VIPS_FORMAT_USHORT: 	
+			LOOPuc( unsigned short ); break; 
+		case VIPS_FORMAT_SHORT: 	
+			LOOPuc( signed short ); break; 
+		case VIPS_FORMAT_UINT: 		
+			LOOPuc( unsigned int ); break; 
+		case VIPS_FORMAT_INT: 		
+			LOOPuc( signed int );  break; 
+		case VIPS_FORMAT_FLOAT: 	
+			LOOPuc( float ); break; 
+		case VIPS_FORMAT_DOUBLE:	
+			LOOPuc( double ); break; 
+		case VIPS_FORMAT_COMPLEX:	
+			LOOPCMPLXNuc( float ); break; 
+		case VIPS_FORMAT_DPCOMPLEX:	
+			LOOPCMPLXNuc( double ); break;
+
+		default:
+			g_assert( 0 );
+		}
+	else
+		switch( vips_image_get_format( im ) ) {
+		case VIPS_FORMAT_UCHAR: 	
+			LOOP( unsigned char, float ); break;
+		case VIPS_FORMAT_CHAR: 		
+			LOOP( signed char, float ); break; 
+		case VIPS_FORMAT_USHORT: 	
+			LOOP( unsigned short, float ); break; 
+		case VIPS_FORMAT_SHORT: 	
+			LOOP( signed short, float ); break; 
+		case VIPS_FORMAT_UINT: 		
+			LOOP( unsigned int, float ); break; 
+		case VIPS_FORMAT_INT: 		
+			LOOP( signed int, float );  break; 
+		case VIPS_FORMAT_FLOAT: 	
+			LOOP( float, float ); break; 
+		case VIPS_FORMAT_DOUBLE:	
+			LOOP( double, double ); break; 
+		case VIPS_FORMAT_COMPLEX:	
+			LOOPCMPLXN( float, float ); break; 
+		case VIPS_FORMAT_DPCOMPLEX:	
+			LOOPCMPLXN( double, double ); break;
+
+		default:
+			g_assert( 0 );
+		}
 
-        default:
-		g_assert( 0 );
-        }
 }
 
 /* Save a bit of typing.
@@ -232,7 +361,7 @@ vips_linear_buffer( VipsArithmetic *arithmetic,
 
 /* Format doesn't change with linear.
  */
-static const VipsBandFormat vips_bandfmt_linear[10] = {
+static const VipsBandFormat vips_linear_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    F,  F,  F,  F,  F,  F,  F,  X,  D,  DX 
 };
@@ -251,10 +380,10 @@ vips_linear_class_init( VipsLinearClass *class )
 	object_class->description = _( "calculate (a * in + b)" );
 	object_class->build = vips_linear_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_linear );
-
 	aclass->process_line = vips_linear_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_linear_format_table ); 
+
 	VIPS_ARG_BOXED( class, "a", 110, 
 		_( "a" ), 
 		_( "Multiply by this" ),
@@ -268,6 +397,14 @@ vips_linear_class_init( VipsLinearClass *class )
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsLinear, b ),
 		VIPS_TYPE_ARRAY_DOUBLE );
+
+	VIPS_ARG_BOOL( class, "uchar", 112, 
+		_( "uchar" ), 
+		_( "Output should be uchar" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsLinear, uchar ),
+		FALSE );
+
 }
 
 static void
@@ -303,9 +440,14 @@ vips_linearv( VipsImage *in, VipsImage **out,
  * @n: length of constant arrays
  * @...: %NULL-terminated list of optional named arguments
  *
+ * Optional arguments:
+ *
+ * @uchar: output uchar pixels
+ *
  * Pass an image through a linear transform, ie. (@out = @in * @a + @b). Output
- * is always float for integer input, double for double input, complex for
- * complex input and double complex for double complex input.
+ * is float for integer input, double for double input, complex for
+ * complex input and double complex for double complex input. Set @uchar to
+ * output uchar pixels. 
  *
  * If the arrays of constants have just one element, that constant is used for 
  * all image bands. If the arrays have more than one element and they have 
@@ -339,6 +481,10 @@ vips_linear( VipsImage *in, VipsImage **out, double *a, double *b, int n, ... )
  * @b: constant for addition
  * @...: %NULL-terminated list of optional named arguments
  *
+ * Optional arguments:
+ *
+ * @uchar: output uchar pixels
+ *
  * Run vips_linear() with a single constant. 
  *
  * See also: vips_linear().
