@@ -97,8 +97,8 @@ vips_math_build( VipsObject *object )
 }
 
 #define LOOP( IN, OUT, OP ) { \
-	IN *p = (IN *) in[0]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = OP( p[x] ); \
@@ -180,7 +180,7 @@ vips_math_buffer( VipsArithmetic *arithmetic,
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static const VipsBandFormat vips_bandfmt_math[10] = {
+static const VipsBandFormat vips_math_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    F,  F,  F,  F,  F,  F,  F,  X,  D,  DX 
 };
@@ -199,10 +199,10 @@ vips_math_class_init( VipsMathClass *class )
 	object_class->description = _( "perform a math function on an image" );
 	object_class->build = vips_math_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_math );
-
 	aclass->process_line = vips_math_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_math_format_table ); 
+
 	VIPS_ARG_ENUM( class, "math", 200, 
 		_( "Operation" ), 
 		_( "math to perform" ),
