@@ -102,9 +102,9 @@ vips_math2_build( VipsObject *object )
 }
 
 #define LOOP( IN, OUT, OP ) { \
-	IN *p1 = (IN *) in[0]; \
-	IN *p2 = (IN *) in[1]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict p1 = (IN *) in[0]; \
+	IN * restrict p2 = (IN *) in[1]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		OP( q[x], p1[x], p2[x] ); \
@@ -137,12 +137,9 @@ vips_math2_build( VipsObject *object )
 	double left = (double) (X); \
 	double right = (double) (E); \
 	\
-	if( left == 0.0 && right < 0.0 ) \
-		/* Division by zero! Difficult to report tho' \
-		 */ \
-		(Y) = 0.0; \
-	else \
-		(Y) = pow( left, right ); \
+	/* Division by zero! Difficult to report tho' \
+	 */ \
+	(Y) = (left == 0.0 && right < 0.0) ? 0.0 : pow( left, right ); \
 }
 
 #define WOP( Y, X, E ) POW( Y, E, X )
@@ -181,7 +178,7 @@ vips_math2_buffer( VipsArithmetic *arithmetic,
 
 /* Type promotion for math2. Keep in sync with math2_buffer() above.
  */
-static int vips_bandfmt_math2[10] = {
+static int vips_math2_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    F,  F,  F,  F,  F,  F,  F,  X,  D,  DX 
 };
@@ -200,10 +197,10 @@ vips_math2_class_init( VipsMath2Class *class )
 	object_class->description = _( "binary math operations" );
 	object_class->build = vips_math2_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_math2 );
-
 	aclass->process_line = vips_math2_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_math2_format_table ); 
+
 	VIPS_ARG_ENUM( class, "math2", 200, 
 		_( "Operation" ), 
 		_( "math to perform" ),
@@ -248,7 +245,7 @@ vips_math2v( VipsImage *left, VipsImage *right, VipsImage **out,
  * one-band image by joining n copies of the one-band image together, and then
  * the two n-band images are operated upon.
  *
- * The two input images are cast up to the smallest common type (see table 
+ * The two input images are cast up to the smallest common format (see table 
  * Smallest common format in 
  * <link linkend="VIPS-arithmetic">arithmetic</link>), and that format is the
  * result type.
@@ -355,9 +352,9 @@ vips_math2_const_build( VipsObject *object )
 }
 
 #define LOOPC( IN, OUT, OP ) { \
-	IN *p = (IN *) in[0]; \
-	OUT *q = (OUT *) out; \
-	double *c = (double *) uconst->c_ready; \
+	IN * restrict p = (IN *) in[0]; \
+	OUT * restrict q = (OUT *) out; \
+	double * restrict c = (double *) uconst->c_ready; \
 	\
 	for( i = 0, x = 0; x < width; x++ ) \
 		for( b = 0; b < bands; b++, i++ ) \
@@ -398,10 +395,10 @@ vips_math2_const_class_init( VipsMath2ConstClass *class )
 	object_class->description = _( "pow( @in, @c )" );
 	object_class->build = vips_math2_const_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_math2 );
-
 	aclass->process_line = vips_math2_const_buffer;
 
+	vips_arithmetic_set_format_table( aclass, vips_math2_format_table ); 
+
 	VIPS_ARG_ENUM( class, "math2", 200, 
 		_( "Operation" ), 
 		_( "math to perform" ),
