@@ -81,9 +81,9 @@ G_DEFINE_TYPE( VipsMultiply, vips_multiply, VIPS_TYPE_BINARY );
 /* Complex multiply.
  */
 #define CLOOP( TYPE ) { \
-	TYPE *left = (TYPE *) in[0]; \
-	TYPE *right = (TYPE *) in[1]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict left = (TYPE *) in[0]; \
+	TYPE * restrict right = (TYPE *) in[1]; \
+	TYPE * restrict q = (TYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		double x1 = left[0]; \
@@ -104,9 +104,9 @@ G_DEFINE_TYPE( VipsMultiply, vips_multiply, VIPS_TYPE_BINARY );
 /* Real multiply.
  */
 #define RLOOP( IN, OUT ) { \
-	IN *left = (IN *) in[0]; \
-	IN *right = (IN *) in[1]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict left = (IN *) in[0]; \
+	IN * restrict right = (IN *) in[1]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = left[x] * right[x]; \
@@ -158,7 +158,7 @@ vips_multiply_buffer( VipsArithmetic *arithmetic,
 /* Type promotion for multiplication. Sign and value preserving. Make sure 
  * these match the case statement in multiply_buffer() above.
  */
-static int vips_bandfmt_multiply[10] = {
+static int vips_multiply_format_table[10] = {
 /* UC  C   US  S   UI  I  F  X  D  DX */
    US, S,  UI, I,  UI, I, F, X, D, DX
 };
@@ -172,9 +172,9 @@ vips_multiply_class_init( VipsMultiplyClass *class )
 	object_class->nickname = "multiply";
 	object_class->description = _( "multiply two images" );
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_multiply );
-
 	aclass->process_line = vips_multiply_buffer;
+
+	vips_arithmetic_set_format_table( aclass, vips_multiply_format_table ); 
 }
 
 static void
@@ -198,7 +198,7 @@ vips_multiply_init( VipsMultiply *multiply )
  * one-band image by joining n copies of the one-band image together, and then
  * the two n-band images are operated upon.
  *
- * The two input images are cast up to the smallest common type (see table 
+ * The two input images are cast up to the smallest common format (see table 
  * Smallest common format in 
  * <link linkend="VIPS-arithmetic">arithmetic</link>), then the 
  * following table is used to determine the output type:
