@@ -110,18 +110,18 @@ vips_relational_build( VipsObject *object )
 }
 
 #define RLOOP( TYPE, ROP ) { \
-	TYPE *left = (TYPE *) in[0]; \
-	TYPE *right = (TYPE *) in[1]; \
-	VipsPel *q = (VipsPel *) out; \
+	TYPE * restrict left = (TYPE *) in[0]; \
+	TYPE * restrict right = (TYPE *) in[1]; \
+	VipsPel * restrict q = (VipsPel *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = (left[x] ROP right[x]) ? 255 : 0; \
 }
 
 #define CLOOP( TYPE, COP ) { \
-	TYPE *left = (TYPE *) in[0]; \
-	TYPE *right = (TYPE *) in[1]; \
-	VipsPel *q = (VipsPel *) out; \
+	TYPE * restrict left = (TYPE *) in[0]; \
+	TYPE * restrict right = (TYPE *) in[1]; \
+	VipsPel * restrict q = (VipsPel *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		q[x] = COP( left[0], left[1], right[0], right[1]) ? 255 : 0; \
@@ -200,7 +200,7 @@ vips_relational_buffer( VipsArithmetic *arithmetic,
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static const VipsBandFormat vips_bandfmt_relational[10] = {
+static const VipsBandFormat vips_relational_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    UC, UC, UC, UC, UC, UC, UC, UC, UC, UC
 };
@@ -220,10 +220,11 @@ vips_relational_class_init( VipsRelationalClass *class )
 		_( "a relational operation on a pair of images" );
 	object_class->build = vips_relational_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_relational );
-
 	aclass->process_line = vips_relational_buffer;
 
+	vips_arithmetic_set_format_table( aclass, 
+		vips_relational_format_table ); 
+
 	VIPS_ARG_ENUM( class, "relational", 200, 
 		_( "Operation" ), 
 		_( "relational to perform" ),
@@ -268,7 +269,7 @@ vips_relationalv( VipsImage *left, VipsImage *right, VipsImage **out,
  * one-band image by joining n copies of the one-band image together, and then
  * the two n-band images are operated upon.
  *
- * The two input images are cast up to the smallest common type (see table 
+ * The two input images are cast up to the smallest common format (see table 
  * Smallest common format in 
  * <link linkend="VIPS-arithmetic">arithmetic</link>).
  *
@@ -478,8 +479,8 @@ vips_relational_const_build( VipsObject *object )
 }
 
 #define RLOOPC( TYPE, OP ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *c = (TYPE *) uconst->c_ready; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict c = (TYPE *) uconst->c_ready; \
  	\
 	for( i = 0, x = 0; x < width; x++ ) \
 		for( b = 0; b < bands; b++, i++ ) \
@@ -487,10 +488,10 @@ vips_relational_const_build( VipsObject *object )
 }
 
 #define CLOOPC( TYPE, OP ) { \
-	TYPE *p = (TYPE *) in[0]; \
+	TYPE * restrict p = (TYPE *) in[0]; \
  	\
 	for( i = 0, x = 0; x < width; x++ ) { \
-		TYPE *c = (TYPE *) uconst->c_ready; \
+		TYPE * restrict c = (TYPE *) uconst->c_ready; \
 		\
 		for( b = 0; b < bands; b++, i++ ) { \
 			out[i] = OP( p[0], p[1], c[0], c[1]) ? 255 : 0; \
@@ -557,10 +558,11 @@ vips_relational_const_class_init( VipsRelationalConstClass *class )
 		_( "relational operations against a constant" );
 	object_class->build = vips_relational_const_build;
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_relational );
-
 	aclass->process_line = vips_relational_const_buffer;
 
+	vips_arithmetic_set_format_table( aclass, 
+		vips_relational_format_table ); 
+
 	VIPS_ARG_ENUM( class, "relational", 200, 
 		_( "Operation" ), 
 		_( "relational to perform" ),
