@@ -58,8 +58,8 @@ typedef VipsUnaryClass VipsSignClass;
 G_DEFINE_TYPE( VipsSign, vips_sign, VIPS_TYPE_UNARY );
 
 #define CSIGN( TYPE ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	TYPE *q = (TYPE *) out; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	TYPE * restrict q = (TYPE *) out; \
 	int x; \
 	\
 	for( x = 0; x < sz; x++ ) { \
@@ -83,8 +83,8 @@ G_DEFINE_TYPE( VipsSign, vips_sign, VIPS_TYPE_UNARY );
 }
 
 #define SIGN( TYPE ) { \
-	TYPE *p = (TYPE *) in[0]; \
-	signed char *q = (signed char *) out; \
+	TYPE * restrict p = (TYPE *) in[0]; \
+	signed char * restrict q = (signed char *) out; \
 	int x; \
 	\
 	for( x = 0; x < sz; x++ ) { \
@@ -137,7 +137,7 @@ vips_sign_buffer( VipsArithmetic *arithmetic,
 #define D VIPS_FORMAT_DOUBLE
 #define DX VIPS_FORMAT_DPCOMPLEX
 
-static const VipsBandFormat vips_bandfmt_sign[10] = {
+static const VipsBandFormat vips_sign_format_table[10] = {
 /* UC  C   US  S   UI  I   F   X   D   DX */
    C,  C,  C,  C,  C,  C,  C,  X,  C,  DX 
 };
@@ -151,9 +151,9 @@ vips_sign_class_init( VipsSignClass *class )
 	object_class->nickname = "sign";
 	object_class->description = _( "unit vector of pixel" );
 
-	vips_arithmetic_set_format_table( aclass, vips_bandfmt_sign );
-
 	aclass->process_line = vips_sign_buffer;
+
+	vips_arithmetic_set_format_table( aclass, vips_sign_format_table ); 
 }
 
 static void
