@@ -83,9 +83,9 @@ typedef VipsBinaryClass VipsSubtractClass;
 G_DEFINE_TYPE( VipsSubtract, vips_subtract, VIPS_TYPE_BINARY );
 
 #define LOOP( IN, OUT ) { \
-	IN *left = (IN *) in[0]; \
-	IN *right = (IN *) in[1]; \
-	OUT *q = (OUT *) out; \
+	IN * restrict left = (IN *) in[0]; \
+	IN * restrict right = (IN *) in[1]; \
+	OUT * restrict q = (OUT *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = left[x] - right[x]; \
@@ -147,7 +147,7 @@ vips_subtract_buffer( VipsArithmetic *arithmetic,
 /* Type promotion for subtraction. Sign and value preserving. Make sure these
  * match the case statement in vips_subtract_buffer() above.
  */
-static const VipsBandFormat bandfmt_subtract[10] = {
+static const VipsBandFormat vips_subtract_format_table[10] = {
 /* UC  C   US  S   UI  I  F  X  D  DX */
    S,  S,  I,  I,  I,  I, F, X, D, DX
 };
@@ -161,9 +161,9 @@ vips_subtract_class_init( VipsSubtractClass *class )
 	object_class->nickname = "subtract";
 	object_class->description = _( "subtract two images" );
 
-	vips_arithmetic_set_format_table( aclass, bandfmt_subtract );
-
 	aclass->process_line = vips_subtract_buffer;
+
+	vips_arithmetic_set_format_table( aclass, vips_subtract_format_table ); 
 }
 
 static void
@@ -188,7 +188,7 @@ vips_subtract_init( VipsSubtract *subtract )
  * one-band image by joining n copies of the one-band image together, and then
  * the two n-band images are operated upon.
  *
- * The two input images are cast up to the smallest common type (see table 
+ * The two input images are cast up to the smallest common format (see table 
  * Smallest common format in 
  * <link linkend="VIPS-arithmetic">arithmetic</link>), then the 
  * following table is used to determine the output type:
