@@ -248,17 +248,17 @@ vips_colour_gen( VipsRegion *or,
 	VipsRegion **ir = (VipsRegion **) seq;
 	VipsColour *colour = VIPS_COLOUR( b ); 
 	VipsColourClass *class = VIPS_COLOUR_GET_CLASS( colour ); 
-	Rect *r = &or->valid;
+	VipsRect *r = &or->valid;
 
-	VipsPel *p[MAX_INPUT_IMAGES], *q;
 	int i, y;
+	VipsPel *p[MAX_INPUT_IMAGES], *q;
 
-	/* Prepare all input regions and make buffer pointers.
-	 */
 	for( i = 0; ir[i]; i++ ) 
 		if( vips_region_prepare( ir[i], r ) ) 
 			return( -1 );
 
+	VIPS_GATE_START( "vips_colour_gen: work" ); 
+
 	for( y = 0; y < r->height; y++ ) {
 		for( i = 0; ir[i]; i++ )
 			p[i] = VIPS_REGION_ADDR( ir[i], r->left, r->top + y );
@@ -268,6 +268,8 @@ vips_colour_gen( VipsRegion *or,
 		class->process_line( colour, q, p, r->width );
 	}
 
+	VIPS_GATE_STOP( "vips_colour_gen: work" ); 
+
 	return( 0 );
 }
 
@@ -320,10 +322,9 @@ vips_colour_build( VipsObject *object )
 	 */
 	g_assert( !colour->in[colour->n] ); 
 
-	if( vips_image_copy_fields_array( colour->out, colour->in ) ) 
+	if( vips_image_pipeline_array( colour->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, colour->in ) ) 
 		return( -1 );
-        vips_demand_hint_array( colour->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, colour->in );
 	colour->out->Coding = colour->coding;
 	colour->out->Type = colour->interpretation;
 	colour->out->BandFmt = colour->format;
@@ -482,12 +483,11 @@ vips_colour_code_build( VipsObject *object )
 	VipsColour *colour = VIPS_COLOUR( object );
 	VipsColourCode *code = VIPS_COLOUR_CODE( object );
 	VipsColourCodeClass *class = VIPS_COLOUR_CODE_GET_CLASS( object ); 
+	VipsImage **t = (VipsImage **) vips_object_local_array( object, 6 );
 
-	VipsImage **t;
 	VipsImage *in;
 	VipsImage *extra;
 
-	t = (VipsImage **) vips_object_local_array( object, 4 );
 
 	in = code->in;
 	extra = NULL;
@@ -539,6 +539,15 @@ vips_colour_code_build( VipsObject *object )
 		in = t[3];
 	}
 
+	if( in &&
+		code->input_coding == VIPS_CODING_NONE &&
+		code->input_interpretation != VIPS_INTERPRETATION_ERROR ) {
+		if( vips_colourspace( in, &t[4], 
+			code->input_interpretation, NULL ) )
+			return( -1 );
+		in = t[4];
+	}
+
 	colour->n = 1;
 	colour->in = (VipsImage **) vips_object_local_array( object, 2 );
 	colour->in[0] = in;
@@ -551,15 +560,19 @@ vips_colour_code_build( VipsObject *object )
 		return( -1 );
 
 	/* Reattach higher bands, if necessary.
+	 *
+	 * Our processing on the first three bands may have changed the image
+	 * format. For example, converting LAB to LABS will make a short
+	 * image. We need to force the extra bands to match this new type. 
 	 */
 	if( extra ) {
 		VipsImage *x;
 
-		if( vips_bandjoin2( colour->out, extra, &x, NULL ) )
+		if( vips_cast( extra, &t[5], colour->out->BandFmt, NULL ) ||
+			vips_bandjoin2( colour->out, t[5], &x, NULL ) )
 			return( -1 );
 
 		VIPS_UNREF( colour->out );
-
 		colour->out = x;
 	}
 
@@ -589,6 +602,7 @@ vips_colour_code_class_init( VipsColourCodeClass *class )
 static void
 vips_colour_code_init( VipsColourCode *code )
 {
+	code->input_interpretation = VIPS_INTERPRETATION_ERROR;
 }
 
 G_DEFINE_ABSTRACT_TYPE( VipsColourDifference, vips_colour_difference, 
