@@ -2,6 +2,12 @@
  *
  * 6/11/12
  * 	- add RGB16 as a destination
+ * 12/1/14
+ * 	- add B_W as a source / dest
+ * 	- add GREY16 as a source / dest
+ * 	- add RGB16 as a source
+ * 19/1/14
+ * 	- auto-decode RAD images
  */
 
 /*
@@ -55,6 +61,58 @@ vips_scRGB2RGB16( VipsImage *in, VipsImage **out, ... )
 	return( vips_scRGB2sRGB( in, out, "depth", 16, NULL ) );
 }
 
+static int
+vips_sRGB2BW( VipsImage *in, VipsImage **out, ... )
+{
+	if( vips_extract_band( in, out, 1, NULL ) )
+		return( -1 );
+	(*out)->Type = VIPS_INTERPRETATION_B_W;
+
+	return( 0 ); 
+}
+
+static int
+vips_BW2sRGB( VipsImage *in, VipsImage **out, ... )
+{
+	VipsImage *t[3];
+
+	t[0] = in;
+	t[1] = in;
+	t[2] = in;
+
+	if( vips_bandjoin( t, out, 3, NULL ) )
+		return( -1 );
+	(*out)->Type = VIPS_INTERPRETATION_sRGB;
+
+	return( 0 );
+}
+
+static int
+vips_RGB162GREY16( VipsImage *in, VipsImage **out, ... )
+{
+	if( vips_extract_band( in, out, 1, NULL ) )
+		return( -1 );
+	(*out)->Type = VIPS_INTERPRETATION_GREY16;
+
+	return( 0 ); 
+}
+
+static int
+vips_GREY162RGB16( VipsImage *in, VipsImage **out, ... )
+{
+	VipsImage *t[3];
+
+	t[0] = in;
+	t[1] = in;
+	t[2] = in;
+
+	if( vips_bandjoin( t, out, 3, NULL ) )
+		return( -1 );
+	(*out)->Type = VIPS_INTERPRETATION_RGB16;
+
+	return( 0 );
+}
+
 /* A colour-transforming function.
  */
 typedef int (*VipsColourTransformFn)( VipsImage *in, VipsImage **out, ... );
@@ -65,8 +123,6 @@ typedef int (*VipsColourTransformFn)( VipsImage *in, VipsImage **out, ... );
 #define MAX_STEPS (10)
 
 /* A route between two colour spaces.
- *
- * 10 steps should be enough for anyone. 
  */
 typedef struct _VipsColourRoute {
 	VipsInterpretation from;
@@ -86,7 +142,9 @@ typedef struct _VipsColourRoute {
 #define scRGB VIPS_INTERPRETATION_scRGB
 #define sRGB VIPS_INTERPRETATION_sRGB
 #define RGB16 VIPS_INTERPRETATION_RGB16
+#define GREY16 VIPS_INTERPRETATION_GREY16
 #define YXY VIPS_INTERPRETATION_YXY
+#define BW VIPS_INTERPRETATION_B_W
 
 /* All the routes we know about.
  */
@@ -98,7 +156,10 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ XYZ, LABS, { vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ XYZ, scRGB, { vips_XYZ2scRGB, NULL } },
 	{ XYZ, sRGB, { vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
+	{ XYZ, BW, { vips_XYZ2scRGB, vips_scRGB2sRGB, vips_sRGB2BW, NULL } },
 	{ XYZ, RGB16, { vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
+	{ XYZ, GREY16, { vips_XYZ2scRGB, vips_scRGB2RGB16, 
+		vips_RGB162GREY16, NULL } },
 	{ XYZ, YXY, { vips_XYZ2Yxy, NULL } },
 
 	{ LAB, XYZ, { vips_Lab2XYZ, NULL } },
@@ -108,8 +169,12 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LAB, LABS, { vips_Lab2LabS, NULL } },
 	{ LAB, scRGB, { vips_Lab2XYZ, vips_XYZ2scRGB, NULL } },
 	{ LAB, sRGB, { vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
+	{ LAB, BW, { vips_Lab2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, 
+		vips_sRGB2BW, NULL } },
 	{ LAB, RGB16, { vips_Lab2XYZ, vips_XYZ2scRGB, 
 		vips_scRGB2RGB16, NULL } },
+	{ LAB, GREY16, { vips_Lab2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
 	{ LAB, YXY, { vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
 
 	{ LABQ, XYZ, { vips_LabQ2Lab, vips_Lab2XYZ, NULL } },
@@ -119,8 +184,11 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LABQ, LABS, { vips_LabQ2LabS, NULL } },
 	{ LABQ, scRGB, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2scRGB } },
 	{ LABQ, sRGB, { vips_LabQ2sRGB, NULL } },
+	{ LABQ, BW, { vips_LabQ2sRGB, vips_sRGB2BW, NULL } },
 	{ LABQ, RGB16, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
 		vips_scRGB2RGB16, NULL } },
+	{ LABQ, GREY16, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
 	{ LABQ, YXY, { vips_LabQ2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
 
 	{ LCH, XYZ, { vips_LCh2Lab, vips_Lab2XYZ, NULL } },
@@ -131,8 +199,12 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LCH, scRGB, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, NULL } },
 	{ LCH, sRGB, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
 		vips_scRGB2sRGB, NULL } },
+	{ LCH, BW, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2sRGB, vips_sRGB2BW, NULL } },
 	{ LCH, RGB16, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
 		vips_scRGB2RGB16, NULL } },
+	{ LCH, GREY16, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
 	{ LCH, YXY, { vips_LCh2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
 
 	{ CMC, XYZ, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, NULL } },
@@ -144,8 +216,12 @@ static VipsColourRoute vips_colour_routes[] = {
 		vips_XYZ2scRGB, NULL } },
 	{ CMC, sRGB, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, 
 		vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
+	{ CMC, BW, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, 
+		vips_XYZ2scRGB, vips_scRGB2sRGB, vips_sRGB2BW, NULL } },
 	{ CMC, RGB16, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, 
 		vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
+	{ CMC, GREY16, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, 
+		vips_XYZ2scRGB, vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
 	{ CMC, YXY, { vips_CMC2LCh, vips_LCh2Lab, vips_Lab2XYZ, 
 		vips_XYZ2Yxy, NULL } },
 
@@ -157,8 +233,12 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ LABS, scRGB, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, NULL } },
 	{ LABS, sRGB, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
 		vips_scRGB2sRGB, NULL } },
+	{ LABS, BW, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2sRGB, vips_sRGB2BW, NULL } },
 	{ LABS, RGB16, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
 		vips_scRGB2RGB16, NULL } },
+	{ LABS, GREY16, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
 	{ LABS, YXY, { vips_LabS2Lab, vips_Lab2XYZ, vips_XYZ2Yxy, NULL } },
 
 	{ scRGB, XYZ, { vips_scRGB2XYZ, NULL } },
@@ -168,8 +248,10 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ scRGB, CMC, { vips_scRGB2XYZ, vips_XYZ2Lab, 
 		vips_Lab2LCh, vips_LCh2CMC, NULL } },
 	{ scRGB, sRGB, { vips_scRGB2sRGB, NULL } },
+	{ scRGB, BW, { vips_scRGB2sRGB, vips_sRGB2BW, NULL } },
 	{ scRGB, LABS, { vips_scRGB2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ scRGB, RGB16, { vips_scRGB2RGB16, NULL } },
+	{ scRGB, GREY16, { vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
 	{ scRGB, YXY, { vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
 
 	{ sRGB, XYZ, { vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
@@ -181,11 +263,71 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ sRGB, CMC, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, 
 		vips_Lab2LCh, vips_LCh2CMC, NULL } },
 	{ sRGB, scRGB, { vips_sRGB2scRGB, NULL } },
+	{ sRGB, BW, { vips_sRGB2BW, NULL } },
 	{ sRGB, LABS, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, 
 		vips_Lab2LabS, NULL } },
 	{ sRGB, RGB16, { vips_sRGB2scRGB, vips_scRGB2RGB16, NULL } },
+	{ sRGB, GREY16, { vips_sRGB2scRGB, vips_scRGB2RGB16, 
+		vips_RGB162GREY16, NULL } },
 	{ sRGB, YXY, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
 
+	{ RGB16, XYZ, { vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
+	{ RGB16, LAB, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, NULL } },
+	{ RGB16, LABQ, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, 
+		vips_Lab2LabQ, NULL } },
+	{ RGB16, LCH, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, 
+		vips_Lab2LCh, NULL } },
+	{ RGB16, CMC, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, 
+		vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ RGB16, scRGB, { vips_sRGB2scRGB, NULL } },
+	{ RGB16, sRGB, { vips_sRGB2scRGB, vips_scRGB2sRGB, NULL } },
+	{ RGB16, BW, { vips_sRGB2scRGB, vips_scRGB2sRGB, vips_sRGB2BW, NULL } },
+	{ RGB16, LABS, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Lab, 
+		vips_Lab2LabS, NULL } },
+	{ RGB16, GREY16, { vips_RGB162GREY16, NULL } },
+	{ RGB16, YXY, { vips_sRGB2scRGB, vips_scRGB2XYZ, vips_XYZ2Yxy, NULL } },
+
+	{ GREY16, XYZ, { vips_GREY162RGB16, vips_sRGB2scRGB, 
+		vips_scRGB2XYZ, NULL } },
+	{ GREY16, LAB, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, NULL } },
+	{ GREY16, LABQ, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ GREY16, LCH, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ GREY16, CMC, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ GREY16, scRGB, { vips_GREY162RGB16, vips_sRGB2scRGB, NULL } },
+	{ GREY16, sRGB, { vips_GREY162RGB16, vips_sRGB2scRGB, 
+		vips_scRGB2sRGB, NULL } },
+	{ GREY16, BW, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2sRGB, 
+		vips_sRGB2BW, NULL } },
+	{ GREY16, LABS, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ GREY16, RGB16, { vips_GREY162RGB16, NULL } },
+	{ GREY16, YXY, { vips_GREY162RGB16, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Yxy, NULL } },
+
+	{ BW, XYZ, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, NULL } },
+	{ BW, LAB, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, NULL } },
+	{ BW, LABQ, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
+	{ BW, LCH, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LCh, NULL } },
+	{ BW, CMC, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LCh, vips_LCh2CMC, NULL } },
+	{ BW, scRGB, { vips_BW2sRGB, vips_sRGB2scRGB, NULL } },
+	{ BW, sRGB, { vips_BW2sRGB, NULL } },
+	{ BW, LABS, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Lab, vips_Lab2LabS, NULL } },
+	{ BW, RGB16, { vips_BW2sRGB, vips_sRGB2scRGB, 
+		vips_scRGB2RGB16, NULL } },
+	{ BW, GREY16, { vips_BW2sRGB, vips_sRGB2scRGB, 
+		vips_scRGB2RGB16, vips_RGB162GREY16, NULL } },
+	{ BW, YXY, { vips_BW2sRGB, vips_sRGB2scRGB, vips_scRGB2XYZ, 
+		vips_XYZ2Yxy, NULL } },
+
 	{ YXY, XYZ, { vips_Yxy2XYZ, NULL } },
 	{ YXY, LAB, { vips_Yxy2XYZ, vips_XYZ2Lab, NULL } },
 	{ YXY, LABQ, { vips_Yxy2XYZ, vips_XYZ2Lab, vips_Lab2LabQ, NULL } },
@@ -195,7 +337,12 @@ static VipsColourRoute vips_colour_routes[] = {
 	{ YXY, LABS, { vips_Yxy2XYZ, vips_XYZ2Lab, vips_Lab2LabS, NULL } },
 	{ YXY, scRGB, { vips_Yxy2XYZ, vips_XYZ2scRGB, NULL } },
 	{ YXY, sRGB, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, NULL } },
-	{ YXY, RGB16, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2RGB16, NULL } },
+	{ YXY, BW, { vips_Yxy2XYZ, vips_XYZ2scRGB, vips_scRGB2sRGB, 
+		vips_sRGB2BW, NULL } },
+	{ YXY, RGB16, { vips_Yxy2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2RGB16, NULL } },
+	{ YXY, GREY16, { vips_Yxy2XYZ, vips_XYZ2scRGB, 
+		vips_scRGB2RGB16, vips_RGB162GREY16, NULL } }
 };
 
 /* Is an image in a supported colourspace.
@@ -254,11 +401,13 @@ vips_colourspace_build( VipsObject *object )
 
 	int i, j;
 	VipsImage *x;
-	VipsImage **t;
+	VipsImage **t = (VipsImage **) 
+		vips_object_local_array( object, 1 );
+	VipsImage **pipe = (VipsImage **) 
+		vips_object_local_array( object, MAX_STEPS );
 
 	VipsInterpretation interpretation;
 
-	t = (VipsImage **) vips_object_local_array( object, MAX_STEPS );
 
 	/* Verify that all input args have been set.
 	 */
@@ -266,7 +415,18 @@ vips_colourspace_build( VipsObject *object )
 		build( object ) )
 		return( -1 );
 
-	interpretation = vips_image_guess_interpretation( colourspace->in );
+	x = colourspace->in;
+
+	/* Unpack radiance-coded images. We can't use interpretation for this,
+	 * since rad images can be scRGB or XYZ.
+	 */
+	if( x->Coding == VIPS_CODING_RAD ) {
+		if( vips_rad2float( x, &t[0], NULL ) )
+			return( -1 );
+		x = t[0]; 
+	}
+
+	interpretation = vips_image_guess_interpretation( x );
 
 	/* Treat RGB and RGB16 as sRGB. If you want some other treatment,
 	 * you'll need to use the icc funcs.
@@ -285,8 +445,6 @@ vips_colourspace_build( VipsObject *object )
 		return( vips_image_write( colourspace->in, colourspace->out ) );
 	}
 
-	x = colourspace->in;
-
 	for( i = 0; i < VIPS_NUMBER( vips_colour_routes ); i++ )
 		if( vips_colour_routes[i].from == interpretation &&
 			vips_colour_routes[i].to == colourspace->space )
@@ -302,9 +460,9 @@ vips_colourspace_build( VipsObject *object )
 	}
 
 	for( j = 0; vips_colour_routes[i].route[j]; j++ ) {
-		if( vips_colour_routes[i].route[j]( x, &t[j], NULL ) ) 
+		if( vips_colour_routes[i].route[j]( x, &pipe[j], NULL ) ) 
 			return( -1 );
-		x = t[j];
+		x = pipe[j];
 	}
 
 	g_object_set( colourspace, "out", vips_image_new(), NULL ); 
