@@ -100,6 +100,8 @@ vips_bandary_gen( VipsRegion *or, void *seq, void *a, void *b, gboolean *stop )
 	p[i] = NULL;
 	q = VIPS_REGION_ADDR( or, r->left, r->top );
 
+	VIPS_GATE_START( "vips_bandary_gen: work" ); 
+
 	for( y = 0; y < r->height; y++ ) {
 		class->process_line( bandary, q, p, r->width );
 
@@ -108,6 +110,8 @@ vips_bandary_gen( VipsRegion *or, void *seq, void *a, void *b, gboolean *stop )
 		q += VIPS_REGION_LSKIP( or );
 	}
 
+	VIPS_GATE_STOP( "vips_bandary_gen: work" ); 
+
 	return( 0 );
 }
 
@@ -147,10 +151,9 @@ vips_bandary_build( VipsObject *object )
 		return( -1 );
 	bandary->ready = size;
 
-	if( vips_image_copy_fields_array( conversion->out, bandary->ready ) )
+	if( vips_image_pipeline_array( conversion->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, bandary->ready ) )
 		return( -1 );
-        vips_demand_hint_array( conversion->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, bandary->ready );
 
 	conversion->out->Bands = bandary->out_bands;
 
