@@ -172,7 +172,7 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 	seq->underflow = 0;
 
 	if( !seq->ir ) {
-		vips_cast_stop( seq, NULL, NULL );
+		vips_cast_stop( seq, a, b );
 		return( NULL );
 	}
 
@@ -182,8 +182,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast int types to an int type.
  */
 #define VIPS_CLIP_INT_INT( ITYPE, OTYPE, VIPS_CLIP ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		int t = p[x]; \
@@ -197,8 +197,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast float types to an int type.
  */
 #define VIPS_CLIP_FLOAT_INT( ITYPE, OTYPE, VIPS_CLIP ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		ITYPE v = floor( p[x] ); \
@@ -212,8 +212,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast complex types to an int type. Just take the real part.
  */
 #define VIPS_CLIP_COMPLEX_INT( ITYPE, OTYPE, VIPS_CLIP ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		ITYPE v = floor( p[0] ); \
@@ -228,8 +228,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast non-complex types to a float type.
  */
 #define VIPS_CLIP_REAL_FLOAT( ITYPE, OTYPE ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) \
 		q[x] = p[x]; \
@@ -238,8 +238,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast complex types to a float type ... just take real.
  */
 #define VIPS_CLIP_COMPLEX_FLOAT( ITYPE, OTYPE ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		q[x] = p[0]; \
@@ -250,8 +250,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast any non-complex to a complex type ... set imaginary to zero.
  */
 #define VIPS_CLIP_REAL_COMPLEX( ITYPE, OTYPE ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		q[0] = p[x]; \
@@ -263,8 +263,8 @@ vips_cast_start( VipsImage *out, void *a, void *b )
 /* Cast any complex to a complex type.
  */
 #define VIPS_CLIP_COMPLEX_COMPLEX( ITYPE, OTYPE ) { \
-	ITYPE *p = (ITYPE *) in; \
-	OTYPE *q = (OTYPE *) out; \
+	ITYPE * restrict p = (ITYPE *) in; \
+	OTYPE * restrict q = (OTYPE *) out; \
 	\
 	for( x = 0; x < sz; x++ ) { \
 		q[0] = p[0]; \
@@ -339,6 +339,8 @@ vips_cast_gen( VipsRegion *or, void *vseq, void *a, void *b,
 	if( vips_region_prepare( ir, r ) )
 		return( -1 );
 
+	VIPS_GATE_START( "vips_cast_gen: work" );
+
 	for( y = to; y < bo; y++ ) {
 		VipsPel *in = VIPS_REGION_ADDR( ir, le, y ); 
 		VipsPel *out = VIPS_REGION_ADDR( or, le, y ); 
@@ -419,6 +421,8 @@ vips_cast_gen( VipsRegion *or, void *vseq, void *a, void *b,
 		} 
 	}
 
+	VIPS_GATE_STOP( "vips_cast_gen: work" );
+
 	return( 0 );
 }
 
@@ -441,10 +445,9 @@ vips_cast_build( VipsObject *object )
 		vips_image_pio_input( cast->in ) )
 		return( -1 );
 
-	if( vips_image_copy_fields( conversion->out, cast->in ) )
+	if( vips_image_pipelinev( conversion->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, cast->in, NULL ) )
 		return( -1 );
-	vips_demand_hint( conversion->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, cast->in, NULL );
 
 	conversion->out->BandFmt = cast->format;
 
