@@ -206,12 +206,15 @@ vips_conversion_operation_init( void )
 	extern GType vips_insert_get_type( void ); 
 	extern GType vips_join_get_type( void ); 
 	extern GType vips_extract_area_get_type( void ); 
+	extern GType vips_crop_get_type( void ); 
 	extern GType vips_extract_band_get_type( void ); 
 	extern GType vips_replicate_get_type( void ); 
 	extern GType vips_cast_get_type( void ); 
 	extern GType vips_bandjoin_get_type( void ); 
+	extern GType vips_bandrank_get_type( void ); 
 	extern GType vips_black_get_type( void ); 
 	extern GType vips_rot_get_type( void ); 
+	extern GType vips_rot45_get_type( void ); 
 	extern GType vips_ifthenelse_get_type( void ); 
 	extern GType vips_recomb_get_type( void ); 
 	extern GType vips_bandmean_get_type( void ); 
@@ -229,7 +232,7 @@ vips_conversion_operation_init( void )
 #endif /*HAVE_PANGOFT2*/
 	extern GType vips_xyz_get_type( void ); 
 	extern GType vips_falsecolour_get_type( void ); 
-	extern GType vips_gammacorrect_get_type( void ); 
+	extern GType vips_gamma_get_type( void ); 
 
 	vips_copy_get_type();
 	vips_tile_cache_get_type(); 
@@ -241,12 +244,15 @@ vips_conversion_operation_init( void )
 	vips_insert_get_type();
 	vips_join_get_type();
 	vips_extract_area_get_type();
+	vips_crop_get_type();
 	vips_extract_band_get_type();
 	vips_replicate_get_type();
 	vips_cast_get_type();
 	vips_bandjoin_get_type();
+	vips_bandrank_get_type();
 	vips_black_get_type();
 	vips_rot_get_type();
+	vips_rot45_get_type();
 	vips_ifthenelse_get_type();
 	vips_recomb_get_type(); 
 	vips_bandmean_get_type(); 
@@ -264,7 +270,7 @@ vips_conversion_operation_init( void )
 #endif /*HAVE_PANGOFT2*/
 	vips_xyz_get_type(); 
 	vips_falsecolour_get_type(); 
-	vips_gammacorrect_get_type(); 
+	vips_gamma_get_type(); 
 }
 
 /* The common part of most binary conversion
