@@ -316,10 +316,9 @@ vips_flatten_build( VipsObject *object )
 		vips_image_pio_input( flatten->in ) )
 		return( -1 );
 
-	if( vips_image_copy_fields( conversion->out, flatten->in ) )
+	if( vips_image_pipelinev( conversion->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, flatten->in, NULL ) )
 		return( -1 );
-	vips_demand_hint( conversion->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, flatten->in, NULL );
 
 	conversion->out->Bands -= 1;
 
