@@ -284,10 +284,9 @@ vips_insert_build( VipsObject *object )
 		insert->main_processed, insert->sub_processed, NULL )) )
 		return( -1 );
 
-	if( vips_image_copy_fields_array( conversion->out, arry ) )
+	if( vips_image_pipeline_array( conversion->out, 
+		VIPS_DEMAND_STYLE_ANY, arry ) )
 		return( -1 );
-        vips_demand_hint_array( conversion->out, 
-		VIPS_DEMAND_STYLE_ANY, arry );
 
 	/* Calculate geometry. 
 	 */
@@ -355,8 +354,6 @@ vips_insert_class_init( VipsInsertClass *class )
 	vobject_class->description = _( "insert an image" );
 	vobject_class->build = vips_insert_build;
 
-	operation_class->flags = VIPS_OPERATION_SEQUENTIAL_UNBUFFERED;
-
 	VIPS_ARG_IMAGE( class, "main", -1, 
 		_( "Main" ), 
 		_( "Main input image" ),
