@@ -59,6 +59,7 @@
  */
 
 /*
+#define VIPS_DEBUG_RED
 #define VIPS_DEBUG
  */
 
@@ -295,8 +296,8 @@ vips_tile_find( VipsBlockCache *cache, int x, int y )
 	/* In cache already?
 	 */
 	if( (tile = vips_tile_search( cache, x, y )) ) {
-		VIPS_DEBUG_MSG( "vips_tile_find: tile %d x %d in cache\n", 
-			x, y ); 
+		VIPS_DEBUG_MSG_RED( "vips_tile_find: "
+			"tile %d x %d in cache\n", x, y ); 
 		return( tile );
 	}
 
@@ -304,8 +305,8 @@ vips_tile_find( VipsBlockCache *cache, int x, int y )
 	 */
 	if( cache->max_tiles == -1 ||
 		cache->ntiles < cache->max_tiles ) {
-		VIPS_DEBUG_MSG( "vips_tile_find: making new tile at %d x %d\n", 
-			x, y ); 
+		VIPS_DEBUG_MSG_RED( "vips_tile_find: "
+			"making new tile at %d x %d\n", x, y ); 
 		if( !(tile = vips_tile_new( cache, x, y )) )
 			return( NULL );
 
@@ -330,7 +331,7 @@ vips_tile_find( VipsBlockCache *cache, int x, int y )
 		return( tile );
 	}
 
-	VIPS_DEBUG_MSG( "tilecache: reusing tile %d x %d\n", 
+	VIPS_DEBUG_MSG_RED( "vips_tile_find: reusing tile %d x %d\n", 
 		tile->pos.left, tile->pos.top );
 
 	if( vips_tile_move( tile, x, y ) )
@@ -366,12 +367,16 @@ vips_block_cache_build( VipsObject *object )
 	VipsConversion *conversion = VIPS_CONVERSION( object );
 	VipsBlockCache *cache = (VipsBlockCache *) object;
 
-	VIPS_DEBUG_MSG( "vips_block_cache_build\n" );
+	VIPS_DEBUG_MSG( "vips_block_cache_build:\n" );
 
 	if( VIPS_OBJECT_CLASS( vips_block_cache_parent_class )->
 		build( object ) )
 		return( -1 );
 
+	VIPS_DEBUG_MSG( "vips_block_cache_build: max size = %g MB\n",
+		(cache->max_tiles * cache->tile_width * cache->tile_height *
+		 	VIPS_IMAGE_SIZEOF_PEL( cache->in )) / (1024 * 1024.0) );
+
 	if( !cache->persistent )
 		g_signal_connect( conversion->out, "minimise", 
 			G_CALLBACK( vips_block_cache_minimise ), cache );
@@ -459,7 +464,7 @@ vips_tile_destroy( VipsTile *tile )
 {
 	VipsBlockCache *cache = tile->cache;
 
-	VIPS_DEBUG_MSG( "vips_tile_destroy: tile %d, %d (%p)\n", 
+	VIPS_DEBUG_MSG_RED( "vips_tile_destroy: tile %d, %d (%p)\n", 
 		tile->pos.left, tile->pos.top, tile ); 
 
 	cache->ntiles -= 1;
@@ -564,7 +569,7 @@ vips_tile_cache_ref( VipsBlockCache *cache, VipsRect *r )
 			 */
 			work = g_slist_append( work, tile );
 
-			VIPS_DEBUG_MSG( "vips_tile_cache_ref: "
+			VIPS_DEBUG_MSG_RED( "vips_tile_cache_ref: "
 				"tile %d, %d (%p)\n", x, y, tile ); 
 		}
 
@@ -598,9 +603,13 @@ vips_tile_cache_gen( VipsRegion *or,
 	GSList *work;
 	GSList *p;
 
+	VIPS_GATE_START( "vips_tile_cache_gen: wait1" );
+
 	g_mutex_lock( cache->lock );
 
-	VIPS_DEBUG_MSG( "vips_tile_cache_gen: "
+	VIPS_GATE_STOP( "vips_tile_cache_gen: wait1" );
+
+	VIPS_DEBUG_MSG_RED( "vips_tile_cache_gen: "
 		"left = %d, top = %d, width = %d, height = %d\n",
 		r->left, r->top, r->width, r->height );
 
@@ -622,8 +631,8 @@ vips_tile_cache_gen( VipsRegion *or,
 			if( !p )
 				break;
 
-			VIPS_DEBUG_MSG( "vips_tile_cache_gen: pasting %p\n", 
-				tile ); 
+			VIPS_DEBUG_MSG_RED( "vips_tile_cache_gen: "
+				"pasting %p\n", tile ); 
 
 			vips_tile_paste( tile, or );
 
@@ -646,7 +655,7 @@ vips_tile_cache_gen( VipsRegion *or,
 
 				tile->state = VIPS_TILE_STATE_CALC;
 
-				VIPS_DEBUG_MSG( "vips_tile_cache_gen: "
+				VIPS_DEBUG_MSG_RED( "vips_tile_cache_gen: "
 					"calc of %p\n", tile ); 
 
 				/* In threaded mode, we let other threads run
@@ -661,9 +670,16 @@ vips_tile_cache_gen( VipsRegion *or,
 					&tile->pos, 
 					tile->pos.left, tile->pos.top );
 
-				if( cache->threaded ) 
+				if( cache->threaded ) {
+					VIPS_GATE_START( "vips_tile_cache_gen: "
+						"wait2" );
+
 					g_mutex_lock( cache->lock );
 
+					VIPS_GATE_STOP( "vips_tile_cache_gen: "
+						"wait2" );
+				}
+
 				/* If there was an error calculating this
 				 * tile, just warn and carry on.
 				 *
@@ -672,7 +688,8 @@ vips_tile_cache_gen( VipsRegion *or,
 				 * read to fail because of one broken tile.
 				 */
 				if( result ) {
-					VIPS_DEBUG_MSG( "vips_tile_cache_gen: "
+					VIPS_DEBUG_MSG_RED( 
+						"vips_tile_cache_gen: "
 						"error on tile %p\n", tile ); 
 
 					vips_warn( class->nickname,
@@ -711,10 +728,14 @@ vips_tile_cache_gen( VipsRegion *or,
 				g_assert( tile->state == VIPS_TILE_STATE_CALC );
 			}
 
-			VIPS_DEBUG_MSG( "vips_tile_cache_gen: waiting\n" ); 
+			VIPS_DEBUG_MSG_RED( "vips_tile_cache_gen: waiting\n" ); 
+
+			VIPS_GATE_START( "vips_tile_cache_gen: wait3" );
 
 			g_cond_wait( cache->new_tile, cache->lock );
 
+			VIPS_GATE_STOP( "vips_tile_cache_gen: wait3" );
+
 			VIPS_DEBUG_MSG( "vips_tile_cache_gen: awake!\n" ); 
 		}
 	}
@@ -740,10 +761,9 @@ vips_tile_cache_build( VipsObject *object )
 	if( vips_image_pio_input( block_cache->in ) )
 		return( -1 );
 
-	if( vips_image_copy_fields( conversion->out, block_cache->in ) )
+	if( vips_image_pipelinev( conversion->out, 
+		VIPS_DEMAND_STYLE_SMALLTILE, block_cache->in, NULL ) )
 		return( -1 );
-        vips_demand_hint( conversion->out, 
-		VIPS_DEMAND_STYLE_SMALLTILE, block_cache->in, NULL );
 
 	if( vips_image_generate( conversion->out,
 		vips_start_one, vips_tile_cache_gen, vips_stop_one, 
@@ -862,8 +882,12 @@ vips_line_cache_gen( VipsRegion *or,
 {
 	VipsBlockCache *block_cache = (VipsBlockCache *) b;
 
+	VIPS_GATE_START( "vips_line_cache_gen: wait" );
+
 	g_mutex_lock( block_cache->lock );
 
+	VIPS_GATE_STOP( "vips_line_cache_gen: wait" );
+
 	/* We size up the cache to the largest request.
 	 */
 	if( or->valid.height > 
@@ -899,7 +923,12 @@ vips_line_cache_build( VipsObject *object )
 	block_cache->access = cache->access; 
 
 	if( cache->access == VIPS_ACCESS_SEQUENTIAL_UNBUFFERED )
-		block_cache->max_tiles = 1; 
+		/* A tile per thread. 
+		 *
+		 * Imagine scanline tiles and four threads. And add a bit for
+		 * slop. 
+		 */
+		block_cache->max_tiles = 2 * vips_concurrency_get();
 	else { 
 		/* Enough lines for two complete buffers would be exactly 
 		 * right. Make it 3 to give us some slop room. 
@@ -923,13 +952,18 @@ vips_line_cache_build( VipsObject *object )
 		"max_tiles = %d, tile_height = %d\n", 
 		block_cache->max_tiles, block_cache->tile_height ); 
 
+	VIPS_DEBUG_MSG( "vips_line_cache_build: max size = %g MB\n",
+		(block_cache->max_tiles * 
+		 block_cache->tile_width * 
+		 block_cache->tile_height * 
+		 VIPS_IMAGE_SIZEOF_PEL( block_cache->in )) / (1024 * 1024.0) );
+
 	if( vips_image_pio_input( block_cache->in ) )
 		return( -1 );
 
-	if( vips_image_copy_fields( conversion->out, block_cache->in ) )
+	if( vips_image_pipelinev( conversion->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, block_cache->in, NULL ) )
 		return( -1 );
-        vips_demand_hint( conversion->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, block_cache->in, NULL );
 
 	if( vips_image_generate( conversion->out,
 		vips_start_one, vips_line_cache_gen, vips_stop_one, 
