@@ -199,9 +199,14 @@ static int
 conv_evalstart( Conv *conv )
 {
 	/* Reset underflow/overflow count.
-	 */
+	 *
+	 * This often doesn't get called until eval has already finished, so
+	 * resetting here just wipes all records. 
+	 *
 	conv->overflow = 0;
 	conv->underflow = 0;
+	 *
+	 */
 
         return( 0 );
 }
@@ -209,12 +214,12 @@ conv_evalstart( Conv *conv )
 static int
 conv_evalend( Conv *conv )
 {
-	/* Print underflow/overflow count.
-	 */
-	if( conv->overflow || conv->underflow )
-		im_warn( "im_conv", 
-			_( "%d overflows and %d underflows detected" ),
-			conv->overflow, conv->underflow );
+	if( conv->overflow ) 
+		vips_info( "im_conv", 
+			_( "%d overflows detected" ), conv->overflow ); 
+	if( conv->underflow )
+		vips_info( "im_conv", 
+			_( "%d underflows detected" ), conv->underflow );
 
         return( 0 );
 }
@@ -705,6 +710,8 @@ conv_gen( REGION *or, void *vseq, void *a, void *b )
 	if( im_prepare( ir, &s ) )
 		return( -1 );
 
+	VIPS_GATE_START( "conv_gen: work" ); 
+
         /* Fill offset array. Only do this if the bpl has changed since the 
 	 * previous im_prepare().
 	 */
@@ -769,6 +776,8 @@ conv_gen( REGION *or, void *vseq, void *a, void *b )
 		}
 	}
 
+	VIPS_GATE_STOP( "conv_gen: work" ); 
+
 	return( 0 );
 }
 
@@ -875,6 +884,8 @@ conv3x3_gen( REGION *or, void *vseq, void *a, void *b )
 	if( im_prepare( ir, &s ) )
 		return( -1 );
 
+	VIPS_GATE_START( "conv3x3_gen: work" ); 
+
 	for( y = to; y < bo; y++ ) { 
 		switch( in->BandFmt ) {
 		case IM_BANDFMT_UCHAR: 	
@@ -922,6 +933,8 @@ conv3x3_gen( REGION *or, void *vseq, void *a, void *b )
 		}
 	}
 
+	VIPS_GATE_STOP( "conv3x3_gen: work" ); 
+
 	return( 0 );
 }
 
@@ -953,6 +966,8 @@ convvec_gen( REGION *or, void *vseq, void *a, void *b )
 	if( im_prepare( ir, &s ) )
 		return( -1 );
 
+	VIPS_GATE_START( "convvec_gen: work" ); 
+
 	for( j = 0; j < conv->n_pass; j++ ) 
 		vips_executor_set_program( &convolve[j], 
 			conv->pass[j].vector, sz );
@@ -1001,6 +1016,8 @@ convvec_gen( REGION *or, void *vseq, void *a, void *b )
 #endif /*DEBUG_PIXELS*/
 	}
 
+	VIPS_GATE_STOP( "convvec_gen: work" ); 
+
 	return( 0 );
 }
 
@@ -1062,10 +1079,7 @@ im_conv_raw( IMAGE *in, IMAGE *out, INTMASK *mask )
 #endif /*DEBUG*/
 	}
 
-	/* Set demand hints. FATSTRIP is good for us, as THINSTRIP will cause
-	 * too many recalculations on overlaps.
-	 */
-	if( im_demand_hint( out, IM_FATSTRIP, in, NULL ) ||
+	if( im_demand_hint( out, IM_SMALLTILE, in, NULL ) ||
 		im_generate( out, conv_start, generate, conv_stop, in, conv ) )
 		return( -1 );
 
@@ -1075,28 +1089,6 @@ im_conv_raw( IMAGE *in, IMAGE *out, INTMASK *mask )
 	return( 0 );
 }
 
-/**
- * im_conv:
- * @in: input image
- * @out: output image
- * @mask: convolution mask
- *
- * Convolve @in with @mask using integer arithmetic. The output image 
- * always has the same #VipsBandFmt as the input image. 
- *
- * Each output pixel is
- * calculated as sigma[i]{pixel[i] * mask[i]} / scale + offset, where scale
- * and offset are part of @mask. For integer @in, the division by scale
- * includes round-to-nearest.
- *
- * Convolutions on unsigned 8-bit images are calculated with the 
- * processor's vector unit,
- * if possible. Disable this with --vips-novector or IM_NOVECTOR.
- *
- * See also: im_conv_f(), im_convsep(), im_create_imaskv().
- *
- * Returns: 0 on success, -1 on error
- */
 int 
 im_conv( IMAGE *in, IMAGE *out, INTMASK *mask )
 {
@@ -1114,72 +1106,3 @@ im_conv( IMAGE *in, IMAGE *out, INTMASK *mask )
 
 	return( 0 );
 }
-
-int
-im_convsep_raw( IMAGE *in, IMAGE *out, INTMASK *mask )
-{
-	IMAGE *t;
-	INTMASK *rmask;
-
-	if( mask->xsize != 1 && mask->ysize != 1 ) {
-                im_error( "im_convsep", 
-			"%s", _( "expect 1xN or Nx1 input mask" ) );
-                return( -1 );
-	}
-
-	if( !(t = im_open_local( out, "im_convsep", "p" )) ||
-		!(rmask = (INTMASK *) im_local( out, 
-		(im_construct_fn) im_dup_imask,
-		(im_callback_fn) im_free_imask, mask, mask->filename, NULL )) )
-		return( -1 );
-
-	rmask->xsize = mask->ysize;
-	rmask->ysize = mask->xsize;
-        rmask->offset = 0.;
-
-	if( im_conv_raw( in, t, rmask ) ||
-		im_conv_raw( t, out, mask ) )
-		return( -1 );
-
-	return( 0 );
-}
-
-/**
- * im_convsep:
- * @in: input image
- * @out: output image
- * @mask: convolution mask
- *
- * Perform a separable convolution of @in with @mask using integer arithmetic. 
- * See im_conv() for a detailed description.
- *
- * The mask must be 1xn or nx1 elements. 
- * The output image 
- * always has the same #VipsBandFmt as the input image. 
- *
- * The image is convolved twice: once with @mask and then again with @mask 
- * rotated by 90 degrees. This is much faster for certain types of mask
- * (gaussian blur, for example) than doing a full 2D convolution.
- *
- * See also: im_convsep_f(), im_conv(), im_create_imaskv().
- *
- * Returns: 0 on success, -1 on error
- */
-int 
-im_convsep( IMAGE *in, IMAGE *out, INTMASK *mask )
-{
-	IMAGE *t1 = im_open_local( out, "im_convsep intermediate", "p" );
-	int n_mask = mask->xsize * mask->ysize;
-
-	if( !t1 || 
-		im_embed( in, t1, 1, n_mask / 2, n_mask / 2, 
-			in->Xsize + n_mask - 1, 
-			in->Ysize + n_mask - 1 ) ||
-		im_convsep_raw( t1, out, mask ) )
-		return( -1 );
-
-	out->Xoffset = 0;
-	out->Yoffset = 0;
-
-	return( 0 );
-}
