@@ -131,7 +131,7 @@ vips_gaussnoise_build( VipsObject *object )
 		gaussnoise->width, gaussnoise->height, 1,
 		VIPS_FORMAT_FLOAT, VIPS_CODING_NONE,
 		VIPS_INTERPRETATION_B_W, 1.0, 1.0 );
-	vips_demand_hint( create->out, 
+	vips_image_pipelinev( create->out, 
 		VIPS_DEMAND_STYLE_ANY, NULL );
 
 	if( vips_image_generate( create->out, 
