@@ -65,7 +65,8 @@ im_mask2vips( DOUBLEMASK *in, IMAGE *out )
 
 	/* Check the mask.
 	 */
-	if( !in || !in->coeff ) {
+	if( !in || 
+		!in->coeff ) {
 		im_error( "im_mask2vips", "%s", _( "bad input mask" ) );
 		return( -1 );
 	}
@@ -98,6 +99,23 @@ im_mask2vips( DOUBLEMASK *in, IMAGE *out )
 			return( -1 );
 	}
 
+	vips_image_set_double( out, "scale", in->scale ); 
+	vips_image_set_double( out, "offset", in->offset ); 
+
 	return( 0 );
 }
 
+int
+im_imask2vips( INTMASK *in, IMAGE *out )
+{
+	DOUBLEMASK *d;
+	int result;
+
+	if( !(d = im_imask2dmask( in, in->filename )) )
+		return( -1 );
+	result = im_mask2vips( d, out );
+	im_free_dmask( d );
+
+	return( result ); 
+}
+
