@@ -491,7 +491,7 @@ im_wrapmany( IMAGE **in, IMAGE *out, im_wrapmany_fn fn, void *a, void *b )
 		if( vips_image_pio_input( in[i] ) )
 			return( -1 );
 	}
-        vips_demand_hint_array( out, VIPS_DEMAND_STYLE_THINSTRIP, in );
+        vips__demand_hint_array( out, VIPS_DEMAND_STYLE_THINSTRIP, in );
 
 	/* Generate!
 	 */
@@ -964,7 +964,35 @@ im_demand_hint (IMAGE * im, VipsDemandStyle hint, ...)
       return (-1);
     }
 
-  return (im_demand_hint_array (im, hint, ar));
+  vips__demand_hint_array (im, hint, ar);
+
+  return (0);
+}
+
+int
+im_cp_descv (IMAGE * im, ...)
+{
+  va_list ap;
+  int i;
+  IMAGE *ar[MAX_IMAGES];
+
+  va_start (ap, im);
+  for (i = 0; i < MAX_IMAGES && (ar[i] = va_arg (ap, IMAGE *)); i++)
+    ;
+  va_end (ap);
+  if (i == MAX_IMAGES)
+    {
+      im_error ("im_cp_descv", "%s", _("too many images"));
+      return (-1);
+    }
+
+  return (vips__image_copy_fields_array (im, ar));
+}
+
+int
+im_cp_desc(IMAGE *out, IMAGE *in )
+{
+	return( im_cp_descv( out, in, NULL)); 
 }
 
 int 
@@ -1367,6 +1395,30 @@ im_gbandjoin( VipsImage **in, VipsImage *out, int n )
 	return( 0 );
 }
 
+int
+im_rank_image( VipsImage **in, VipsImage *out, int n, int index )
+{
+	VipsImage *t;
+
+	if( vips_bandrank( in, &t, n,
+		"index", index,
+		NULL ) )
+		return( -1 );
+	if( vips_image_write( t, out ) ) {
+		g_object_unref( t );
+		return( -1 );
+	}
+	g_object_unref( t );
+
+	return( 0 );
+}
+
+int
+im_maxvalue( IMAGE **in, IMAGE *out, int n )
+{
+	return( im_rank_image( in, out, n, n - 1 ) );
+}
+
 int 
 im_invert( IMAGE *in, IMAGE *out )
 {
@@ -1905,6 +1957,119 @@ im_stats( VipsImage *in )
 	return( msk );
 }
 
+DOUBLEMASK *
+im_gauss_dmask( const char *filename, double sigma, double min_ampl )
+{
+	VipsImage *t;
+	DOUBLEMASK *msk;
+
+	if( vips_gaussmat( &t, sigma, min_ampl,
+		NULL ) )
+		return( NULL );
+	if( !(msk = im_vips2mask( t, filename )) ) {
+		g_object_unref( t );
+		return( NULL );
+	}
+	g_object_unref( t );
+
+	return( msk );
+}
+
+DOUBLEMASK *
+im_gauss_dmask_sep( const char *filename, double sigma, double min_ampl )
+{
+	VipsImage *t;
+	DOUBLEMASK *msk;
+
+	if( vips_gaussmat( &t, sigma, min_ampl,
+		"separable", TRUE,
+		NULL ) )
+		return( NULL );
+	if( !(msk = im_vips2mask( t, filename )) ) {
+		g_object_unref( t );
+		return( NULL );
+	}
+	g_object_unref( t );
+
+	return( msk );
+}
+
+INTMASK *
+im_gauss_imask( const char *filename, double sigma, double min_ampl )
+{
+	VipsImage *t;
+	INTMASK *msk;
+
+	if( vips_gaussmat( &t, sigma, min_ampl,
+		"integer", TRUE,
+		NULL ) )
+		return( NULL );
+	if( !(msk = im_vips2imask( t, filename )) ) {
+		g_object_unref( t );
+		return( NULL );
+	}
+	g_object_unref( t );
+
+	return( msk );
+}
+
+INTMASK *
+im_gauss_imask_sep( const char *filename, double sigma, double min_ampl )
+{
+	VipsImage *t;
+	INTMASK *msk;
+
+	if( vips_gaussmat( &t, sigma, min_ampl,
+		"integer", TRUE,
+		"separable", TRUE,
+		NULL ) )
+		return( NULL );
+	if( !(msk = im_vips2imask( t, filename )) ) {
+		g_object_unref( t );
+		return( NULL );
+	}
+	g_object_unref( t );
+
+	return( msk );
+}
+
+INTMASK *
+im_log_imask( const char *filename, double sigma, double min_ampl )
+{
+	VipsImage *t;
+	INTMASK *msk;
+
+	if( vips_logmat( &t, sigma, min_ampl,
+		"integer", TRUE,
+		NULL ) )
+		return( NULL );
+	if( !(msk = im_vips2imask( t, filename )) ) {
+		g_object_unref( t );
+		return( NULL );
+	}
+	g_object_unref( t );
+
+	return( msk );
+}
+
+DOUBLEMASK *
+im_log_dmask( const char *filename, double sigma, double min_ampl )
+{
+	VipsImage *t;
+	DOUBLEMASK *msk;
+
+	if( vips_logmat( &t, sigma, min_ampl,
+		NULL ) )
+		return( NULL );
+	if( !(msk = im_vips2mask( t, filename )) ) {
+		g_object_unref( t );
+		return( NULL );
+	}
+	g_object_unref( t );
+
+	return( msk );
+}
+
 int 
 im_recomb( IMAGE *in, IMAGE *out, DOUBLEMASK *recomb )
 {
@@ -1928,6 +2093,270 @@ im_recomb( IMAGE *in, IMAGE *out, DOUBLEMASK *recomb )
 	return( 0 );
 }
 
+int 
+im_compass( VipsImage *in, VipsImage *out, INTMASK *mask )
+{
+	VipsImage *t1, *t2;
+
+	if( !(t1 = vips_image_new()) ||
+		im_imask2vips( mask, t1 ) )
+		return( -1 );
+	if( vips_compass( in, &t2, t1, 
+		"times", 8, 
+		NULL ) ) {
+		g_object_unref( t1 );
+		return( -1 );
+	}
+	g_object_unref( t1 );
+	if( vips_image_write( t2, out ) ) {
+		g_object_unref( t2 );
+		return( -1 );
+	}
+	g_object_unref( t2 );
+
+	return( 0 );
+}
+
+int 
+im_lindetect( IMAGE *in, IMAGE *out, INTMASK *mask )
+{
+	VipsImage *t1, *t2;
+
+	if( !(t1 = vips_image_new()) ||
+		im_imask2vips( mask, t1 ) )
+		return( -1 );
+	if( vips_compass( in, &t2, t1, 
+		"times", 4, 
+		NULL ) ) {
+		g_object_unref( t1 );
+		return( -1 );
+	}
+	g_object_unref( t1 );
+	if( vips_image_write( t2, out ) ) {
+		g_object_unref( t2 );
+		return( -1 );
+	}
+	g_object_unref( t2 );
+
+	return( 0 );
+}
+
+int
+im_gradient( IMAGE *in, IMAGE *out, INTMASK *mask )
+{
+	VipsImage *t1, *t2;
+
+	if( !(t1 = vips_image_new()) ||
+		im_imask2vips( mask, t1 ) )
+		return( -1 );
+	if( vips_compass( in, &t2, t1, 
+		"times", 2, 
+		"angle", VIPS_ANGLE45_90, 
+		"combine", VIPS_COMBINE_SUM, 
+		NULL ) ) {
+		g_object_unref( t1 );
+		return( -1 );
+	}
+	g_object_unref( t1 );
+	if( vips_image_write( t2, out ) ) {
+		g_object_unref( t2 );
+		return( -1 );
+	}
+	g_object_unref( t2 );
+
+	return( 0 );
+}
+
+int
+im_convsep_raw( IMAGE *in, IMAGE *out, INTMASK *mask )
+{
+	im_error( "im_convsep_raw", "no compat function" );
+	return( -1 );
+}
+
+int 
+im_convsep( IMAGE *in, IMAGE *out, INTMASK *mask )
+{
+	VipsImage *t1, *t2;
+
+	if( !(t1 = vips_image_new()) ||
+		im_imask2vips( mask, t1 ) )
+		return( -1 );
+	if( vips_convsep( in, &t2, t1, 
+		NULL ) ) {
+		g_object_unref( t1 );
+		return( -1 );
+	}
+	g_object_unref( t1 );
+	if( vips_image_write( t2, out ) ) {
+		g_object_unref( t2 );
+		return( -1 );
+	}
+	g_object_unref( t2 );
+
+	return( 0 );
+}
+
+int
+im_convsep_f_raw( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
+{
+	im_error( "im_convsep_raw", "no compat function" );
+	return( -1 );
+}
+
+int 
+im_convsep_f( IMAGE *in, IMAGE *out, DOUBLEMASK *mask )
+{
+	VipsImage *t1, *t2;
+
+	if( !(t1 = vips_image_new()) ||
+		im_mask2vips( mask, t1 ) )
+		return( -1 );
+	if( vips_convsep( in, &t2, t1, 
+		"precision", VIPS_PRECISION_FLOAT,
+		NULL ) ) {
+		g_object_unref( t1 );
+		return( -1 );
+	}
+	g_object_unref( t1 );
+	if( vips_image_write( t2, out ) ) {
+		g_object_unref( t2 );
+		return( -1 );
+	}
+	g_object_unref( t2 );
+
+	return( 0 );
+}
+
+int
+im_addgnoise( IMAGE *in, IMAGE *out, double sigma )
+{
+	IMAGE *t;
+
+	if( !(t = im_open_local( out, "im_addgnoise", "p" )) ||
+		im_gaussnoise( t, in->Xsize, in->Ysize, 0, sigma ) ||
+		im_add( in, t, out ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+int
+im_contrast_surface_raw( IMAGE *in, IMAGE *out, int half_win_size, int spacing )
+{
+	im_error( "im_contrast_surface_raw", "no compat function" );
+	return( -1 );
+}
+
+/* This replaces some custom code in 7.36 and earlier. The hand-made one was
+ * slower for spacing == 1, though faster for large values of spacing. 
+ *
+ * Not worth maintaining a special operator for. 
+ */
+int
+im_contrast_surface( IMAGE *in, IMAGE *out, int half_win_size, int spacing )
+{
+	VipsImage **t = (VipsImage **) 
+		vips_object_local_array( VIPS_OBJECT( out ), 10 );
+	int size = half_win_size * 2; 
+
+	int x, y;
+
+	t[0] = vips_image_new_matrixv( 1, 2, -1.0, 1.0 );
+	t[1] = vips_image_new_matrixv( 2, 1, -1.0, 1.0 );
+	t[8] = vips_image_new_matrix( size, size ); 
+
+	for( y = 0; y < size; y++ )
+		for( x = 0; x < size; x++ )
+			*VIPS_MATRIX( t[8], x, y ) = 1.0;
+
+	if( vips_conv( in, &t[2], t[0], NULL ) ||
+		vips_conv( in, &t[3], t[1], NULL ) ||
+		vips_abs( t[2], &t[4], NULL ) ||
+		vips_abs( t[3], &t[5], NULL ) ||
+		vips_add( t[4], t[5], &t[6], NULL ) ||
+		vips_conv( t[6], &t[7], t[8], NULL ) ||
+		vips_subsample( t[7], &t[9], spacing, spacing, NULL ) ||
+		vips_image_write( t[9], out ) )
+		return( -1 ); 
+
+	return( 0 );
+}
+
+int
+im_spcor_raw( IMAGE *in, IMAGE *ref, IMAGE *out )
+{
+	im_error( "im_spcor_raw", "no compat function" );
+	return( -1 );
+}
+
+int
+im_spcor( IMAGE *in, IMAGE *ref, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_call( "spcor", in, ref, &x, NULL ) )
+		return( -1 );
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_fastcor_raw( IMAGE *in, IMAGE *ref, IMAGE *out )
+{
+	im_error( "im_fastcor_raw", "no compat function" );
+	return( -1 );
+}
+
+int
+im_fastcor( IMAGE *in, IMAGE *ref, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_call( "fastcor", in, ref, &x, NULL ) )
+		return( -1 );
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_sharpen( IMAGE *in, IMAGE *out, 
+	int mask_size, 
+	double x1, double y2, double y3, 
+	double m1, double m2 )
+{
+	VipsImage **t = (VipsImage **) 
+		vips_object_local_array( VIPS_OBJECT( out ), 2 );
+
+	/* im_sharpen() always recoded as labq and im_benchmark() depends
+	 * upon this behaviour. 
+	 */
+	if( vips_call( "sharpen", in, &t[0], 
+		"radius", mask_size / 2,
+		"x1", x1,
+		"y2", y2,
+		"y3", y3,
+		"m1", m1,
+		"m2", m2,
+		NULL ) ||
+		vips_colourspace( t[0], &t[1], 
+			VIPS_INTERPRETATION_LABQ, NULL ) ||
+		vips_image_write( t[1], out ) ) 
+		return( -1 );
+
+	return( 0 );
+}
+
 static int
 vips__round( VipsImage *in, VipsImage *out, VipsOperationRound round )
 {
@@ -3754,8 +4183,8 @@ im_gammacorrect( IMAGE *in, IMAGE *out, double exponent )
 {
 	VipsImage *x;
 
-	if( vips_gammacorrect( in, &x, 
-		"exponent", exponent,
+	if( vips_gamma( in, &x, 
+		"exponent", 1.0 / exponent,
 		NULL ) )
 		return( -1 );
 
@@ -3812,14 +4241,22 @@ static int
 im__affinei( VipsImage *in, VipsImage *out, 
 	VipsInterpolate *interpolate, VipsTransformation *trn )
 {
-	VipsImage *x;
+	VipsImage **t = (VipsImage **) 
+		vips_object_local_array( VIPS_OBJECT( out ), 2 );
+
 	VipsArea *oarea;
+	gboolean repack;
 
 	oarea = (VipsArea *) vips_array_int_newv( 4, 
 		trn->oarea.left, trn->oarea.top,
 		trn->oarea.width, trn->oarea.height );
 
-	if( vips_affine( in, &x, 
+	/* vips7 affine would repack labq and im_benchmark() depends upon
+	 * this.
+	 */
+	repack = in->Coding == IM_CODING_LABQ;
+
+	if( vips_affine( in, &t[0], 
 		trn->a, trn->b, trn->c, trn->d,
 		"interpolate", interpolate,
 		"oarea", oarea,
@@ -3829,15 +4266,18 @@ im__affinei( VipsImage *in, VipsImage *out,
 		vips_area_unref( oarea );
 		return( -1 );
 	}
-
 	vips_area_unref( oarea );
+	in = t[0];
 
-	if( im_copy( x, out ) ) {
-		g_object_unref( x );
-		return( -1 );
+	if( repack ) {
+		if( vips_colourspace( in, &t[1], 
+			VIPS_INTERPRETATION_LABQ, NULL ) )
+			return( -1 );
+		in = t[1];
 	}
 
-	g_object_unref( x );
+	if( vips_image_write( in, out ) ) 
+		return( -1 );
 
 	return( 0 );
 }
@@ -3919,3 +4359,165 @@ im_copy_file( IMAGE *in, IMAGE *out )
 
 	return( 0 );
 }
+
+int
+im_video_v4l1( IMAGE *im, const char *device,
+	int channel, int brightness, int colour, int contrast, int hue, 
+	int ngrabs )
+{
+	im_error( "im_video_v4l1", 
+		"%s", _( "compiled without im_video_v4l1 support" ) );
+	return( -1 );
+}
+
+int 
+im_fwfft( IMAGE *in, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_fwfft( in, &x, NULL ) )
+		return( -1 );
+
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int 
+im_invfft( IMAGE *in, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_invfft( in, &x, NULL ) )
+		return( -1 );
+
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int 
+im_invfftr( IMAGE *in, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_invfft( in, &x, 
+		"real", TRUE,
+		NULL ) )
+		return( -1 );
+
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int 
+im_freqflt( IMAGE *in, IMAGE *mask, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_freqmult( in, mask, &x, NULL ) )
+		return( -1 );
+
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int 
+im_disp_ps( IMAGE *in, IMAGE *out )
+{
+	VipsImage *t;
+
+	if( vips_spectrum( in, &t, NULL ) )
+		return( -1 );
+	if( vips_image_write( t, out ) ) {
+		g_object_unref( t );
+		return( -1 );
+	}
+	g_object_unref( t );
+
+	return( 0 );
+}
+
+int 
+im_fractsurf( IMAGE *out, int size, double frd )
+{
+	VipsImage *t;
+
+	if( vips_fractsurf( &t, size, size, frd, NULL ) )
+		return( -1 );
+	if( vips_image_write( t, out ) ) {
+		g_object_unref( t );
+		return( -1 );
+	}
+	g_object_unref( t );
+
+	return( 0 );
+}
+
+int 
+im_phasecor_fft( IMAGE *in1, IMAGE *in2, IMAGE *out )
+{
+	VipsImage *x;
+
+	if( vips_phasecor( in1, in2, &x, NULL ) )
+		return( -1 );
+
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int 
+im_cntlines( VipsImage *im, double *nolines, int flag )
+{
+	return( vips_countlines( im, nolines, 
+		flag == 0 ? 
+			VIPS_DIRECTION_HORIZONTAL : VIPS_DIRECTION_VERTICAL,
+		NULL ) );
+}
+
+int 
+im_rank( IMAGE *in, IMAGE *out, int width, int height, int index )
+{
+	VipsImage *x;
+
+	if( vips_rank( in, &x, width, height, index, NULL ) )
+		return( -1 );
+
+	if( im_copy( x, out ) ) {
+		g_object_unref( x );
+		return( -1 );
+	}
+	g_object_unref( x );
+
+	return( 0 );
+}
+
+int
+im_rank_raw( IMAGE *in, IMAGE *out, int width, int height, int index )
+{
+	im_error( "im_rank_raw", "no compat function" );
+	return( -1 );
+}
