@@ -20,7 +20,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -316,13 +317,13 @@ vips_wrap7_object_set_property( GObject *gobject,
 		vips__argument_table_lookup( oclass->argument_table, pspec );
 	VipsArgumentInstance *argument_instance =
 		vips__argument_get_instance( argument_class, object );
-
 	VipsWrap7 *wrap7 = VIPS_WRAP7( gobject );
 	VipsWrap7Class *class = VIPS_WRAP7_GET_CLASS( wrap7 );
-	int i = argument_class->offset;
-	im_arg_desc *arg = &class->fn->argv[i];
-	im_type_desc *type = arg->desc;
-	im_arg_type vt = type->type;
+
+	int i;
+	im_arg_desc *arg;
+	im_type_desc *type;
+	im_arg_type vt;
 
 	g_assert( argument_instance );
 
@@ -332,6 +333,11 @@ vips_wrap7_object_set_property( GObject *gobject,
 		return;
 	}
 
+	i = argument_class->offset;
+	arg = &class->fn->argv[i];
+	type = arg->desc;
+	vt = type->type;
+
 #ifdef DEBUG
 {
 	char *str_value;
@@ -417,13 +423,13 @@ vips_wrap7_object_get_property( GObject *gobject,
 		vips__argument_table_lookup( class->argument_table, pspec );
 	VipsArgumentInstance *argument_instance =
 		vips__argument_get_instance( argument_class, object );
-
 	VipsWrap7 *wrap7 = VIPS_WRAP7( gobject );
 	VipsWrap7Class *wclass = VIPS_WRAP7_GET_CLASS( wrap7 );
-	int i = argument_class->offset;
-	im_arg_desc *arg = &wclass->fn->argv[i];
-	im_type_desc *type = arg->desc;
-	im_arg_type vt = type->type;
+
+	int i;
+	im_arg_desc *arg;
+	im_type_desc *type;
+	im_arg_type vt;
 
 	if( !argument_class ) {
 		G_OBJECT_WARN_INVALID_PROPERTY_ID( gobject,
@@ -431,6 +437,11 @@ vips_wrap7_object_get_property( GObject *gobject,
 		return;
 	}
 
+	i = argument_class->offset;
+	arg = &wclass->fn->argv[i];
+	type = arg->desc;
+	vt = type->type;
+
 	g_assert( ((VipsArgument *) argument_class)->pspec == pspec );
 
 	if( !argument_instance->assigned ) {
@@ -626,18 +637,12 @@ vips_wrap7_summary_class( VipsObjectClass *oclass, VipsBuf *buf )
 	im_function *fn = class->fn;
 
 	if( fn )
-		vips_buf_appendf( buf, "%s ", fn->name );
+		vips_buf_appends( buf, fn->name );
 	else
-		vips_buf_appendf( buf, "%s ", G_OBJECT_CLASS_NAME( class ) );
+		vips_buf_appends( buf, G_OBJECT_CLASS_NAME( class ) );
 
-	if( oclass->nickname )
-		vips_buf_appendf( buf, "(%s), ", oclass->nickname );
 	if( oclass->description )
-		vips_buf_appendf( buf, "%s", oclass->description );
-
-	if( fn )
-		vips_buf_appendf( buf, ", from package \"%s\"", 
-			im_package_of_function( fn->name )->name );
+		vips_buf_appendf( buf, " - %s", oclass->description );
 }
 
 static void
@@ -776,6 +781,10 @@ vips_wrap7_subclass_class_init( VipsWrap7Class *class )
 
 		default:
 			g_assert( 0 );
+
+			/* Keep -Wall happy.
+			 */
+			return;
 		}
 
 		if( pspec ) {
