@@ -36,9 +36,6 @@
  * 	- add --angle option
  * 19/6/13
  * 	- faster --centre logic, thanks Kacey
- * 25/6/13
- * 	- ping classes beore starting workers, see comment below, thanks
- * 	  Kacey
  */
 
 /*
@@ -263,7 +260,8 @@ pyramid_build( VipsForeignSaveDz *dz, Layer *above,
 	 * easy.
 	 */
 	layer->image = vips_image_new();
-	if( vips_image_copy_fields( layer->image, save->ready ) ) {
+	if( vips_image_pipelinev( layer->image, 
+		VIPS_DEMAND_STYLE_ANY, save->ready, NULL ) ) {
 		layer_free( layer );
 		return( NULL );
 	}
@@ -363,12 +361,12 @@ pyramid_build( VipsForeignSaveDz *dz, Layer *above,
 static int
 pyramid_mkdir( VipsForeignSaveDz *dz )
 {
-	char buf[PATH_MAX];
+	char buf[VIPS_PATH_MAX];
 
 	if( dz->layout == VIPS_FOREIGN_DZ_LAYOUT_DZ )
-		vips_snprintf( buf, PATH_MAX, "%s_files", dz->basename );
+		vips_snprintf( buf, VIPS_PATH_MAX, "%s_files", dz->basename );
 	else
-		vips_snprintf( buf, PATH_MAX, "%s", dz->basename );
+		vips_snprintf( buf, VIPS_PATH_MAX, "%s", dz->basename );
 
 	if( vips_existsf( "%s", buf ) ) { 
 		vips_error( "dzsave", 
@@ -385,14 +383,14 @@ static int
 write_dzi( VipsForeignSaveDz *dz )
 {
 	FILE *fp;
-	char buf[PATH_MAX];
+	char buf[VIPS_PATH_MAX];
 	char *p;
 
-	vips_snprintf( buf, PATH_MAX, "%s.dzi", dz->basename );
+	vips_snprintf( buf, VIPS_PATH_MAX, "%s.dzi", dz->basename );
 	if( !(fp = vips__file_open_write( buf, TRUE )) )
 		return( -1 );
 
-	vips_snprintf( buf, PATH_MAX, "%s", dz->suffix + 1 );
+	vips_snprintf( buf, VIPS_PATH_MAX, "%s", dz->suffix + 1 );
 	if( (p = (char *) vips__find_rightmost_brackets( buf )) )
 		*p = '\0';
 
@@ -418,9 +416,10 @@ static int
 write_properties( VipsForeignSaveDz *dz )
 {
 	FILE *fp;
-	char buf[PATH_MAX];
+	char buf[VIPS_PATH_MAX];
 
-	vips_snprintf( buf, PATH_MAX, "%s/ImageProperties.xml", dz->basename );
+	vips_snprintf( buf, VIPS_PATH_MAX, 
+		"%s/ImageProperties.xml", dz->basename );
 	if( !(fp = vips__file_open_write( buf, TRUE )) )
 		return( -1 );
 
@@ -440,14 +439,14 @@ write_properties( VipsForeignSaveDz *dz )
 static int
 write_blank( VipsForeignSaveDz *dz )
 {
-	char buf[PATH_MAX];
+	char buf[VIPS_PATH_MAX];
 	VipsImage *x, *t;
 	int n;
 	VipsArea *ones;
 	double *d;
 	int i;
 
-	vips_snprintf( buf, PATH_MAX, "%s/blank.png", dz->basename );
+	vips_snprintf( buf, VIPS_PATH_MAX, "%s/blank.png", dz->basename );
 	if( vips_black( &x, dz->tile_size, dz->tile_size, NULL ) ) 
 		return( -1 );
 
@@ -710,8 +709,8 @@ tile_name( Layer *layer, char *buf, int x, int y )
 
 	VipsForeignSaveDz *dz = layer->dz;
 
-	char dirname[PATH_MAX];
-	char dirname2[PATH_MAX];
+	char dirname[VIPS_PATH_MAX];
+	char dirname2[VIPS_PATH_MAX];
 	Layer *p;
 	int n;
 
@@ -721,10 +720,10 @@ tile_name( Layer *layer, char *buf, int x, int y )
 
 	switch( dz->layout ) {
 	case VIPS_FOREIGN_DZ_LAYOUT_DZ:
-		vips_snprintf( dirname, PATH_MAX, "%s_files/%d", 
-			dz->basename, layer->n );
-		vips_snprintf( buf, PATH_MAX, "%s/%d_%d%s", 
-			dirname, x, y, dz->suffix );
+		vips_snprintf( dirname, VIPS_PATH_MAX, 
+			"%s_files/%d", dz->basename, layer->n );
+		vips_snprintf( buf, VIPS_PATH_MAX, 
+			"%s/%d_%d%s", dirname, x, y, dz->suffix );
 
 		if( !vips_existsf( "%s", dirname ) &&
 			vips_mkdirf( "%s", dirname ) ) {
@@ -751,10 +750,10 @@ tile_name( Layer *layer, char *buf, int x, int y )
 		 */
 		n += y * layer->tiles_across + x;
 
-		vips_snprintf( dirname, PATH_MAX, "%s/TileGroup%d", 
-			dz->basename, n / 256 );
-		vips_snprintf( buf, PATH_MAX, "%s/%d-%d-%d%s", 
-			dirname, layer->n, x, y, dz->suffix );
+		vips_snprintf( dirname, VIPS_PATH_MAX, 
+			"%s/TileGroup%d", dz->basename, n / 256 );
+		vips_snprintf( buf, VIPS_PATH_MAX, 
+			"%s/%d-%d-%d%s", dirname, layer->n, x, y, dz->suffix );
 
 		/* Used at the end in ImageProperties.xml
 		 */
@@ -769,12 +768,12 @@ tile_name( Layer *layer, char *buf, int x, int y )
 		break;
 
 	case VIPS_FOREIGN_DZ_LAYOUT_GOOGLE:
-		vips_snprintf( dirname, PATH_MAX, "%s/%d", 
-			dz->basename, layer->n );
-		vips_snprintf( dirname2, PATH_MAX, "%s/%d", 
-			dirname, y );
-		vips_snprintf( buf, PATH_MAX, "%s/%d%s", 
-			dirname2, x, dz->suffix );
+		vips_snprintf( dirname, VIPS_PATH_MAX, 
+			"%s/%d", dz->basename, layer->n );
+		vips_snprintf( dirname2, VIPS_PATH_MAX, 
+			"%s/%d", dirname, y );
+		vips_snprintf( buf, VIPS_PATH_MAX, 
+			"%s/%d%s", dirname2, x, dz->suffix );
 
 		if( !vips_existsf( "%s", dirname ) &&
 			vips_mkdirf( "%s", dirname ) ) {
@@ -808,7 +807,7 @@ strip_work( VipsThreadState *state, void *a )
 	Layer *layer = strip->layer;
 	VipsForeignSaveDz *dz = layer->dz;
 
-	char buf[PATH_MAX];
+	char buf[VIPS_PATH_MAX];
 	VipsImage *x;
 	VipsImage *t;
 
@@ -1168,37 +1167,6 @@ pyramid_strip( VipsRegion *region, VipsRect *area, void *a )
 	return( 0 );
 }
 
-static void *
-vips_class_ping( VipsObjectClass *class, void *dummy )
-{
-        return( NULL );
-}
-
-/* Loop over all classes. This will make sure they are all built.
- *
- * vips_dzsave() runs a set of pipelines from worker threads, and if the
- * operations it uses have not all been used previously, they can run their
- * class_init in parallel.
- *
- * This should be safe, but seems not to be for reasons I don't understand.
- * For now, just ping all classes from the main thread before we set the
- * workers going. 
- *
- * This stops a crash on very many core machines, see
- *
- * https://github.com/jcupitt/libvips/issues/64
- */
-static void
-vips_class_ping_all( void )
-{
-	GType base;
-
-	if( !(base = g_type_from_name( "VipsObject" )) )
-		return;
-	vips_class_map_all( base, 
-		(VipsClassMapFn) vips_class_ping, NULL );
-}
-
 static int
 vips_foreign_save_dz_build( VipsObject *object )
 {
@@ -1326,10 +1294,6 @@ vips_foreign_save_dz_build( VipsObject *object )
 	if( pyramid_mkdir( dz ) )
 		return( -1 );
 
-	/* Build all vips classes, see comment above. 
-	 */
-	vips_class_ping_all();
-
 	if( vips_sink_disc( save->ready, pyramid_strip, dz ) )
 		return( -1 );
 
@@ -1433,7 +1397,7 @@ vips_foreign_save_dz_class_init( VipsForeignSaveDzClass *class )
 		_( "Tile size in pixels" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSaveDz, tile_size ),
-		1, 1024, 256 );
+		1, 8192, 256 );
 
 	VIPS_ARG_BOXED( class, "background", 12, 
 		_( "Background" ), 
@@ -1486,14 +1450,14 @@ vips_foreign_save_dz_class_init( VipsForeignSaveDzClass *class )
 		_( "Tile width in pixels" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignSaveDz, tile_size ),
-		1, 1024, 256 );
+		1, 8192, 256 );
 
 	VIPS_ARG_INT( class, "tile_height", 12, 
 		_( "Tile height" ), 
 		_( "Tile height in pixels" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
 		G_STRUCT_OFFSET( VipsForeignSaveDz, tile_size ),
-		1, 1024, 256 );
+		1, 8192, 256 );
 
 }
 
