@@ -225,7 +225,6 @@ vips_fits_get_header( VipsFits *fits, VipsImage *out )
 		VIPS_DEBUG_MSG( "%d) %lld\n", i, fits->naxes[i] );
 #endif /*VIPS_DEBUG*/
 
-	width = 1;
 	height = 1;
 	bands = 1;
 	switch( fits->naxis ) {
@@ -296,11 +295,11 @@ vips_fits_get_header( VipsFits *fits, VipsImage *out )
 	else
 		type = VIPS_INTERPRETATION_MULTIBAND;
 
+	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_SMALLTILE, NULL );
 	vips_image_init_fields( out,
 		 width, height, bands,
 		 format,
 		 VIPS_CODING_NONE, type, 1.0, 1.0 );
-	vips_demand_hint( out, VIPS_DEMAND_STYLE_SMALLTILE, NULL );
 
 	/* Read all keys into meta.
 	 */
