@@ -6,6 +6,8 @@
  * 	- flatten alpha with vips_flatten()
  * 28/5/13
  * 	- auto rshift down to 8 bits during save
+ * 19/1/14
+ * 	- pack and unpack rad to scrgb
  */
 
 /*
@@ -772,7 +774,7 @@ vips_foreign_load_start( VipsImage *out, void *a, void *b )
 		/* We have to tell vips that out depends on real. We've set
 		 * the demand hint below, but not given an input there.
 		 */
-		vips_demand_hint( load->out, load->out->dhint, 
+		vips_image_pipelinev( load->out, load->out->dhint, 
 			load->real, NULL );
 	}
 
@@ -877,7 +879,7 @@ vips_foreign_load_build( VipsObject *object )
 		/* ->header() should set the dhint. It'll default to the safe
 		 * SMALLTILE if header() did not set it.
 		 */
-		vips_demand_hint( load->out, load->out->dhint, NULL );
+		vips_image_pipelinev( load->out, load->out->dhint, NULL );
 
 		/* Then 'start' creates the real image and 'gen' fetches 
 		 * pixels for @out from @real on demand.
@@ -1149,8 +1151,8 @@ vips_foreign_convert_saveable( VipsForeignSave *save )
 		in = out;
 	}
 
-	/* If this is an VIPS_CODING_RAD, we go to float RGB or XYZ. We should
-	 * probably un-gamma-correct the RGB :(
+	/* If this is an VIPS_CODING_RAD, we unpack to float. This could be
+	 * scRGB or XYZ. 
 	 */
 	if( in->Coding == VIPS_CODING_RAD ) {
 		VipsImage *out;
@@ -1248,10 +1250,27 @@ vips_foreign_convert_saveable( VipsForeignSave *save )
 		 */
 	}
 
-	/* Interpret the Type field for colorimetric images.
+	/* If the saver supports RAD, we need to go to scRGB or XYZ. 
 	 */
-	if( in->Bands == 3 &&
+	if( class->coding[VIPS_CODING_RAD] ) {
+		if( in->Type != VIPS_INTERPRETATION_scRGB &&
+			in->Type != VIPS_INTERPRETATION_XYZ ) {
+			VipsImage *out;
+
+			if( vips_colourspace( in, &out, 
+				VIPS_INTERPRETATION_scRGB, NULL ) ) {
+				g_object_unref( in );
+				return( -1 );
+			}
+			g_object_unref( in );
+
+			in = out;
+		}
+	}
+	else if( in->Bands == 3 &&
 		vips_colourspace_issupported( in ) ) {
+		/* Interpret the Type field for colorimetric images.
+		 */
 		VipsImage *out;
 
 		if( vips_colourspace( in, &out, 
@@ -1307,10 +1326,10 @@ vips_foreign_convert_saveable( VipsForeignSave *save )
 		/* Already NONE, nothing to do.
 		 */
 	}
-	else if( class->coding[VIPS_CODING_RAD] ) {
+	else if( class->coding[VIPS_CODING_LABQ] ) {
 		VipsImage *out;
 
-		if( vips_float2rad( in, &out, NULL ) ) {
+		if( vips_Lab2LabQ( in, &out, NULL ) ) {
 			g_object_unref( in );
 			return( -1 );
 		}
@@ -1318,10 +1337,10 @@ vips_foreign_convert_saveable( VipsForeignSave *save )
 
 		in = out;
 	}
-	else if( class->coding[VIPS_CODING_LABQ] ) {
+	else if( class->coding[VIPS_CODING_RAD] ) {
 		VipsImage *out;
 
-		if( vips_Lab2LabQ( in, &out, NULL ) ) {
+		if( vips_float2rad( in, &out, NULL ) ) {
 			g_object_unref( in );
 			return( -1 );
 		}
@@ -1389,7 +1408,7 @@ vips_foreign_save_class_init( VipsForeignSaveClass *class )
 
 	/* I think all savers are sequential. Hopefully.
 	 */
-	operation_class->flags |= VIPS_OPERATION_SEQUENTIAL;
+	operation_class->flags |= VIPS_OPERATION_SEQUENTIAL_UNBUFFERED;
 
 	/* Must not cache savers.
 	 */
@@ -1410,6 +1429,13 @@ vips_foreign_save_class_init( VipsForeignSaveClass *class )
 		_( "Image to save" ),
 		VIPS_ARGUMENT_REQUIRED_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSave, in ) );
+
+	VIPS_ARG_BOOL( class, "strip", 100,
+		_( "Strip" ),
+		_( "Strip all metadata from image" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSave, strip ),
+		FALSE );
 }
 
 static void
@@ -1614,6 +1640,7 @@ vips_foreign_operation_init( void )
 	extern GType vips_foreign_save_csv_get_type( void ); 
 	extern GType vips_foreign_load_matrix_get_type( void ); 
 	extern GType vips_foreign_save_matrix_get_type( void ); 
+	extern GType vips_foreign_print_matrix_get_type( void ); 
 	extern GType vips_foreign_load_fits_get_type( void ); 
 	extern GType vips_foreign_save_fits_get_type( void ); 
 	extern GType vips_foreign_load_analyze_get_type( void ); 
@@ -1646,6 +1673,7 @@ vips_foreign_operation_init( void )
 	vips_foreign_save_csv_get_type(); 
 	vips_foreign_load_matrix_get_type(); 
 	vips_foreign_save_matrix_get_type(); 
+	vips_foreign_print_matrix_get_type(); 
 	vips_foreign_load_analyze_get_type(); 
 	vips_foreign_load_raw_get_type(); 
 	vips_foreign_save_raw_get_type(); 
@@ -1991,6 +2019,9 @@ vips_jpegload( const char *filename, VipsImage **out, ... )
  * @Q: quality factor
  * @profile: attach this ICC profile
  * @optimize_coding: compute optimal Huffman coding tables
+ * @interlace: write an interlaced (progressive) jpeg
+ * @strip: remove all metadata from image
+ * @no-subsample: disable chroma subsampling
  *
  * Write a VIPS image to a file as JPEG.
  *
@@ -2015,6 +2046,19 @@ vips_jpegload( const char *filename, VipsImage **out, ... )
  * IPCT as @VIPS_META_IPCT_NAME ("ipct-data") and XMP as VIPS_META_XMP_NAME
  * ("xmp-data") are coded and attached. 
  *
+ * If @optimize_coding is set, the Huffman tables are optimised. This is
+ * sllightly slower and produces slightly smaller files. 
+ *
+ * If @interlace is set, the jpeg files will be interlaced (progressive jpeg,
+ * in jpg parlance). These files may be better for display over a slow network
+ * conection, but need much more memory to encode and decode. 
+ *
+ * If @strip is set, no EXIF data, IPCT data, ICC profile or XMP metadata is 
+ * written into the output file. 
+ *
+ * If @no-subsample is set, chrominance subsampling is disabled. This will 
+ * improve quality at the cost of larger file size. Useful for high Q factors. 
+ *
  * See also: vips_jpegsave_buffer(), vips_image_write_file().
  *
  * Returns: 0 on success, -1 on error.
@@ -2044,6 +2088,9 @@ vips_jpegsave( VipsImage *in, const char *filename, ... )
  * @Q: JPEG quality factor
  * @profile: attach this ICC profile
  * @optimize_coding: compute optimal Huffman coding tables
+ * @interlace: write an interlaced (progressive) jpeg
+ * @strip: remove all metadata from image
+ * @no-subsample: disable chroma subsampling
  *
  * As vips_jpegsave(), but save to a memory buffer. 
  *
@@ -2093,6 +2140,8 @@ vips_jpegsave_buffer( VipsImage *in, void **buf, size_t *len, ... )
  * @Q: JPEG quality factor
  * @profile: attach this ICC profile
  * @optimize_coding: compute optimal Huffman coding tables
+ * @strip: remove all metadata from image
+ * @no-subsample: disable chroma subsampling
  *
  * As vips_jpegsave(), but save as a mime jpeg on stdout.
  *
