@@ -55,6 +55,8 @@
  * 	- attach IPCT data (app13), thanks Gary
  * 6/7/13
  * 	- null-terminate exif strings, thanks Mike
+ * 24/2/14
+ * 	- don't write to our input buffer, thanks Lovell
  */
 
 /*
@@ -150,6 +152,10 @@ typedef struct _ReadJpeg {
 	/* Set if we need to finish the decompress.
 	 */
 	gboolean decompressing;
+
+	/* Track the y pos during a read with this.
+	 */
+	int y_pos;
 } ReadJpeg;
 
 static int
@@ -222,6 +228,8 @@ readjpeg_new( VipsImage *out, int shrink, gboolean fail, gboolean readbehind )
 	jpeg->eman.pub.output_message = vips__new_output_message;
 	jpeg->eman.fp = NULL;
 
+	jpeg->y_pos = 0;
+
 	/* jpeg_create_decompress() can fail on some sanity checks. Don't
 	 * readjpeg_free() since we don't want to jpeg_destroy_decompress().
 	 */
@@ -764,7 +772,7 @@ read_jpeg_header( ReadJpeg *jpeg, VipsImage *out )
 		interpretation,
 		xres, yres );
 
-	vips_demand_hint( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
+	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_FATSTRIP, NULL );
 
 	/* Interlaced jpegs need lots of memory to read, so our caller needs
 	 * to know.
@@ -885,10 +893,12 @@ read_jpeg_generate( VipsRegion *or,
 	int y;
 
 #ifdef DEBUG
-	printf( "read_jpeg_generate: line %d, %d rows\n", 
-		r->top, r->height );
+	printf( "read_jpeg_generate: %p line %d, %d rows\n", 
+		g_thread_self(), r->top, r->height );
 #endif /*DEBUG*/
 
+	VIPS_GATE_START( "read_jpeg_generate: work" );
+
 	/* We're inside a tilecache where tiles are the full image width, so
 	 * this should always be true.
 	 */
@@ -905,6 +915,11 @@ read_jpeg_generate( VipsRegion *or,
 	 */
 	g_assert( r->height == VIPS_MIN( 8, or->im->Ysize - r->top ) ); 
 
+	/* And check that y_pos is correct. It should be, since we are inside
+	 * a vips_sequential().
+	 */
+	g_assert( r->top == jpeg->y_pos ); 
+
 	/* Here for longjmp() from vips__new_error_exit().
 	 */
 	if( setjmp( jpeg->eman.jmp ) ) 
@@ -924,8 +939,12 @@ read_jpeg_generate( VipsRegion *or,
 			for( x = 0; x < sz; x++ )
 				row_pointer[0][x] = 255 - row_pointer[0][x];
 		}
+
+		jpeg->y_pos += 1; 
 	}
 
+	VIPS_GATE_STOP( "read_jpeg_generate: work" );
+
 	return( 0 );
 }
 
@@ -1087,24 +1106,23 @@ init_source (j_decompress_ptr cinfo)
 static boolean
 fill_input_buffer (j_decompress_ptr cinfo)
 {
+  static const JOCTET eoi_buffer[4] = {
+    (JOCTET) 0xFF, (JOCTET) JPEG_EOI, 0, 0
+  };
+
   InputBuffer *src = (InputBuffer *) cinfo->src;
-  size_t nbytes;
 
   if (src->start_of_file) {
-    nbytes = src->len;
+    src->pub.next_input_byte = src->buf;
+    src->pub.bytes_in_buffer = src->len;
+    src->start_of_file = FALSE;
   }
   else {
     WARNMS(cinfo, JWRN_JPEG_EOF);
-    /* Insert a fake EOI marker */
-    src->buf[0] = (JOCTET) 0xFF;
-    src->buf[1] = (JOCTET) JPEG_EOI;
-    nbytes = 2;
+    src->pub.next_input_byte = eoi_buffer;
+    src->pub.bytes_in_buffer = 2;
   }
 
-  src->pub.next_input_byte = src->buf;
-  src->pub.bytes_in_buffer = nbytes;
-  src->start_of_file = FALSE;
-
   return TRUE;
 }
 
