@@ -20,7 +20,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -72,9 +73,9 @@ vips_foreign_load_png_get_flags_filename( const char *filename )
 
 	flags = 0;
 	if( vips__png_isinterlaced( filename ) )
-		flags = VIPS_FOREIGN_PARTIAL;
+		flags |= VIPS_FOREIGN_PARTIAL;
 	else
-		flags = VIPS_FOREIGN_SEQUENTIAL;
+		flags |= VIPS_FOREIGN_SEQUENTIAL;
 
 	return( flags );
 }
@@ -103,7 +104,8 @@ vips_foreign_load_png_load( VipsForeignLoad *load )
 {
 	VipsForeignLoadPng *png = (VipsForeignLoadPng *) load;
 
-	if( vips__png_read( png->filename, load->real ) )
+	if( vips__png_read( png->filename, load->real, 
+		load->access == VIPS_ACCESS_SEQUENTIAL ) )
 		return( -1 );
 
 	return( 0 );
@@ -145,4 +147,72 @@ vips_foreign_load_png_init( VipsForeignLoadPng *png )
 {
 }
 
+typedef struct _VipsForeignLoadPngBuffer {
+	VipsForeignLoad parent_object;
+
+	/* Load from a buffer.
+	 */
+	VipsArea *buf;
+
+} VipsForeignLoadPngBuffer;
+
+typedef VipsForeignLoadClass VipsForeignLoadPngBufferClass;
+
+G_DEFINE_TYPE( VipsForeignLoadPngBuffer, vips_foreign_load_png_buffer, 
+	VIPS_TYPE_FOREIGN_LOAD );
+
+static int
+vips_foreign_load_png_buffer_header( VipsForeignLoad *load )
+{
+	VipsForeignLoadPngBuffer *png = (VipsForeignLoadPngBuffer *) load;
+
+	if( vips__png_header_buffer( png->buf->data, png->buf->length, 
+		load->out ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_png_buffer_load( VipsForeignLoad *load )
+{
+	VipsForeignLoadPngBuffer *png = (VipsForeignLoadPngBuffer *) load;
+
+	if( vips__png_read_buffer( png->buf->data, png->buf->length, 
+		load->real, load->access == VIPS_ACCESS_SEQUENTIAL ) )
+		return( -1 );
+
+	return( 0 );
+}
+
+static void
+vips_foreign_load_png_buffer_class_init( VipsForeignLoadPngBufferClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *object_class = (VipsObjectClass *) class;
+	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	object_class->nickname = "pngload_buffer";
+	object_class->description = _( "load png from buffer" );
+
+	load_class->header = vips_foreign_load_png_buffer_header;
+	load_class->load = vips_foreign_load_png_buffer_load;
+
+	VIPS_ARG_BOXED( class, "buffer", 1, 
+		_( "Buffer" ),
+		_( "Buffer to load from" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT, 
+		G_STRUCT_OFFSET( VipsForeignLoadPngBuffer, buf ),
+		VIPS_TYPE_BLOB );
+}
+
+static void
+vips_foreign_load_png_buffer_init( VipsForeignLoadPngBuffer *png )
+{
+}
+
 #endif /*HAVE_PNG*/
+
