@@ -8,6 +8,13 @@
  * 	- reworked as some fns ready for new-style classes
  * 13/12/12
  * 	- tag RGB rad images as scRGB
+ * 4/11/13
+ * 	- support sequential read
+ * 5/11/13
+ * 	- rewritten scanline encode and decode, now much faster
+ * 23/1/14
+ * 	- put the reader globals into a struct so we can have many active
+ * 	  readers
  */
 
 /*
@@ -580,99 +587,6 @@ char  *buf;
 #define  MAXELEN	0x7fff	/* maximum scanline length for encoding */
 #define  MINRUN		4	/* minimum run length */
 
-
-
-static int
-oldreadcolrs(scanline, len, fp)		/* read in an old colr scanline */
-register COLR  *scanline;
-int  len;
-register FILE  *fp;
-{
-	int  rshift;
-	register int  i;
-	
-	rshift = 0;
-	
-	while (len > 0) {
-		scanline[0][RED] = getc(fp);
-		scanline[0][GRN] = getc(fp);
-		scanline[0][BLU] = getc(fp);
-		scanline[0][EXP] = getc(fp);
-		if (feof(fp) || ferror(fp))
-			return(-1);
-		if (scanline[0][RED] == 1 &&
-				scanline[0][GRN] == 1 &&
-				scanline[0][BLU] == 1) {
-			for (i = scanline[0][EXP] << rshift; i > 0; i--) {
-				copycolr(scanline[0], scanline[-1]);
-				scanline++;
-				len--;
-			}
-			rshift += 8;
-		} else {
-			scanline++;
-			len--;
-			rshift = 0;
-		}
-	}
-	return(0);
-}
-
-
-static int
-freadcolrs(scanline, len, fp)		/* read in an encoded colr scanline */
-register COLR  *scanline;
-int  len;
-register FILE  *fp;
-{
-	register int  i, j;
-	int  code, val;
-					/* determine scanline type */
-	if ((len < MINELEN) | (len > MAXELEN))
-		return(oldreadcolrs(scanline, len, fp));
-	if ((i = getc(fp)) == EOF)
-		return(-1);
-	if (i != 2) {
-		ungetc(i, fp);
-		return(oldreadcolrs(scanline, len, fp));
-	}
-	scanline[0][GRN] = getc(fp);
-	scanline[0][BLU] = getc(fp);
-	if ((i = getc(fp)) == EOF)
-		return(-1);
-	if (scanline[0][GRN] != 2 || scanline[0][BLU] & 128) {
-		scanline[0][RED] = 2;
-		scanline[0][EXP] = i;
-		return(oldreadcolrs(scanline+1, len-1, fp));
-	}
-	if ((scanline[0][BLU]<<8 | i) != len)
-		return(-1);		/* length mismatch! */
-					/* read each component */
-	for (i = 0; i < 4; i++)
-	    for (j = 0; j < len; ) {
-		if ((code = getc(fp)) == EOF)
-		    return(-1);
-		if (code > 128) {	/* run */
-		    code &= 127;
-		    if ((val = getc(fp)) == EOF)
-			return -1;
-		    if (j + code > len)
-		    	return -1;	/* overrun */
-		    while (code--)
-			scanline[j++][i] = val;
-		} else {		/* non-run */
-		    if (j + code > len)
-		    	return -1;	/* overrun */
-		    while (code--) {
-			if ((val = getc(fp)) == EOF)
-			    return -1;
-			scanline[j++][i] = val;
-		    }
-		}
-	    }
-	return(0);
-}
-
 static void
 fputformat(		/* put out a format value */
 	char  *s,
@@ -700,65 +614,263 @@ register RESOLU  *rp;
 	return(buf);
 }
 
-static int
-fwritecolrs(scanline, len, fp)		/* write out a colr scanline */
-register COLR  *scanline;
-int  len;
-register FILE  *fp;
-{
-	register int  i, j, beg, cnt = 1;
-	int  c2;
-	
-	if ((len < MINELEN) | (len > MAXELEN))	/* OOBs, write out flat */
-		return(fwrite((char *)scanline,sizeof(COLR),len,fp) - len);
-					/* put magic header */
-	putc(2, fp);
-	putc(2, fp);
-	putc(len>>8, fp);
-	putc(len&255, fp);
-					/* put components seperately */
-	for (i = 0; i < 4; i++) {
-	    for (j = 0; j < len; j += cnt) {	/* find next run */
-		for (beg = j; beg < len; beg += cnt) {
-		    for (cnt = 1; cnt < 127 && beg+cnt < len &&
-			    scanline[beg+cnt][i] == scanline[beg][i]; cnt++)
-			;
-		    if (cnt >= MINRUN)
-			break;			/* long enough */
-		}
-		if (beg-j > 1 && beg-j < MINRUN) {
-		    c2 = j+1;
-		    while (scanline[c2++][i] == scanline[j][i])
-			if (c2 == beg) {	/* short run */
-			    putc(128+beg-j, fp);
-			    putc(scanline[j][i], fp);
-			    j = beg;
-			    break;
-			}
-		}
-		while (j < beg) {		/* write out non-run */
-		    if ((c2 = beg-j) > 128) c2 = 128;
-		    putc(c2, fp);
-		    while (c2--)
-			putc(scanline[j++][i], fp);
-		}
-		if (cnt >= MINRUN) {		/* write out run */
-		    putc(128+cnt, fp);
-		    putc(scanline[beg][i], fp);
-		} else
-		    cnt = 0;
-	    }
-	}
-	return(ferror(fp) ? -1 : 0);
-}
-
-
-
-
-
 /* End copy-paste from Radiance sources.
  */
 
+#define BUFFER_SIZE (4096)
+#define BUFFER_MARGIN (256)
+
+/* Read from a FILE with a rolling memory buffer ... this lets us reduce the
+ * number of fgetc() and gives us some very quick readahead.
+ */
+
+typedef struct _Buffer { 
+	unsigned char text[BUFFER_SIZE + BUFFER_MARGIN];
+	int length;
+	int position;
+	FILE *fp;
+} Buffer; 
+
+static Buffer *
+buffer_new( FILE *fp )
+{
+	Buffer *buffer = g_new0( Buffer, 1 );
+
+	buffer->length = 0;
+	buffer->position = 0;
+	buffer->fp = fp;
+
+	return( buffer ); 
+}
+
+static void
+buffer_free( Buffer *buffer )
+{
+	g_free( buffer ); 
+}
+
+/* Make sure there are at least @require bytes of readahead available.
+ */
+static int
+buffer_need( Buffer *buffer, int require )
+{
+	int remaining;
+
+	g_assert( require < BUFFER_MARGIN ); 
+
+	remaining = buffer->length - buffer->position;
+	if( remaining < require ) {
+		size_t len;
+
+		memcpy( buffer->text, 
+			buffer->text + buffer->position, remaining ); 
+		buffer->position = 0;
+		buffer->length = remaining;
+
+		len = fread( buffer->text + buffer->length, 
+			1, BUFFER_SIZE, buffer->fp );
+		buffer->length += len;
+		remaining = buffer->length - buffer->position;
+
+		if( remaining < require ) {
+			vips_error( "rad2vips", "%s", _( "end of file" ) ); 
+			return( -1 );
+		}
+	}
+
+	return( 0 );
+}
+
+#define BUFFER_FETCH(B) ((B)->text[(B)->position++])
+#define BUFFER_PEEK(B) ((B)->text[(B)->position])
+
+/* Read a single scanlne, encoded in the old style.
+ */
+static int
+scanline_read_old( Buffer *buffer, COLR *scanline, int width )
+{
+	int rshift;
+
+	rshift = 0;
+	
+	while( width > 0 ) {
+		if( buffer_need( buffer, 4 ) )
+			return( -1 ); 
+
+		scanline[0][RED] = BUFFER_FETCH( buffer );
+		scanline[0][GRN] = BUFFER_FETCH( buffer );
+		scanline[0][BLU] = BUFFER_FETCH( buffer );
+		scanline[0][EXP] = BUFFER_FETCH( buffer );
+
+		if( scanline[0][RED] == 1 &&
+			scanline[0][GRN] == 1 &&
+			scanline[0][BLU] == 1 ) {
+			int i;
+
+			for( i = scanline[0][EXP] << rshift; i > 0; i-- ) {
+				copycolr( scanline[0], scanline[-1] );
+				scanline += 1;
+				width -= 1;
+			}
+
+			rshift += 8;
+		} 
+		else {
+			scanline += 1;
+			width -= 1;
+			rshift = 0;
+		}
+	}
+
+	return( 0 );
+}
+
+/* Read a single encoded scanline.
+ */
+static int
+scanline_read( Buffer *buffer, COLR *scanline, int width )
+{
+	int i, j;
+
+	/* Detect old-style scanlines.
+	 */
+	if( width < MINELEN ||
+		width > MAXELEN )
+		return( scanline_read_old( buffer, scanline, width ) );
+
+	if( buffer_need( buffer, 4 ) )
+		return( -1 ); 
+
+	if( BUFFER_PEEK( buffer ) != 2 ) 
+		return( scanline_read_old( buffer, scanline, width ) );
+
+	scanline[0][RED] = BUFFER_FETCH( buffer );
+	scanline[0][GRN] = BUFFER_FETCH( buffer );
+	scanline[0][BLU] = BUFFER_FETCH( buffer );
+	scanline[0][EXP] = BUFFER_FETCH( buffer );
+	if( scanline[0][GRN] != 2 || 
+		scanline[0][BLU] & 128 ) 
+		return( scanline_read_old( buffer, scanline + 1, width - 1 ) );
+
+	if( ((scanline[0][BLU] << 8) | scanline[0][EXP]) != width ) {
+		vips_error( "rad2vips", "%s", _( "scanline length mismatch" ) );
+		return( -1 ); 
+	}
+
+	for( i = 0; i < 4; i++ ) 
+		for( j = 0; j < width; ) {
+			int code, len;
+			gboolean run;
+
+			if( buffer_need( buffer, 2 ) )
+				return( -1 ); 
+
+			code = BUFFER_FETCH( buffer ); 
+			run = code > 128;
+			len = run ? code & 127 : code; 
+
+			if( j + len > width ) {
+				vips_error( "rad2vips", "%s", _( "overrun" ) ); 
+				return( -1 );
+			}
+
+			if( run ) { 
+				int val;
+
+				val = BUFFER_FETCH( buffer ); 
+				while( len-- )
+					scanline[j++][i] = val;
+			} 
+			else {
+				if( buffer_need( buffer, len ) )
+					return( -1 ); 
+				while( len-- ) 
+					scanline[j++][i] = 
+						BUFFER_FETCH( buffer );
+			}
+		}
+
+	return( 0 );
+}
+
+/* An encoded scanline can't be larger than this.
+ */
+#define MAX_LINE (2 * MAXELEN * sizeof( COLR ))
+
+/* Write a single scanline.
+ */
+static int
+scanline_write( COLR *scanline, int width, FILE *fp )
+{
+	unsigned char buffer[MAX_LINE];
+	int buffer_pos = 0;
+
+#define PUTC( CH ) { \
+	buffer[buffer_pos++] = (CH); \
+	g_assert( buffer_pos <= MAX_LINE ); \
+}
+
+	int i, j, beg, cnt;
+
+	if( width < MINELEN || 
+		width > MAXELEN )
+		/* Write as a flat scanline.
+		 */
+		return( fwrite( scanline, sizeof( COLR ), width, fp ) - width );
+
+	/* An RLE scanline. Write magic header.
+	 */
+	PUTC( 2 ); 
+	PUTC( 2 ); 
+	PUTC( width >> 8 ); 
+	PUTC( width & 255 ); 
+
+	for( i = 0; i < 4; i++ ) {
+		for( j = 0; j < width; ) {
+			/* Set beg / cnt to the start and length of the next 
+			 * run longer than MINRUN.
+			 */
+			for( beg = j; beg < width; beg += cnt ) {
+				for( cnt = 1; 
+					cnt < 127 && 
+					beg + cnt < width &&
+					scanline[beg + cnt][i] == 
+						scanline[beg][i]; 
+					cnt++ )
+					;
+
+				if( cnt >= MINRUN )
+					break;
+			}
+
+			/* Code pixels leading up to the run as a set of
+			 * non-runs. 
+			 */
+			while( j < beg ) {
+				int len = VIPS_MIN( 128, beg - j ); 
+				COLR *p = scanline + j; 
+
+				int k;
+
+				PUTC( len ); 
+				for( k = 0; k < len; k++ )
+					PUTC( p[k][i] );
+				j += len;
+			}
+
+			/* Code the run we found, if any
+			 */
+			if( cnt >= MINRUN ) {
+				PUTC( 128 + cnt ); 
+				PUTC( scanline[j][i] ); 
+				j += cnt; 
+			} 
+		}
+	}
+
+	return( fwrite( buffer, 1, buffer_pos, fp ) - buffer_pos );
+}
+
 /* What we track during radiance file read.
  */
 typedef struct {
@@ -772,8 +884,7 @@ typedef struct {
 	double aspect;
 	RGBPRIMS prims;
 	RESOLU rs;
-
-	COLR *buf;
+	Buffer *buffer; 
 } Read;
 
 int
@@ -797,13 +908,11 @@ vips__rad_israd( const char *filename )
 }
 
 static void
-read_destroy( Read *read )
+read_destroy( VipsObject *object, Read *read )
 {
 	VIPS_FREE( read->filename );
 	VIPS_FREEF( fclose, read->fin );
-	VIPS_FREE( read->buf );
-
-	vips_free( read );
+	VIPS_FREEF( buffer_free, read->buffer );
 }
 
 static Read *
@@ -812,7 +921,7 @@ read_new( const char *filename, VipsImage *out )
 	Read *read;
 	int i;
 
-	if( !(read = VIPS_NEW( NULL, Read )) )
+	if( !(read = VIPS_NEW( out, Read )) )
 		return( NULL );
 
 	read->filename = vips_strdup( NULL, filename );
@@ -831,12 +940,13 @@ read_new( const char *filename, VipsImage *out )
 	read->prims[2][1] = CIE_y_b;
 	read->prims[3][0] = CIE_x_w;
 	read->prims[3][1] = CIE_y_w;
-	read->buf = NULL;
 
-	if( !(read->fin = vips__file_open_read( filename, NULL, FALSE )) ) {
-		read_destroy( read );
+	g_signal_connect( out, "close", 
+		G_CALLBACK( read_destroy ), read );
+
+	if( !(read->fin = vips__file_open_read( filename, NULL, FALSE )) || 
+		!(read->buffer = buffer_new( read->fin )) )
 		return( NULL );
-	}
 
 	return( read );
 }
@@ -883,36 +993,35 @@ static const char *colcor_name[3] = {
 };
 
 static int
-rad2vips_get_header( Read *read, FILE *fin, VipsImage *out )
+rad2vips_get_header( Read *read, VipsImage *out )
 {
 	int i, j;
+	VipsInterpretation interpretation;
 
-	if( getheader( fin, (gethfunc *) rad2vips_process_line, read ) ||
-		!fgetsresolu( &read->rs, fin ) ) {
+	if( getheader( read->fin, (gethfunc *) rad2vips_process_line, read ) ||
+		!fgetsresolu( &read->rs, read->fin ) ) {
 		vips_error( "rad2vips", "%s", 
 			_( "error reading radiance header" ) );
 		return( -1 );
 	}
-	out->Xsize = scanlen( &read->rs );
-	out->Ysize = numscans( &read->rs );
-
-	out->Bands = 4;
-	out->BandFmt = VIPS_FORMAT_UCHAR;
-
-	out->Coding = VIPS_CODING_RAD;
-	out->Xres = 1.0;
-	out->Yres = read->aspect;
-	out->Xoffset = 0.0;
-	out->Yoffset = 0.0;
-
-	vips_image_set_string( out, "rad-format", read->format );
 
 	if( strcmp( read->format, COLRFMT ) == 0 )
-		out->Type = VIPS_INTERPRETATION_scRGB;
+		interpretation = VIPS_INTERPRETATION_scRGB;
 	else if( strcmp( read->format, CIEFMT ) == 0 )
-		out->Type = VIPS_INTERPRETATION_XYZ;
+		interpretation = VIPS_INTERPRETATION_XYZ;
 	else
-		out->Type = VIPS_INTERPRETATION_MULTIBAND;
+		interpretation = VIPS_INTERPRETATION_MULTIBAND;
+
+	vips_image_init_fields( out,
+		scanlen( &read->rs ), numscans( &read->rs ),
+		4,
+		VIPS_FORMAT_UCHAR, VIPS_CODING_RAD,
+		interpretation,
+		1, read->aspect );
+
+	vips_image_pipelinev( out, VIPS_DEMAND_STYLE_THINSTRIP, NULL );
+
+	vips_image_set_string( out, "rad-format", read->format );
 
 	vips_image_set_double( out, "rad-expos", read->expos );
 
@@ -941,42 +1050,50 @@ vips__rad_header( const char *filename, VipsImage *out )
 
 	if( !(read = read_new( filename, out )) ) 
 		return( -1 );
-	if( rad2vips_get_header( read, read->fin, read->out ) ) {
-		read_destroy( read );
+	if( rad2vips_get_header( read, read->out ) ) 
 		return( -1 );
-	}
-	read_destroy( read );
 
 	return( 0 );
 }
 
 static int
-rad2vips_get_data( Read *read, FILE *fin, VipsImage *im )
+rad2vips_generate( VipsRegion *or, 
+	void *seq, void *a, void *b, gboolean *stop )
 {
+        VipsRect *r = &or->valid;
+	Read *read = (Read *) a; 
+
 	int y;
 
 #ifdef DEBUG
-	printf( "rad2vips_get_data\n" );
+	printf( "rad2vips_generate: line %d, %d rows\n", 
+		r->top, r->height );
 #endif /*DEBUG*/
 
-	if( !(read->buf = VIPS_ARRAY( NULL, im->Xsize, COLR )) )
-		return( -1 );
+	VIPS_GATE_START( "rad2vips_generate: work" );
 
-	for( y = 0; y < im->Ysize; y++ ) {
-		if( freadcolrs( read->buf, im->Xsize, fin ) ) {
-			vips_error( "rad2vips", "%s", _( "read error" ) );
+	for( y = 0; y < r->height; y++ ) {
+		COLR *buf = (COLR *) 
+			VIPS_REGION_ADDR( or, 0, r->top + y );
+
+		if( scanline_read( read->buffer, buf, or->im->Xsize ) ) {
+			vips_error( "rad2vips", 
+				_( "read error line %d" ), r->top + y );
 			return( -1 );
 		}
-		if( vips_image_write_line( im, y, (void *) read->buf ) )
-			return( -1 );
 	}
 
+	VIPS_GATE_STOP( "rad2vips_generate: work" );
+
 	return( 0 );
 }
 
 int
-vips__rad_load( const char *filename, VipsImage *out )
+vips__rad_load( const char *filename, VipsImage *out, gboolean readbehind )
 {
+	VipsImage **t = (VipsImage **) 
+		vips_object_local_array( VIPS_OBJECT( out ), 3 );
+
 	Read *read;
 
 #ifdef DEBUG
@@ -985,17 +1102,27 @@ vips__rad_load( const char *filename, VipsImage *out )
 
 	if( !(read = read_new( filename, out )) ) 
 		return( -1 );
-	if( rad2vips_get_header( read, read->fin, read->out ) ||
-		rad2vips_get_data( read, read->fin, read->out ) ) {
-		read_destroy( read );
+
+	t[0] = vips_image_new();
+	if( rad2vips_get_header( read, t[0] ) )
+		return( -1 );
+
+	if( vips_image_generate( t[0], 
+		NULL, rad2vips_generate, NULL, 
+		read, NULL ) ||
+		vips_sequential( t[0], &t[1], 
+			"tile_height", 8,
+			"access", readbehind ? 
+				VIPS_ACCESS_SEQUENTIAL : 
+				VIPS_ACCESS_SEQUENTIAL_UNBUFFERED,
+			NULL ) ||
+		vips_image_write( t[1], out ) )
 		return( -1 );
-	}
-	read_destroy( read );
 
 	return( 0 );
 }
 
-/* What we track during an radiance-file write.
+/* What we track during a radiance file write.
  */
 typedef struct {
 	VipsImage *in;
@@ -1065,7 +1192,7 @@ vips2rad_put_header( Write *write )
 
 	if( !vips_image_get_string( write->in, "rad-format", &str ) )
 		vips_strncpy( write->format, str, 256 );
-	if( write->in->Type == VIPS_INTERPRETATION_RGB )
+	if( write->in->Type == VIPS_INTERPRETATION_scRGB )
 		strcpy( write->format, COLRFMT );
 	if( write->in->Type == VIPS_INTERPRETATION_XYZ )
 		strcpy( write->format, CIEFMT );
@@ -1108,7 +1235,7 @@ vips2rad_put_data_block( VipsRegion *region, Rect *area, void *a )
 	for( i = 0; i < area->height; i++ ) {
 		VipsPel *p = VIPS_REGION_ADDR( region, 0, area->top + i );
 
-		if( fwritecolrs( p, area->width, write->fout ) ) 
+		if( scanline_write( (COLR *) p, area->width, write->fout ) ) 
 			return( -1 );
 	}
 
