@@ -58,6 +58,10 @@
  * 	- attach IPCT data (app13), thanks Gary
  * 2/10/13 Lovell Fuller
  * 	- add optimize_coding parameter
+ * 12/11/13
+ * 	- add "strip" option to remove all metadata
+ * 13/11/13
+ * 	- add a "no_subsample" option to disable chroma subsample
  */
 
 /*
@@ -841,7 +845,8 @@ write_jpeg_block( REGION *region, Rect *area, void *a )
  */
 static int
 write_vips( Write *write, int qfac, const char *profile, 
-	gboolean optimize_coding )
+	gboolean optimize_coding, gboolean progressive, gboolean strip, 
+	gboolean no_subsample )
 {
 	VipsImage *in;
 	J_COLOR_SPACE space;
@@ -898,28 +903,48 @@ write_vips( Write *write, int qfac, const char *profile,
 	 */
 	write->cinfo.optimize_coding = optimize_coding;
 
+	/* Enable progressive write.
+	 */
+	if( progressive ) 
+		jpeg_simple_progression( &write->cinfo ); 
+
+	/* Turn off chroma subsampling.
+	 */
+	if( no_subsample ) { 
+		int i;
+
+		for( i = 0; i < in->Bands; i++ ) { 
+			write->cinfo.comp_info[i].h_samp_factor = 1;
+			write->cinfo.comp_info[i].v_samp_factor = 1;
+		}
+	}
+
 	/* Build compress tables.
 	 */
 	jpeg_start_compress( &write->cinfo, TRUE );
 
 	/* Write any APP markers we need.
 	 */
-	if( write_exif( write ) ||
-		write_blob( write, VIPS_META_XMP_NAME, JPEG_APP0 + 1 ) ||
-		write_blob( write, VIPS_META_IPCT_NAME, JPEG_APP0 + 13 ) )
-		return( -1 );
+	if( !strip ) { 
+		if( write_exif( write ) ||
+			write_blob( write, 
+				VIPS_META_XMP_NAME, JPEG_APP0 + 1 ) ||
+			write_blob( write, 
+				VIPS_META_IPCT_NAME, JPEG_APP0 + 13 ) )
+			return( -1 );
 
-	/* A profile supplied as an argument overrides an embedded profile.
-	 * "none" means don't attach a profile.
-	 */
-	if( profile && 
-		strcmp( profile, "none" ) != 0 &&
-		write_profile_file( write, profile ) )
-		return( -1 );
-	if( !profile && 
-		vips_image_get_typeof( in, VIPS_META_ICC_NAME ) && 
-		write_profile_meta( write ) )
-		return( -1 );
+		/* A profile supplied as an argument overrides an embedded 
+		 * profile. "none" means don't attach a profile.
+		 */
+		if( profile && 
+			strcmp( profile, "none" ) != 0 &&
+			write_profile_file( write, profile ) )
+			return( -1 );
+		if( !profile && 
+			vips_image_get_typeof( in, VIPS_META_ICC_NAME ) && 
+			write_profile_meta( write ) )
+			return( -1 );
+	}
 
 	/* Write data. Note that the write function grabs the longjmp()!
 	 */
@@ -941,7 +966,8 @@ write_vips( Write *write, int qfac, const char *profile,
 int
 vips__jpeg_write_file( VipsImage *in, 
 	const char *filename, int Q, const char *profile, 
-	gboolean optimize_coding )
+	gboolean optimize_coding, gboolean progressive, gboolean strip, 
+	gboolean no_subsample )
 {
 	Write *write;
 
@@ -971,7 +997,9 @@ vips__jpeg_write_file( VipsImage *in,
 
 	/* Convert!
 	 */
-	if( write_vips( write, Q, profile, optimize_coding ) ) {
+	if( write_vips( write, 
+		Q, profile, optimize_coding, progressive, strip, 
+		no_subsample ) ) {
 		write_destroy( write );
 		return( -1 );
 	}
@@ -1218,7 +1246,8 @@ buf_dest( j_compress_ptr cinfo, void **obuf, size_t *olen )
 int
 vips__jpeg_write_buffer( VipsImage *in, 
 	void **obuf, size_t *olen, int Q, const char *profile, 
-	gboolean optimize_coding )
+	gboolean optimize_coding, gboolean progressive,
+	gboolean strip, gboolean no_subsample )
 {
 	Write *write;
 
@@ -1247,7 +1276,9 @@ vips__jpeg_write_buffer( VipsImage *in,
 
 	/* Convert!
 	 */
-	if( write_vips( write, Q, profile, optimize_coding ) ) {
+	if( write_vips( write, 
+		Q, profile, optimize_coding, progressive, strip, 
+		no_subsample ) ) {
 		write_destroy( write );
 
 		return( -1 );
