@@ -145,11 +145,23 @@ vips_histogram_build( VipsObject *object )
 			vips_check_hist( class->nickname, histogram->in[i] ) )
 			return( -1 ); 
 
-	/* Cast our input images up to a common format, bands and size.
+	/* Cast our input images up to a common format, bands and size. If
+	 * input_format is set, cast to a fixed input type.
 	 */
-	if( vips__formatalike_vec( histogram->in, format, histogram->n ) ||
-		vips__bandalike_vec( class->nickname, 
-			format, band, histogram->n, 1 ) ||
+	if( hclass->input_format != VIPS_FORMAT_NOTSET ) {
+		for( i = 0; i < histogram->n; i++ ) 
+			if( vips_cast( histogram->in[i], &format[i],
+				hclass->input_format, NULL ) )
+				return( -1 ); 
+	}
+	else {
+		if( vips__formatalike_vec( histogram->in, 
+			format, histogram->n ) )
+			return( -1 );
+	}
+		
+	if( vips__bandalike_vec( class->nickname, 
+		format, band, histogram->n, 1 ) ||
 		vips__hist_sizealike_vec( band, size, histogram->n ) ) 
 		return( -1 );
 
@@ -157,10 +169,9 @@ vips_histogram_build( VipsObject *object )
 	 */
 	histogram->ready = size;
 
-	if( vips_image_copy_fields_array( histogram->out, histogram->ready ) ) 
+	if( vips_image_pipeline_array( histogram->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, histogram->ready ) ) 
 		return( -1 );
-        vips_demand_hint_array( histogram->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, histogram->ready );
 
 	histogram->out->Xsize = VIPS_IMAGE_N_PELS( histogram->ready[0] );
 	histogram->out->Ysize = 1;
@@ -203,6 +214,8 @@ vips_histogram_class_init( VipsHistogramClass *class )
 	vobject_class->description = _( "histogram operations" );
 	vobject_class->build = vips_histogram_build;
 
+	class->input_format = VIPS_FORMAT_NOTSET;
+
 	/* Inputs set by subclassess.
 	 */
 
