@@ -77,30 +77,30 @@ int im_c2ps( IMAGE *in, IMAGE *out );
 
 int im_clip( IMAGE *in, IMAGE *out );
 
-#define MASK_IDEAL_HIGHPASS VIPS_MASK_IDEAL_HIGHPASS
-#define MASK_IDEAL_LOWPASS VIPS_MASK_IDEAL_LOWPASS 
-#define MASK_BUTTERWORTH_HIGHPASS VIPS_MASK_BUTTERWORTH_HIGHPASS 
-#define MASK_BUTTERWORTH_LOWPASS VIPS_MASK_BUTTERWORTH_LOWPASS 
-#define MASK_GAUSS_HIGHPASS VIPS_MASK_GAUSS_HIGHPASS 
-#define MASK_GAUSS_LOWPASS VIPS_MASK_GAUSS_LOWPASS 
+#define MASK_IDEAL_HIGHPASS IM_MASK_IDEAL_HIGHPASS
+#define MASK_IDEAL_LOWPASS IM_MASK_IDEAL_LOWPASS 
+#define MASK_BUTTERWORTH_HIGHPASS IM_MASK_BUTTERWORTH_HIGHPASS 
+#define MASK_BUTTERWORTH_LOWPASS IM_MASK_BUTTERWORTH_LOWPASS 
+#define MASK_GAUSS_HIGHPASS IM_MASK_GAUSS_HIGHPASS 
+#define MASK_GAUSS_LOWPASS IM_MASK_GAUSS_LOWPASS 
 
-#define MASK_IDEAL_RINGPASS VIPS_MASK_IDEAL_RINGPASS
-#define MASK_IDEAL_RINGREJECT VIPS_MASK_IDEAL_RINGREJECT 
-#define MASK_BUTTERWORTH_RINGPASS VIPS_MASK_BUTTERWORTH_RINGPASS 
-#define MASK_BUTTERWORTH_RINGREJECT VIPS_MASK_BUTTERWORTH_RINGREJECT 
-#define MASK_GAUSS_RINGPASS VIPS_MASK_GAUSS_RINGPASS 
-#define MASK_GAUSS_RINGREJECT VIPS_MASK_GAUSS_RINGREJECT 
+#define MASK_IDEAL_RINGPASS IM_MASK_IDEAL_RINGPASS
+#define MASK_IDEAL_RINGREJECT IM_MASK_IDEAL_RINGREJECT 
+#define MASK_BUTTERWORTH_RINGPASS IM_MASK_BUTTERWORTH_RINGPASS 
+#define MASK_BUTTERWORTH_RINGREJECT IM_MASK_BUTTERWORTH_RINGREJECT 
+#define MASK_GAUSS_RINGPASS IM_MASK_GAUSS_RINGPASS 
+#define MASK_GAUSS_RINGREJECT IM_MASK_GAUSS_RINGREJECT 
 
-#define MASK_IDEAL_BANDPASS VIPS_MASK_IDEAL_BANDPASS
-#define MASK_IDEAL_BANDREJECT VIPS_MASK_IDEAL_BANDREJECT 
-#define MASK_BUTTERWORTH_BANDPASS VIPS_MASK_BUTTERWORTH_BANDPASS 
-#define MASK_BUTTERWORTH_BANDREJECT VIPS_MASK_BUTTERWORTH_BANDREJECT 
-#define MASK_GAUSS_BANDPASS VIPS_MASK_GAUSS_BANDPASS 
-#define MASK_GAUSS_BANDREJECT VIPS_MASK_GAUSS_BANDREJECT 
+#define MASK_IDEAL_BANDPASS IM_MASK_IDEAL_BANDPASS
+#define MASK_IDEAL_BANDREJECT IM_MASK_IDEAL_BANDREJECT 
+#define MASK_BUTTERWORTH_BANDPASS IM_MASK_BUTTERWORTH_BANDPASS 
+#define MASK_BUTTERWORTH_BANDREJECT IM_MASK_BUTTERWORTH_BANDREJECT 
+#define MASK_GAUSS_BANDPASS IM_MASK_GAUSS_BANDPASS 
+#define MASK_GAUSS_BANDREJECT IM_MASK_GAUSS_BANDREJECT 
 
-#define MASK_FRACTAL_FLT VIPS_MASK_FRACTAL_FLT
+#define MASK_FRACTAL_FLT IM_MASK_FRACTAL_FLT
 
-#define MaskType VipsMaskType
+#define MaskType ImMaskType
 
 /* Copy and swap types.
  */
