@@ -41,46 +41,27 @@ extern "C" {
 typedef enum {
 	VIPS_PRECISION_INTEGER,
 	VIPS_PRECISION_FLOAT,
-	VIPS_PRECISION_APPROXIMATE
+	VIPS_PRECISION_APPROXIMATE,
+	VIPS_PRECISION_LAST
 } VipsPrecision;
 
+typedef enum {
+	VIPS_COMBINE_MAX,
+	VIPS_COMBINE_SUM,
+	VIPS_COMBINE_LAST
+} VipsCombine;
+
 int vips_conv( VipsImage *in, VipsImage **out, VipsImage *mask, ... )
 	__attribute__((sentinel));
+int vips_compass( VipsImage *in, VipsImage **out, VipsImage *mask, ... )
+	__attribute__((sentinel));
+int vips_convsep( VipsImage *in, VipsImage **out, VipsImage *mask, ... )
+	__attribute__((sentinel));
 
-void vips_convolution_operation_init( void );
-
-
-
-
-
-int im_aconvsep( VipsImage *in, VipsImage *out, 
-	DOUBLEMASK *mask, int n_layers );
-int im_aconv( VipsImage *in, VipsImage *out, 
-	DOUBLEMASK *mask, int n_layers, int cluster );
-int im_conv( VipsImage *in, VipsImage *out, INTMASK *mask );
-int im_conv_f( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
-int im_convsep( VipsImage *in, VipsImage *out, INTMASK *mask );
-int im_convsep_f( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
-
-int im_compass( VipsImage *in, VipsImage *out, INTMASK *mask );
-int im_gradient( VipsImage *in, VipsImage *out, INTMASK *mask );
-int im_lindetect( VipsImage *in, VipsImage *out, INTMASK *mask );
-
-int im_sharpen( VipsImage *in, VipsImage *out, 
-	int mask_size, 
-	double x1, double y2, double y3, 
-	double m1, double m2 );
-
-int im_grad_x( VipsImage *in, VipsImage *out );
-int im_grad_y( VipsImage *in, VipsImage *out );
-
-int im_fastcor( VipsImage *in, VipsImage *ref, VipsImage *out );
-int im_spcor( VipsImage *in, VipsImage *ref, VipsImage *out );
-int im_gradcor( VipsImage *in, VipsImage *ref, VipsImage *out );
-int im_contrast_surface( VipsImage *in, VipsImage *out, 
-	int half_win_size, int spacing );
-
-int im_addgnoise( VipsImage *in, VipsImage *out, double sigma );
+int vips_sharpen( VipsImage *in, VipsImage **out, ... ) 
+	__attribute__((sentinel));
+int vips_gaussblur( VipsImage *in, VipsImage **out, int radius, ... )
+	__attribute__((sentinel));
 
 #ifdef __cplusplus
 }
