@@ -83,10 +83,12 @@ int vips_check_8or16( const char *domain, VipsImage *im );
 int vips_check_u8or16orf( const char *domain, VipsImage *im );
 int vips_check_format_same( const char *domain, VipsImage *im1, VipsImage *im2 );
 int vips_check_size_same( const char *domain, VipsImage *im1, VipsImage *im2 );
+int vips_check_oddsquare( const char *domain, VipsImage *im );
 int vips_check_vector_length( const char *domain, int n, int len );
 int vips_check_vector( const char *domain, int n, VipsImage *im );
 int vips_check_hist( const char *domain, VipsImage *im );
 int vips_check_matrix( const char *domain, VipsImage *im, VipsImage **out );
+int vips_check_separable( const char *domain, VipsImage *im );
 
 int vips_check_imask( const char *domain, INTMASK *mask );
 int vips_check_dmask( const char *domain, DOUBLEMASK *mask );
