@@ -31,26 +31,37 @@
 
  */
 
-#ifndef IM_MORPHOLOGY_H
-#define IM_MORPHOLOGY_H
+#ifndef VIPS_MORPHOLOGY_H
+#define VIPS_MORPHOLOGY_H
 
 #ifdef __cplusplus
 extern "C" {
 #endif /*__cplusplus*/
 
-int im_dilate( VipsImage *in, VipsImage *out, INTMASK *mask );
-int im_erode( VipsImage *in, VipsImage *out, INTMASK *mask );
+typedef enum {
+	VIPS_OPERATION_MORPHOLOGY_ERODE,
+	VIPS_OPERATION_MORPHOLOGY_DILATE,
+	VIPS_OPERATION_MORPHOLOGY_LAST
+} VipsOperationMorphology;
+
+int vips_morph( VipsImage *in, VipsImage **out, VipsImage *mask, 
+	VipsOperationMorphology morph, ... )
+	__attribute__((sentinel));
+int vips_rank( VipsImage *in, VipsImage **out, 
+	int width, int height, int index, ... )
+	__attribute__((sentinel));
+int vips_countlines( VipsImage *in, double *nolines, 
+	VipsDirection direction, ... )
+	__attribute__((sentinel));
+int vips_median( VipsImage *in, VipsImage **out, int size, ... )
+	__attribute__((sentinel));
+
 
-int im_rank( VipsImage *in, VipsImage *out, int width, int height, int index );
-int im_rank_image( VipsImage **in, VipsImage *out, int n, int index );
-int im_maxvalue( VipsImage **in, VipsImage *out, int n );
 
-int im_cntlines( VipsImage *im, double *nolines, int flag );
-int im_zerox( VipsImage *in, VipsImage *out, int sign );
 int im_label_regions( VipsImage *test, VipsImage *mask, int *segments );
 
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
 
-#endif /*IM_MORPHOLOGY_H*/
+#endif /*VIPS_MORPHOLOGY_H*/
