@@ -56,6 +56,11 @@ extern "C" {
  */
 #define VIPS_SIZEOF_HEADER (64)
 
+/* Startup plus ABI check.
+ */
+int vips__init( const char *argv0 );
+size_t vips__get_sizeof_vipsobject( void );
+
 /* What we track for each mmap window. Have a list of these on an openin
  * VipsImage.
  */
@@ -77,26 +82,29 @@ VipsWindow *vips_window_ref( struct _VipsImage *im, int top, int height );
 int vips_window_unref( VipsWindow *window );
 void vips_window_print( VipsWindow *window );
 
-/* Per-thread buffer cache. Held in a GPrivate.
+/* Per-thread buffer state. Held in a GPrivate.
  */
 typedef struct {
-	GHashTable *hash;	/* Hash to VipsBufferCacheList* */
+	GHashTable *hash;	/* VipsImage -> VipsBufferCache* */
 	GThread *thread;	/* Just for sanity checking */
-} VipsBufferCache;
+} VipsBufferThread;
 
 /* Per-image buffer cache. Hash to this from VipsBufferCache.
  * We can't store the GSList directly in the hash table, as GHashTable lacks an
  * update operation and we'd need to _remove() and _insert() on every list
  * operation.
  */
-typedef struct _VipsBufferCacheList {
+typedef struct _VipsBufferCache {
 	GSList *buffers;	/* GSList of VipsBuffer* */
 	GThread *thread;	/* Just for sanity checking */
 	struct _VipsImage *im;
-	VipsBufferCache *cache;
-} VipsBufferCacheList;
+	VipsBufferThread *buffer_thread;
+	GSList *reserve;	/* VipsBuffer kept in reserve */
+	int n_reserve;		/* Number in reserve */
+} VipsBufferCache;
 
-/* What we track for each pixel buffer. 
+/* What we track for each pixel buffer. These can move between caches and
+ * between threads, but not between images. 
  */
 typedef struct _VipsBuffer {
 	int ref_count;		/* # of regions referencing us */
@@ -104,11 +112,12 @@ typedef struct _VipsBuffer {
 
 	VipsRect area;		/* Area this pixel buffer covers */
 	gboolean done;		/* Calculated and in cache */
-	VipsBufferCache *cache;
+	VipsBufferCache *cache;	/* The cache this buffer is published on */
 	VipsPel *buf;		/* Private malloc() area */
 	size_t bsize;		/* Size of private malloc() */
 } VipsBuffer;
 
+void vips_buffer_dump_all( void );
 void vips_buffer_done( VipsBuffer *buffer );
 void vips_buffer_undone( VipsBuffer *buffer );
 void vips_buffer_unref( VipsBuffer *buffer );
@@ -149,9 +158,6 @@ void vips__region_take_ownership( struct _VipsRegion *reg );
 void vips__region_check_ownership( struct _VipsRegion *reg );
 void vips__region_no_ownership( struct _VipsRegion *reg );
 
-int vips__region_start( struct _VipsRegion *reg );
-void vips__region_stop( struct _VipsRegion *reg );
-
 typedef int (*VipsRegionFillFn)( struct _VipsRegion *, void * );
 int vips_region_fill( struct _VipsRegion *reg, 
 	VipsRect *r, VipsRegionFillFn fn, void *a );
@@ -165,6 +171,11 @@ VipsArgumentInstance *vips__argument_get_instance(
 VipsArgument *vips__argument_table_lookup( VipsArgumentTable *table, 
 	GParamSpec *pspec);
 
+void vips__demand_hint_array( struct _VipsImage *image, 
+	int hint, struct _VipsImage **in );
+int vips__image_copy_fields_array( struct _VipsImage *out, 
+	struct _VipsImage *in[] );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
