@@ -101,6 +101,7 @@ extern "C" {
 #include <vips/util.h>
 #include <vips/object.h>
 #include <vips/type.h>
+#include <vips/gate.h>
 
 #include <vips/version.h>
 #include <vips/rect.h>
@@ -130,7 +131,7 @@ extern "C" {
 #include <vips/morphology.h>
 #include <vips/mosaicing.h>
 #include <vips/histogram.h>
-#include <vips/freq_filt.h>
+#include <vips/freqfilt.h>
 #include <vips/resample.h>
 #include <vips/colour.h>
 #include <vips/inplace.h>
@@ -149,10 +150,26 @@ extern "C" {
 #include <vips/almostdeprecated.h>
 #include <vips/dispatch.h>
 
+/* We can't use _ here since this will be compiled by our clients and they may
+ * not have _().
+ */
+#define vips_init( ARGV0 ) \
+	(sizeof( VipsObject ) != vips__get_sizeof_vipsobject() ? ( \
+		vips_info( "vips_init", "ABI mismatch" ), \
+		vips_info( "vips_init", \
+			"library has sizeof(VipsObject) == %zd", \
+			vips__get_sizeof_vipsobject() ), \
+		vips_info( "vips_init", \
+			"application has sizeof(VipsObject) == %zd", \
+			sizeof( VipsObject ) ), \
+		vips_error( "vips_init", "ABI mismatch" ), \
+		-1 ) : \
+		vips__init( ARGV0 ))
+
 const char *vips_get_argv0( void );
-int vips_init( const char *argv0 );
 void vips_check_init( void );
 void vips_shutdown( void );
+void vips_thread_shutdown( void );
 GOptionGroup *vips_get_option_group( void );
 
 const char *vips_version_string( void );
