@@ -208,9 +208,12 @@ extern "C" {
 #define im_guess_libdir vips_guess_libdir
 #define im__global_lock vips__global_lock
 
-#define im_cp_desc vips_image_copy_fields
-#define im_cp_descv vips_image_copy_fieldsv
-#define im_cp_desc_array vips_image_copy_fields_array
+int im_cp_desc(IMAGE *out, IMAGE *in );
+int im_cp_descv (IMAGE * im, ...);
+#define im_cp_desc_array(I, A) vips__image_copy_fields_array(I, A)
+int im_demand_hint (IMAGE * im, VipsDemandStyle hint, ...);
+#define im_demand_hint_array( A, B, C ) (vips__demand_hint_array( A, B, C ), 0)
+
 #define im_image vips_image_new_from_memory
 #define im_binfile vips_image_new_from_file_raw
 #define im__open_temp vips_image_new_temp_file
@@ -326,9 +329,6 @@ VipsDemandStyle im_char2dhint( const char *str );
 #define im_rect_dup vips_rect_dup
 #define im_rect_normalise vips_rect_normalise
 
-int im_demand_hint (IMAGE * im, VipsDemandStyle hint, ...);
-#define im_demand_hint_array( A, B, C ) (vips_demand_hint_array( A, B, C ), 0)
-
 #define im_start_one vips_start_one
 #define im_stop_one vips_stop_one
 #define im_start_many vips_start_many
@@ -587,6 +587,8 @@ size_t im_ref_string_get_length( const GValue *value );
 #define im_concurrency_set vips_concurrency_set
 #define im_concurrency_get vips_concurrency_get
 
+int im_init_world( const char *argv0 ); 
+
 int im_add( VipsImage *in1, VipsImage *in2, VipsImage *out );
 int im_subtract( VipsImage *in1, VipsImage *in2, VipsImage *out );
 int im_multiply( VipsImage *in1, VipsImage *in2, VipsImage *out );
@@ -712,6 +714,8 @@ int im_rotquad( VipsImage *in, VipsImage *out );
 int im_clip2fmt( VipsImage *in, VipsImage *out, VipsBandFormat fmt );
 int im_bandjoin( VipsImage *in1, VipsImage *in2, VipsImage *out );
 int im_gbandjoin( VipsImage **in, VipsImage *out, int n );
+int im_rank_image( VipsImage **in, VipsImage *out, int n, int index );
+int im_maxvalue( VipsImage **in, VipsImage *out, int n );
 int im_grid( VipsImage *in, VipsImage *out, int tile_height, int across, int down );
 int im_scale( VipsImage *in, VipsImage *out );
 int im_scaleps( VipsImage *in, VipsImage *out );
@@ -769,6 +773,7 @@ int im_blend( VipsImage *c, VipsImage *a, VipsImage *b, VipsImage *out );
 DOUBLEMASK *im_vips2mask( VipsImage *in, const char *filename );
 INTMASK *im_vips2imask( IMAGE *in, const char *filename );
 int im_mask2vips( DOUBLEMASK *in, VipsImage *out );
+int im_imask2vips( INTMASK *in, VipsImage *out );
 
 int im_bandmean( VipsImage *in, VipsImage *out );
 int im_recomb( VipsImage *in, VipsImage *out, DOUBLEMASK *recomb );
@@ -895,6 +900,107 @@ int im_tone_map( VipsImage *in, VipsImage *out, VipsImage *lut );
  */
 #define vips_class_map_concrete_all vips_class_map_all
 
+int im_dilate( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_erode( VipsImage *in, VipsImage *out, INTMASK *mask );
+
+int im_aconv( VipsImage *in, VipsImage *out, 
+	DOUBLEMASK *mask, int n_layers, int cluster );
+int im_conv( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_conv_f( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
+
+int im_aconvsep( VipsImage *in, VipsImage *out, 
+	DOUBLEMASK *mask, int n_layers );
+
+int im_convsep( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_convsep_f( VipsImage *in, VipsImage *out, DOUBLEMASK *mask );
+
+int im_compass( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_gradient( VipsImage *in, VipsImage *out, INTMASK *mask );
+int im_lindetect( VipsImage *in, VipsImage *out, INTMASK *mask );
+
+int im_addgnoise( VipsImage *in, VipsImage *out, double sigma );
+
+int im_contrast_surface_raw( IMAGE *in, IMAGE *out, 
+	int half_win_size, int spacing );
+int im_contrast_surface( VipsImage *in, VipsImage *out, 
+	int half_win_size, int spacing );
+
+int im_grad_x( VipsImage *in, VipsImage *out );
+int im_grad_y( VipsImage *in, VipsImage *out );
+
+int im_fastcor( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_spcor( VipsImage *in, VipsImage *ref, VipsImage *out );
+int im_gradcor( VipsImage *in, VipsImage *ref, VipsImage *out );
+
+int im_sharpen( VipsImage *in, VipsImage *out, 
+	int mask_size, 
+	double x1, double y2, double y3, 
+	double m1, double m2 );
+
+typedef enum {
+        IM_MASK_IDEAL_HIGHPASS = 0,
+        IM_MASK_IDEAL_LOWPASS = 1,
+        IM_MASK_BUTTERWORTH_HIGHPASS = 2,
+        IM_MASK_BUTTERWORTH_LOWPASS = 3,
+        IM_MASK_GAUSS_HIGHPASS = 4,
+        IM_MASK_GAUSS_LOWPASS = 5,
+
+        IM_MASK_IDEAL_RINGPASS = 6,
+        IM_MASK_IDEAL_RINGREJECT = 7,
+        IM_MASK_BUTTERWORTH_RINGPASS = 8,
+        IM_MASK_BUTTERWORTH_RINGREJECT = 9,
+        IM_MASK_GAUSS_RINGPASS = 10,
+        IM_MASK_GAUSS_RINGREJECT = 11,
+
+        IM_MASK_IDEAL_BANDPASS = 12,
+        IM_MASK_IDEAL_BANDREJECT = 13,
+        IM_MASK_BUTTERWORTH_BANDPASS = 14,
+        IM_MASK_BUTTERWORTH_BANDREJECT = 15,
+        IM_MASK_GAUSS_BANDPASS = 16,
+        IM_MASK_GAUSS_BANDREJECT = 17,
+
+        IM_MASK_FRACTAL_FLT = 18
+} ImMaskType;
+
+/* We had them in the VIPS namespace for a while before deprecating them.
+ */
+#define VIPS_MASK_IDEAL_HIGHPASS IM_MASK_IDEAL_HIGHPASS 
+#define VIPS_MASK_IDEAL_LOWPASS IM_MASK_IDEAL_LOWPASS 
+#define VIPS_MASK_BUTTERWORTH_HIGHPASS IM_MASK_BUTTERWORTH_HIGHPASS 
+#define VIPS_MASK_BUTTERWORTH_LOWPASS IM_MASK_BUTTERWORTH_LOWPASS 
+#define VIPS_MASK_GAUSS_HIGHPASS IM_MASK_GAUSS_HIGHPASS 
+#define VIPS_MASK_GAUSS_LOWPASS IM_MASK_GAUSS_LOWPASS 
+#define VIPS_MASK_IDEAL_RINGPASS IM_MASK_IDEAL_RINGPASS 
+#define VIPS_MASK_IDEAL_RINGREJECT IM_MASK_IDEAL_RINGREJECT 
+#define VIPS_MASK_BUTTERWORTH_RINGPASS IM_MASK_BUTTERWORTH_RINGPASS 
+#define VIPS_MASK_BUTTERWORTH_RINGREJECT IM_MASK_BUTTERWORTH_RINGREJECT 
+#define VIPS_MASK_GAUSS_RINGPASS IM_MASK_GAUSS_RINGPASS 
+#define VIPS_MASK_GAUSS_RINGREJECT IM_MASK_GAUSS_RINGREJECT 
+#define VIPS_MASK_IDEAL_BANDPASS IM_MASK_IDEAL_BANDPASS 
+#define VIPS_MASK_IDEAL_BANDREJECT IM_MASK_IDEAL_BANDREJECT 
+#define VIPS_MASK_BUTTERWORTH_BANDPASS IM_MASK_BUTTERWORTH_BANDPASS 
+#define VIPS_MASK_BUTTERWORTH_BANDREJECT IM_MASK_BUTTERWORTH_BANDREJECT 
+#define VIPS_MASK_GAUSS_BANDPASS IM_MASK_GAUSS_BANDPASS 
+#define VIPS_MASK_GAUSS_BANDREJECT IM_MASK_GAUSS_BANDREJECT 
+#define VIPS_MASK_FRACTAL_FLT IM_MASK_FRACTAL_FLT 
+
+int im_flt_image_freq( VipsImage *in, VipsImage *out, ImMaskType flag, ... );
+int im_create_fmask( VipsImage *out, 
+	int xsize, int ysize, ImMaskType flag, ... );
+
+int im_fwfft( VipsImage *in, VipsImage *out );
+int im_invfft( VipsImage *in, VipsImage *out );
+int im_invfftr( VipsImage *in, VipsImage *out );
+
+int im_freqflt( VipsImage *in, VipsImage *mask, VipsImage *out );
+int im_disp_ps( VipsImage *in, VipsImage *out );
+int im_fractsurf( VipsImage *out, int size, double frd );
+int im_phasecor_fft( VipsImage *in1, VipsImage *in2, VipsImage *out );
+
+int im_cntlines( VipsImage *im, double *nolines, int flag );
+int im_rank( VipsImage *in, VipsImage *out, int width, int height, int index );
+int im_zerox( VipsImage *in, VipsImage *out, int sign );
+
 #ifdef __cplusplus
 }
 #endif /*__cplusplus*/
