@@ -105,7 +105,7 @@
  * @VIPS_DEMAND_STYLE_THINSTRIP: demand in thin (typically 1 pixel high) strips
  * @VIPS_DEMAND_STYLE_ANY: demand geometry does not matter
  *
- * See vips_demand_hint(). Operations can hint to the VIPS image IO system about
+ * See vips_image_pipelinev(). Operations can hint to the VIPS image IO system about
  * the kind of demand geometry they prefer. 
  *
  * These demand styles are given below in order of increasing
@@ -134,7 +134,7 @@
  * file (even indirectly) so any demand style is OK. It's used for things like
  * im_black() where the pixels are calculated.
  *
- * See also: vips_demand_hint().
+ * See also: vips_image_pipelinev().
  */
 
 /**
@@ -710,6 +710,10 @@ vips_image_eval_cb( VipsImage *image, VipsProgress *progress, int *last )
 		fflush( stdout );
 
 		*last = progress->percent;
+
+		/* Needs DEBUG in region.c
+		vips_region_dump_all();
+		 */
 	}
 
 	return( 0 );
@@ -822,7 +826,7 @@ vips_image_build( VipsObject *object )
 
 			if( mode[1] == 's' ) {
 				if( vips_foreign_load_options( filename, &t, 
-					"sequential", TRUE,
+					"access", VIPS_ACCESS_SEQUENTIAL,
 					NULL ) )
 					return( -1 );
 			}
@@ -879,9 +883,9 @@ vips_image_build( VipsObject *object )
                 break;
 
 	case 'a':
-		/* Check parameters.
+		/* Ban crazy numbers. 
 		 */
-		if( image->sizeof_header < 0 ) {
+		if( image->sizeof_header > 1000000 ) {
 			vips_error( "VipsImage", "%s", _( "bad parameters" ) );
 			return( -1 );
 		}
@@ -933,6 +937,7 @@ vips_image_build( VipsObject *object )
 			image->Type = VIPS_INTERPRETATION_MULTIBAND;
 
 		image->dtype = VIPS_IMAGE_SETBUF_FOREIGN;
+		image->dhint = VIPS_DEMAND_STYLE_ANY;
 
 		break;
 
@@ -960,9 +965,15 @@ vips_image_real_invalidate( VipsImage *image )
 {
 	VIPS_DEBUG_MSG( "vips_image_real_invalidate: %p\n", image );
 
+	VIPS_GATE_START( "vips_image_real_invalidate: wait" );
+
 	g_mutex_lock( image->sslock );
+
+	VIPS_GATE_STOP( "vips_image_real_invalidate: wait" );
+
 	(void) vips_slist_map2( image->regions,
 		(VipsSListMap2Fn) vips_region_invalidate, NULL, NULL );
+
 	g_mutex_unlock( image->sslock );
 }
 
@@ -1939,10 +1950,9 @@ int
 vips_image_write( VipsImage *image, VipsImage *out )
 {
 	if( vips_image_pio_input( image ) || 
-		vips_image_copy_fields( out, image ) )
+		vips_image_pipelinev( out, 
+			VIPS_DEMAND_STYLE_THINSTRIP, image, NULL ) )
 		return( -1 );
-        vips_demand_hint( out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, image, NULL );
 
 	/* We generate from @image partially, so we need to keep it about as
 	 * long as @out is about. 
@@ -2358,7 +2368,7 @@ int
 vips__image_wio_output( VipsImage *image )
 {
 #ifdef DEBUG_IO
-	printf( "vips_image_wio_output: WIO output for %s\n", 
+	printf( "vips__image_wio_output: WIO output for %s\n", 
 		image->filename );
 #endif/*DEBUG_IO*/
 
@@ -2367,7 +2377,7 @@ vips__image_wio_output( VipsImage *image )
 		/* Make sure nothing is attached.
 		 */
 		if( image->generate_fn ) {
-			vips_error( "vips_image_wio_output", 
+			vips_error( "vips__image_wio_output", 
 				"%s", _( "image already written" ) );
 			return( -1 );
 		}
@@ -2391,7 +2401,7 @@ vips__image_wio_output( VipsImage *image )
 		break;
 
 	default:
-		vips_error( "vips_image_wio_output", 
+		vips_error( "vips__image_wio_output", 
 			"%s", _( "image not writeable" ) );
 		return( -1 );
 	}
