@@ -140,7 +140,7 @@ vips_get_argv0( void )
  *     </para>
  *   </listitem>
  *   <listitem> 
- *     <para>loads any plugins from $libdir/vips-x.y, where x and y are the
+ *     <para>loads any plugins from $libdir/vips-x.y/, where x and y are the
  *     major and minor version numbers for this VIPS.
  *     </para>
  *   </listitem>
@@ -165,8 +165,13 @@ vips_get_argv0( void )
  *
  * Returns: 0 on success, -1 otherwise
  */
+
+/* vips_init() is actually a macro which checks library and application
+ * compatibility before calling vips__init().
+ */
+
 int
-vips_init( const char *argv0 )
+vips__init( const char *argv0 )
 {
 	extern GType vips_system_get_type( void );
 
@@ -214,6 +219,15 @@ vips_init( const char *argv0 )
 	g_set_prgname( prgname );
 	g_free( prgname );
 
+	vips__thread_profile_attach( "main" );
+
+	/* We can't do VIPS_GATE_START() until command-line processing
+	 * happens, since vips__thread_profile may not be set yet. Call
+	 * directly. 
+	 */
+	vips__thread_gate_start( "init: main" ); 
+	vips__thread_gate_start( "init: startup" ); 
+
 	/* Try to discover our prefix. 
 	 */
 	if( !(prefix = vips_guess_prefix( argv0, "VIPSHOME" )) || 
@@ -256,6 +270,8 @@ vips_init( const char *argv0 )
 	vips_colour_operation_init();
 	vips_histogram_operation_init();
 	vips_convolution_operation_init();
+	vips_freqfilt_operation_init();
+	vips_morphology_operation_init();
 
 	/* Load up any plugins in the vips libdir. We don't error on failure,
 	 * it's too annoying to have VIPS refuse to start because of a broken
@@ -296,9 +312,25 @@ vips_init( const char *argv0 )
 
 	done = TRUE;
 
+	vips__thread_gate_stop( "init: startup" ); 
+
 	return( 0 );
 }
 
+/* Return the sizeof() various important data structures. These are checked
+ * against the headers used to build our caller by vips_init().
+ *
+ * We allow direct access to members of VipsImage and VipsRegion (mostly for
+ * reasons of history), so any change to a superclass of either of these
+ * objects will break our ABI.
+ */
+
+size_t
+vips__get_sizeof_vipsobject( void )
+{
+	return( sizeof( VipsObject ) ); 
+}
+
 /* Call this before vips stuff that uses stuff we need to have inited.
  */
 void
@@ -338,6 +370,27 @@ vips_leak( void )
 	vips__type_leak();
 }
 
+/**
+ * vips_thread_shutdown: 
+ *
+ * Free any thread-private data and flush any profiling information.
+ *
+ * This function needs to be called when a thread that has been using vips
+ * exits. It is called for you by vips_shutdown() and for any threads created
+ * by vips_g_thread_new(). 
+ *
+ * You will need to call it from threads created in
+ * other ways. If you do not call it, vips will generate an error message.
+ *
+ * May be called many times. 
+ */
+void
+vips_thread_shutdown( void )
+{
+	vips__buffer_shutdown();
+	vips__thread_profile_detach();
+}
+
 /**
  * vips_shutdown:
  *
@@ -352,8 +405,23 @@ vips_shutdown( void )
 #endif /*DEBUG*/
 
 	vips_cache_drop_all();
+
 	im_close_plugins();
 
+	/* Mustn't run this more than once. Don't use the VIPS_GATE macro,
+	 * since we don't for gate start.
+	 */
+{
+	static gboolean done = FALSE;
+
+	if( !done ) 
+		vips__thread_gate_stop( "init: main" ); 
+}
+
+	vips_thread_shutdown();
+
+	vips__thread_profile_stop();
+
 	/* In dev releases, always show leaks. But not more than once, it's
 	 * annoying.
 	 */
@@ -441,6 +509,9 @@ static GOptionEntry option_entries[] = {
 	{ "vips-leak", 0, 0, 
 		G_OPTION_ARG_NONE, &vips__leak, 
 		N_( "leak-check on exit" ), NULL },
+	{ "vips-profile", 0, 0, 
+		G_OPTION_ARG_NONE, &vips__thread_profile, 
+		N_( "profile and dump timing on exit" ), NULL },
 	{ "vips-disc-threshold", 0, 0, 
 		G_OPTION_ARG_STRING, &vips__disc_threshold, 
 		N_( "images larger than N are decompressed to disc" ), "N" },
@@ -495,30 +566,6 @@ vips_get_option_group( void )
 	return( option_group );
 }
 
-static char *
-get_current_dir( void )
-{
-	static char buffer[PATH_MAX];
-	char *dir;
-
-	/* We don't use getcwd(3) on SUNOS, because, it does a popen("pwd")
-	 * and, if that wasn't bad enough, hangs in doing so.
-	 */
-#if defined( sun ) && !defined( __SVR4 )
-	dir = getwd( buffer );
-#else   /* !sun */
-	dir = getcwd( buffer, PATH_MAX );
-#endif  /* !sun */
-
-	if( !dir ) {
-		buffer[0] = G_DIR_SEPARATOR;
-		buffer[1] = '\0';
-		dir = buffer;
-	}
-
-	return( dir );
-}
-
 /* Find the prefix part of a dir ... name is the name of this prog from argv0.
  *
  * dir					name		guess prefix
@@ -531,8 +578,8 @@ get_current_dir( void )
 static char *
 extract_prefix( const char *dir, const char *name )
 {
-	char edir[PATH_MAX];
-	char vname[PATH_MAX];
+	char edir[VIPS_PATH_MAX];
+	char vname[VIPS_PATH_MAX];
 	int i;
 
 #ifdef DEBUG
@@ -543,11 +590,15 @@ extract_prefix( const char *dir, const char *name )
 	/* Is dir relative? Prefix with cwd.
 	 */
 	if( !g_path_is_absolute( dir ) ) {
-		vips_snprintf( edir, PATH_MAX, "%s" G_DIR_SEPARATOR_S "%s",
-			get_current_dir(), dir );
+		char *cwd; 
+
+		cwd = g_get_current_dir();
+		vips_snprintf( edir, VIPS_PATH_MAX, 
+			"%s" G_DIR_SEPARATOR_S "%s", cwd, dir );
+		g_free( cwd );
 	}
 	else {
-		vips_strncpy( edir, dir, PATH_MAX );
+		vips_strncpy( edir, dir, VIPS_PATH_MAX );
 	}
 
 	/* Chop off the trailing prog name, plus the trailing
@@ -555,7 +606,7 @@ extract_prefix( const char *dir, const char *name )
 	 */
 	if( !vips_ispostfix( edir, name ) ) 
 		return( NULL );
-	vips_strncpy( vname, edir, PATH_MAX );
+	vips_strncpy( vname, edir, VIPS_PATH_MAX );
 	vname[strlen( edir ) - strlen( name ) - 1] = '\0';
 
 	/* Remove any "/./", any trailing "/.", any trailing "/".
@@ -597,11 +648,11 @@ scan_path( char *path, const char *name )
 
 	for( p = path; 
 		(q = vips_break_token( p, G_SEARCHPATH_SEPARATOR_S )); p = q ) {
-		char str[PATH_MAX];
+		char str[VIPS_PATH_MAX];
 
 		/* Form complete path.
 		 */
-		vips_snprintf( str, PATH_MAX, 
+		vips_snprintf( str, VIPS_PATH_MAX, 
 			"%s" G_DIR_SEPARATOR_S "%s", p, name );
 
 #ifdef DEBUG
@@ -625,7 +676,7 @@ find_file( const char *name )
 {
 	const char *path = g_getenv( "PATH" );
 	char *prefix;
-	char full_path[PATH_MAX];
+	char full_path[VIPS_PATH_MAX];
 
 	if( !path )
 		return( NULL );
@@ -635,12 +686,18 @@ find_file( const char *name )
 #endif /*DEBUG*/
 
 #ifdef OS_WIN32
+{
+	char *dir; 
+
 	/* Windows always searches '.' first, so prepend cwd to path.
 	 */
-	vips_snprintf( full_path, PATH_MAX, "%s" G_SEARCHPATH_SEPARATOR_S "%s",
-		get_current_dir(), path );
+	dir = g_get_current_dir();
+	vips_snprintf( full_path, VIPS_PATH_MAX, 
+		"%s" G_SEARCHPATH_SEPARATOR_S "%s", dir, path );
+	g_free( dir ); 
+}
 #else /*!OS_WIN32*/
-	vips_strncpy( full_path, path, PATH_MAX );
+	vips_strncpy( full_path, path, VIPS_PATH_MAX );
 #endif /*OS_WIN32*/
 
 	if( (prefix = scan_path( full_path, name )) ) 
@@ -688,15 +745,19 @@ guess_prefix( const char *argv0, const char *name )
  	 * a full path in argv[0].
 	 */
 	if( !g_path_is_absolute( argv0 ) ) {
-		char full_path[PATH_MAX];
-		char resolved[PATH_MAX];
+		char full_path[VIPS_PATH_MAX];
+		char *resolved;
+		char *dir;
 
-		vips_snprintf( full_path, PATH_MAX, 
-			"%s" G_DIR_SEPARATOR_S "%s", get_current_dir(), argv0 );
-
-		if( realpath( full_path, resolved ) ) {
-			if( (prefix = extract_prefix( resolved, name )) ) {
+		dir = g_get_current_dir(); 
+		vips_snprintf( full_path, VIPS_PATH_MAX, 
+			"%s" G_DIR_SEPARATOR_S "%s", dir, argv0 );
+		g_free( dir ); 
 
+		if( (resolved = realpath( full_path, NULL )) ) {
+			prefix = extract_prefix( resolved, name );
+			free( resolved ); 
+			if( prefix ) { 
 #ifdef DEBUG
 				printf( "vips_guess_prefix: found \"%s\" "
 					"from cwd\n", prefix );
@@ -736,7 +797,7 @@ vips_guess_prefix( const char *argv0, const char *env_name )
 {
         const char *prefix;
         const char *p;
-        char name[PATH_MAX];
+        char name[VIPS_PATH_MAX];
 
 	/* Already set?
 	 */
@@ -757,15 +818,15 @@ vips_guess_prefix( const char *argv0, const char *env_name )
 	if( strlen( VIPS_EXEEXT ) > 0 ) {
 		const char *olds[] = { VIPS_EXEEXT };
 
-		vips__change_suffix( p, name, PATH_MAX, VIPS_EXEEXT, olds, 1 );
+		vips__change_suffix( p, name, 
+			VIPS_PATH_MAX, VIPS_EXEEXT, olds, 1 );
 	}
 	else
-		vips_strncpy( name, p, PATH_MAX );
+		vips_strncpy( name, p, VIPS_PATH_MAX );
 
 #ifdef DEBUG
 	printf( "vips_guess_prefix: argv0 = %s\n", argv0 );
 	printf( "vips_guess_prefix: name = %s\n", name );
-	printf( "vips_guess_prefix: cwd = %s\n", get_current_dir() );
 #endif /*DEBUG*/
 
 	prefix = guess_prefix( argv0, name );
