@@ -244,6 +244,8 @@ vips_tracked_free( void *s )
 	g_mutex_unlock( vips_tracked_mutex );
 
 	g_free( s );
+
+	VIPS_GATE_FREE( size ); 
 }
 
 static void
@@ -311,6 +313,8 @@ vips_tracked_malloc( size_t size )
 
 	g_mutex_unlock( vips_tracked_mutex );
 
+	VIPS_GATE_MALLOC( size ); 
+
         return( buf );
 }
 
