@@ -349,7 +349,13 @@ vips_argument_instance_free( VipsArgumentInstance *argument_instance )
 VipsArgument *
 vips__argument_table_lookup( VipsArgumentTable *table, GParamSpec *pspec )
 {
-	return( g_hash_table_lookup( table, pspec ) );
+	VipsArgument *argument;
+
+	g_mutex_lock( vips__global_lock );
+	argument = (VipsArgument *) g_hash_table_lookup( table, pspec );
+	g_mutex_unlock( vips__global_lock );
+
+	return( argument );
 }
 
 static void
@@ -1400,6 +1406,7 @@ vips_object_class_install_argument( VipsObjectClass *object_class,
 	GParamSpec *pspec, VipsArgumentFlags flags, int priority, guint offset )
 {
 	VipsArgumentClass *argument_class = g_new( VipsArgumentClass, 1 );
+	GSList *argument_table_traverse;
 
 #ifdef DEBUG
 	printf( "vips_object_class_install_argument: %p %s %s\n", 
@@ -1408,10 +1415,13 @@ vips_object_class_install_argument( VipsObjectClass *object_class,
 		g_param_spec_get_name( pspec ) );
 #endif /*DEBUG*/
 
+	/* object_class->argument* is shared, so we must lock.
+	 */
+	g_mutex_lock( vips__global_lock );
+
 	/* Must be a new one.
 	 */
-	g_assert( !vips__argument_table_lookup( object_class->argument_table,
-		pspec ) );
+	g_assert( !g_hash_table_lookup( object_class->argument_table, pspec ) );
 
 	/* Mustn't have INPUT and OUTPUT both set.
 	 */
@@ -1444,10 +1454,21 @@ vips_object_class_install_argument( VipsObjectClass *object_class,
 			G_TYPE_FROM_CLASS( object_class );
 	}
 
-	object_class->argument_table_traverse = g_slist_prepend(
-		object_class->argument_table_traverse, argument_class );
-	object_class->argument_table_traverse = g_slist_sort(
-		object_class->argument_table_traverse, traverse_sort );
+	/* We read argument_table_traverse without a lock (eg. see 
+	 * vips_argument_map()), so we must be very careful updating it.
+	 */
+	argument_table_traverse = 
+		g_slist_copy( object_class->argument_table_traverse );
+
+	argument_table_traverse = g_slist_prepend(
+		argument_table_traverse, argument_class );
+	argument_table_traverse = g_slist_sort(
+		argument_table_traverse, traverse_sort );
+	VIPS_SWAP( GSList *, 
+		argument_table_traverse, 
+		object_class->argument_table_traverse ); 
+
+	g_slist_free( argument_table_traverse );  
 
 #ifdef DEBUG
 {
@@ -1467,6 +1488,8 @@ vips_object_class_install_argument( VipsObjectClass *object_class,
 	}
 }
 #endif /*DEBUG*/
+
+	g_mutex_unlock( vips__global_lock );
 }
 
 static void
@@ -1959,34 +1982,36 @@ vips_object_set_args( VipsObject *object, const char *p )
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( object );
 
 	VipsToken token;
-	char string[PATH_MAX];
-	char string2[PATH_MAX];
+	char string[VIPS_PATH_MAX];
+	char string2[VIPS_PATH_MAX];
 	GParamSpec *pspec;
 	VipsArgumentClass *argument_class;
 	VipsArgumentInstance *argument_instance;
 
-	if( !(p = vips__token_need( p, VIPS_TOKEN_LEFT, string, PATH_MAX )) )
+	if( !(p = vips__token_need( p, VIPS_TOKEN_LEFT, 
+		string, VIPS_PATH_MAX )) )
 		return( -1 );
 
 	do {
 		if( !(p = vips__token_need( p, VIPS_TOKEN_STRING,
-			string, PATH_MAX )) )
+			string, VIPS_PATH_MAX )) )
 			return( -1 );
 
 		/* We have to look for a '=', ')' or a ',' to see if string is
 		 * a param name or a value.
 		 */
-		if( !(p = vips__token_must( p, &token, string2, PATH_MAX )) )
+		if( !(p = vips__token_must( p, &token, 
+			string2, VIPS_PATH_MAX )) )
 			return( -1 );
 		if( token == VIPS_TOKEN_EQUALS ) {
 			if( !(p = vips__token_need( p, VIPS_TOKEN_STRING,
-				string2, PATH_MAX )) )
+				string2, VIPS_PATH_MAX )) )
 				return( -1 );
 			if( vips_object_set_argument_from_string( object, 
 				string, string2 ) )
 				return( -1 );
 			if( !(p = vips__token_must( p, &token,
-				string2, PATH_MAX )) )
+				string2, VIPS_PATH_MAX )) )
 				return( -1 );
 		}
 		else if( g_object_class_find_property( 
@@ -2022,7 +2047,7 @@ vips_object_set_args( VipsObject *object, const char *p )
 		}
 	} while( token != VIPS_TOKEN_RIGHT );
 
-	if( (p = vips__token_get( p, &token, string, PATH_MAX )) ) {
+	if( (p = vips__token_get( p, &token, string, VIPS_PATH_MAX )) ) {
 		vips_error( class->nickname,
 			"%s", _( "extra tokens after ')'" ) );
 		return( -1 );
@@ -2035,7 +2060,7 @@ VipsObject *
 vips_object_new_from_string( VipsObjectClass *object_class, const char *p )
 {
 	const char *q;
-	char str[PATH_MAX];
+	char str[VIPS_PATH_MAX];
 	VipsObject *object;
 
 	g_assert( object_class );
@@ -2045,9 +2070,9 @@ vips_object_new_from_string( VipsObjectClass *object_class, const char *p )
 	 * everything before that as the principal arg for the constructor.
 	 */
 	if( (q = vips__find_rightmost_brackets( p )) )
-		vips_strncpy( str, p, VIPS_MIN( PATH_MAX, q - p + 1 ) );
+		vips_strncpy( str, p, VIPS_MIN( VIPS_PATH_MAX, q - p + 1 ) );
 	else
-		vips_strncpy( str, p, PATH_MAX );
+		vips_strncpy( str, p, VIPS_PATH_MAX );
 	if( !(object = object_class->new_from_string( str )) )
 		return( NULL );
 
