@@ -93,8 +93,17 @@ G_DEFINE_ABSTRACT_TYPE( VipsOperation, vips_operation, VIPS_TYPE_OBJECT );
 static void
 vips_operation_finalize( GObject *gobject )
 {
+	VipsOperation *operation = VIPS_OPERATION( gobject );
+
 	VIPS_DEBUG_MSG( "vips_operation_finalize: %p\n", gobject );
 
+	if( operation->pixels ) {
+		VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( gobject );
+
+		vips_info( class->nickname, 
+			_( "%d pixels calculated" ), operation->pixels ); 
+	}
+
 	G_OBJECT_CLASS( vips_operation_parent_class )->finalize( gobject );
 }
 
@@ -371,6 +380,7 @@ vips_operation_new( const char *name )
 			_( "class \"%s\" not found" ), name );
 		return( NULL );
 	}
+
 	operation = VIPS_OPERATION( g_object_new( type, NULL ) );
 
 	VIPS_DEBUG_MSG( "vips_operation_new: %s (%p)\n", name, operation );
@@ -555,6 +565,9 @@ vips_call_required_optional( VipsOperation **operation,
 	va_end( a );
 	va_end( b );
 
+	if( result )
+		return( -1 );
+
 	/* Build from cache.
 	 */
 	if( vips_cache_operation_buildp( operation ) )
