@@ -75,6 +75,7 @@
 #define DEBUG_ENVIRONMENT 1
 #define DEBUG_CREATE
 #define DEBUG
+#define VIPS_DEBUG
  */
 
 #ifdef HAVE_CONFIG_H
@@ -184,12 +185,23 @@ enum {
 
 G_DEFINE_TYPE( VipsRegion, vips_region, VIPS_TYPE_OBJECT );
 
+#ifdef VIPS_DEBUG
+static GSList *vips__regions_all = NULL;
+#endif /*VIPS_DEBUG*/
+
 static void
 vips_region_finalize( GObject *gobject )
 {
 #ifdef VIPS_DEBUG
 	VIPS_DEBUG_MSG( "vips_region_finalize: " );
 	vips_object_print_name( VIPS_OBJECT( gobject ) );
+	VIPS_DEBUG_MSG( "\n" );
+#endif /*VIPS_DEBUG*/
+
+#ifdef VIPS_DEBUG
+	g_mutex_lock( vips__global_lock );
+	vips__regions_all = g_slist_remove( vips__regions_all, gobject ); 
+	g_mutex_unlock( vips__global_lock );
 #endif /*VIPS_DEBUG*/
 
 	G_OBJECT_CLASS( vips_region_parent_class )->finalize( gobject );
@@ -203,9 +215,15 @@ vips__region_start( VipsRegion *region )
 	VipsImage *image = region->im;
 
         if( !region->seq && image->start_fn ) {
+		VIPS_GATE_START( "vips__region_start: wait" );
+
                 g_mutex_lock( image->sslock );
+
+		VIPS_GATE_STOP( "vips__region_start: wait" );
+
                 region->seq = image->start_fn( image, 
 			image->client1, image->client2 );
+
                 g_mutex_unlock( image->sslock );
  
                 if( !region->seq ) {
@@ -229,9 +247,15 @@ vips__region_stop( VipsRegion *region )
         if( region->seq && image->stop_fn ) {
 		int result;
 
+		VIPS_GATE_START( "vips__region_stop: wait" );
+
                 g_mutex_lock( image->sslock );
+
+		VIPS_GATE_STOP( "vips__region_stop: wait" );
+
                	result = image->stop_fn( region->seq, 
 			image->client1, image->client2 );
+
                 g_mutex_unlock( image->sslock );
 
 		/* stop function can return an error, but we have nothing we
@@ -246,16 +270,6 @@ vips__region_stop( VipsRegion *region )
         }
 }
 
-/* Free any resources we have.
- */
-static void
-vips_region_reset( VipsRegion *region )
-{
-	VIPS_FREEF( vips_window_unref, region->window );
-	VIPS_FREEF( vips_buffer_unref, region->buffer );
-	region->invalid = FALSE;
-}
-
 static void
 vips_region_dispose( GObject *gobject )
 {
@@ -265,6 +279,7 @@ vips_region_dispose( GObject *gobject )
 #ifdef VIPS_DEBUG
 	VIPS_DEBUG_MSG( "vips_region_dispose: " );
 	vips_object_print_name( VIPS_OBJECT( gobject ) );
+	VIPS_DEBUG_MSG( "\n" );
 #endif /*VIPS_DEBUG*/
 
 	vips_object_preclose( VIPS_OBJECT( gobject ) );
@@ -275,13 +290,21 @@ vips_region_dispose( GObject *gobject )
 
 	/* Free any attached memory.
 	 */
-	vips_region_reset( region ); 
+	VIPS_FREEF( vips_window_unref, region->window );
+	VIPS_FREEF( vips_buffer_unref, region->buffer );
 
 	/* Detach from image. 
 	 */
+	VIPS_GATE_START( "vips_region_dispose: wait" );
+
 	g_mutex_lock( image->sslock );
+
+	VIPS_GATE_STOP( "vips_region_dispose: wait" );
+
 	image->regions = g_slist_remove( image->regions, region );
+
 	g_mutex_unlock( image->sslock );
+
 	region->im = NULL;
 
 	g_object_unref( image );
@@ -325,7 +348,8 @@ vips_region_summary( VipsObject *object, VipsBuf *buf )
 	vips_buf_appendf( buf, "height = %d", region->valid.height );
 
 	if( region->buffer && region->buffer->buf )
-		vips_buf_appendf( buf, ", bytes = %zd", region->buffer->bsize );
+		vips_buf_appendf( buf, ", %.3gMB", 
+			region->buffer->bsize / (1024 * 1024.0) );
 
 	VIPS_OBJECT_CLASS( vips_region_parent_class )->summary( object, buf );
 }
@@ -363,8 +387,12 @@ vips__region_take_ownership( VipsRegion *region )
 	/* Lock so that there's a memory barrier with the thread doing the
 	 * vips__region_no_ownership() before us.
 	 */
+	VIPS_GATE_START( "vips__region_take_ownership: wait" );
+
 	g_mutex_lock( region->im->sslock );
 
+	VIPS_GATE_STOP( "vips__region_take_ownership: wait" );
+
 	if( region->thread != g_thread_self() ) {
 		g_assert( region->thread == NULL );
 
@@ -399,8 +427,12 @@ vips__region_check_ownership( VipsRegion *region )
 void
 vips__region_no_ownership( VipsRegion *region )
 {
+	VIPS_GATE_START( "vips__region_no_ownership: wait" );
+
 	g_mutex_lock( region->im->sslock );
 
+	VIPS_GATE_STOP( "vips__region_no_ownership: wait" );
+
 	vips__region_check_ownership( region );
 
 	region->thread = NULL;
@@ -425,8 +457,14 @@ vips_region_build( VipsObject *object )
 
 	/* We're usually inside the ss lock anyway. But be safe ...
 	 */
+	VIPS_GATE_START( "vips_region_build: wait" );
+
 	g_mutex_lock( image->sslock );
+
+	VIPS_GATE_STOP( "vips_region_build: wait" );
+
 	image->regions = g_slist_prepend( image->regions, region );
+
 	g_mutex_unlock( image->sslock );
 
 	return( 0 );
@@ -451,6 +489,14 @@ static void
 vips_region_init( VipsRegion *region )
 {
 	region->type = VIPS_REGION_NONE;
+
+#ifdef VIPS_DEBUG
+	g_mutex_lock( vips__global_lock );
+	vips__regions_all = g_slist_prepend( vips__regions_all, region ); 
+	printf( "vips_region_init: %d regions in vips\n", 
+		g_slist_length( vips__regions_all ) ); 
+	g_mutex_unlock( vips__global_lock );
+#endif /*VIPS_DEBUG*/
 }
 
 /**
@@ -534,22 +580,24 @@ vips_region_buffer( VipsRegion *reg, VipsRect *r )
 		return( -1 );
 	}
 
+	VIPS_FREEF( vips_window_unref, reg->window );
+
 	/* Have we been asked to drop caches? We want to throw everything
 	 * away.
 	 *
 	 * If not, try to reuse the current buffer.
 	 */
 	if( reg->invalid ) {
-		vips_region_reset( reg );
+		VIPS_FREEF( vips_buffer_unref, reg->buffer );
+		reg->invalid = FALSE;
+
 		if( !(reg->buffer = vips_buffer_new( im, &clipped )) ) 
 			return( -1 );
 	}
 	else {
-		/* Don't call vips_region_reset() ... we combine buffer unref 
-		 * and new buffer ref in one call to reduce malloc/free 
-		 * cycling.
+		/* We combine buffer unref and new buffer ref in one call 
+		 * to reduce malloc/free cycling.
 		 */
-		VIPS_FREEF( vips_window_unref, reg->window );
 		if( !(reg->buffer = 
 			vips_buffer_unref_ref( reg->buffer, im, &clipped )) ) 
 			return( -1 );
@@ -604,10 +652,13 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 		return( -1 );
 	}
 
+	VIPS_FREEF( vips_buffer_unref, reg->buffer );
+	VIPS_FREEF( vips_window_unref, reg->window );
+	reg->invalid = FALSE;
+
 	if( image->data ) {
 		/* We have the whole image available ... easy!
 		 */
-		vips_region_reset( reg );
 
 		/* We can't just set valid = clipped, since this may be an
 		 * incompletely calculated memory buffer. Just set valid to r.
@@ -625,8 +676,6 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 			reg->window->top > clipped.top ||
 			reg->window->top + reg->window->height < 
 				clipped.top + clipped.height ) {
-			vips_region_reset( reg );
-
 			if( !(reg->window = vips_window_ref( image, 
 				clipped.top, clipped.height )) )
 				return( -1 );
@@ -753,7 +802,9 @@ vips_region_region( VipsRegion *reg,
 
 	/* Init new stuff.
 	 */
-	vips_region_reset( reg );
+	VIPS_FREEF( vips_buffer_unref, reg->buffer );
+	VIPS_FREEF( vips_window_unref, reg->window );
+	reg->invalid = FALSE;
 	reg->valid = final;
 	reg->bpl = dest->bpl;
 	reg->data = VIPS_REGION_ADDR( dest, clipped2.left, clipped2.top );
@@ -1323,3 +1374,34 @@ vips_region_prepare_many( VipsRegion **reg, VipsRect *r )
 
 	return( 0 );
 }
+
+#ifdef VIPS_DEBUG
+static void *
+vips_region_dump_all_cb( VipsRegion *region, size_t *alive )
+{
+	char str[2048];
+	VipsBuf buf = VIPS_BUF_STATIC( str );
+
+	vips_object_summary( VIPS_OBJECT( region ), &buf ); 
+	printf( "%s\n", vips_buf_all( &buf ) ); 
+
+	if( region->buffer && region->buffer->buf )
+		*alive += region->buffer->bsize;
+
+	return( NULL ); 
+}
+
+void
+vips_region_dump_all( void )
+{
+	size_t alive;
+
+	g_mutex_lock( vips__global_lock );
+	alive = 0;
+	printf( "%d regions in vips\n", g_slist_length( vips__regions_all ) );
+	vips_slist_map2( vips__regions_all, 
+		(VipsSListMap2Fn) vips_region_dump_all_cb, &alive, NULL );
+	printf( "%gMB alive\n", alive / (1024 * 1024.0) ); 
+	g_mutex_unlock( vips__global_lock );
+}
+#endif /*VIPS_DEBUG*/
