@@ -55,6 +55,9 @@
  * 	- switch to new history thing, switch im_errormsg() too
  * 24/1/11
  * 	- gtk-doc
+ * 12/7/12
+ * 	- always allocate local to an output descriptor ... stops ref cycles
+ * 	  with the new base class
  */
 
 /*
@@ -73,7 +76,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -101,7 +105,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <assert.h>
 #include <math.h>
 
 #include <vips/vips.h>
@@ -212,7 +215,7 @@ build_node( SymbolTable *st, char *name )
 	node->dirty = 0;
 	node->mwidth = -2;
 	node->st = st;
-	im__transform_init( &node->cumtrn );
+	vips__transform_init( &node->cumtrn );
 	node->trnim = NULL;
 	node->arg1 = NULL;
 	node->arg2 = NULL;
@@ -270,7 +273,7 @@ overlap_destroy( OverlapInfo *lap )
 	JoinNode *node = lap->node;
 
 	node->overlaps = g_slist_remove( node->overlaps, lap );
-	assert( node->st->novl > 0 );
+	g_assert( node->st->novl > 0 );
 	node->st->novl--;
 }
 
@@ -405,7 +408,7 @@ calc_geometry( JoinNode *node )
 		node->cumtrn.iarea.top = 0;
 		node->cumtrn.iarea.width = um.width;
 		node->cumtrn.iarea.height = um.height;
-		im__transform_set_area( &node->cumtrn );
+		vips__transform_set_area( &node->cumtrn );
 		break;
 
 	case JOIN_CP:
@@ -422,7 +425,7 @@ calc_geometry( JoinNode *node )
 			node->cumtrn.iarea.top = 0;
 			node->cumtrn.iarea.width = node->im->Xsize;
 			node->cumtrn.iarea.height = node->im->Ysize;
-			im__transform_set_area( &node->cumtrn );
+			vips__transform_set_area( &node->cumtrn );
 		}
 		break;
 
@@ -438,7 +441,7 @@ calc_geometry( JoinNode *node )
  * have circularity.
  */
 static int
-propogate_transform( JoinNode *node, Transformation *trn )
+propogate_transform( JoinNode *node, VipsTransformation *trn )
 {
 	if( !node )
 		return( 0 );
@@ -458,7 +461,7 @@ propogate_transform( JoinNode *node, Transformation *trn )
 
 	/* Transform us, and recalculate our position and size.
 	 */
-	im__transform_add( &node->cumtrn, trn, &node->cumtrn );
+	vips__transform_add( &node->cumtrn, trn, &node->cumtrn );
 	calc_geometry( node );
 
 	return( 0 );
@@ -473,7 +476,7 @@ make_join( SymbolTable *st, JoinType type,
 	JoinNode *arg1, JoinNode *arg2, JoinNode *out, 
 	double a, double b, double dx, double dy, int mwidth )
 {
-	Transformation trn;
+	VipsTransformation trn;
 
 	/* Check output is ok.
 	 */
@@ -497,8 +500,10 @@ make_join( SymbolTable *st, JoinType type,
 	out->thistrn.b = -b;
 	out->thistrn.c = b;
 	out->thistrn.d = a;
-	out->thistrn.dx = dx;
-	out->thistrn.dy = dy;
+	out->thistrn.idx = 0;
+	out->thistrn.idy = 0;
+	out->thistrn.odx = dx;
+	out->thistrn.ody = dy;
 
 	/* Clean the table and propogate the transform down the RHS of the
 	 * graph.
@@ -517,8 +522,10 @@ make_join( SymbolTable *st, JoinType type,
 	trn.b = 0.0;
 	trn.c = 0.0;
 	trn.d = 1.0;
-	trn.dx = -out->cumtrn.oarea.left;
-	trn.dy = -out->cumtrn.oarea.top;
+	trn.idx = 0;
+	trn.idy = 0;
+	trn.odx = -out->cumtrn.oarea.left;
+	trn.ody = -out->cumtrn.oarea.top;
 	clean_table( st );
 	if( propogate_transform( out, &trn ) )
 		return( -1 );
@@ -705,7 +712,6 @@ static JoinNode *
 find_root( SymbolTable *st )
 {
 	JoinNode *root;
-	JoinNode *notroot;
 
 	/* Clean the table, then scan it, setting all pointed-to nodes dirty.
 	 */
@@ -729,7 +735,7 @@ find_root( SymbolTable *st )
 	 * more than one root.
 	 */
 	root->dirty = 1;
-	if( (notroot = im__map_table( st, is_root, NULL, NULL )) ) {
+	if( im__map_table( st, is_root, NULL, NULL ) ) {
 		im_error( "im_global_balance", 
 			"%s", _( "more than one root" ) );
 		return( NULL );
@@ -748,7 +754,7 @@ im__parse_desc( SymbolTable *st, IMAGE *in )
 	for( p = in->history_list; p; p = p->next ) {
 		GValue *value = (GValue *) p->data;
 
-		assert( G_VALUE_TYPE( value ) == IM_TYPE_REF_STRING );
+		g_assert( G_VALUE_TYPE( value ) == IM_TYPE_REF_STRING );
 
 		if( process_line( st, im_ref_string_get( value ) ) )
 			return( -1 );
@@ -978,12 +984,12 @@ extract_rect( IMAGE *in, IMAGE *out, Rect *r )
  * has 255 for every pixel where both images are non-zero.
  */
 static int
-make_overlap_mask( IMAGE *ref, IMAGE *sec, IMAGE *mask, 
+make_overlap_mask( IMAGE *mem, IMAGE *ref, IMAGE *sec, IMAGE *mask, 
 	Rect *rarea, Rect *sarea )
 {
 	IMAGE *t[6];
 
-	if( im_open_local_array( mask, t, 6, "mytemps", "p" ) ||
+	if( im_open_local_array( mem, t, 6, "mytemps", "p" ) ||
 		extract_rect( ref, t[0], rarea ) ||
 		extract_rect( sec, t[1], sarea ) ||
 		im_extract_band( t[0], t[2], 0 ) ||
@@ -1014,7 +1020,7 @@ count_nonzero( IMAGE *in, gint64 *count )
  * mask is true.
  */
 static DOUBLEMASK *
-find_image_stats( IMAGE *in, IMAGE *mask, Rect *area )
+find_image_stats( IMAGE *mem, IMAGE *in, IMAGE *mask, Rect *area )
 {
 	DOUBLEMASK *stats;
 	IMAGE *t[4];
@@ -1022,7 +1028,7 @@ find_image_stats( IMAGE *in, IMAGE *mask, Rect *area )
 
 	/* Extract area, build black image, mask out pixels we want.
 	 */
-	if( im_open_local_array( in, t, 4, "find_image_stats", "p" ) ||
+	if( im_open_local_array( mem, t, 4, "find_image_stats", "p" ) ||
 		extract_rect( in, t[0], area ) ||
 		im_black( t[1], t[0]->Xsize, t[0]->Ysize, t[0]->Bands ) ||
 		im_clip2fmt( t[1], t[2], t[0]->BandFmt ) ||
@@ -1031,7 +1037,7 @@ find_image_stats( IMAGE *in, IMAGE *mask, Rect *area )
 
 	/* Get stats from masked image.
 	 */
-	if( !(stats = im_local_dmask( in, im_stats( t[3] ) )) ) 
+	if( !(stats = im_local_dmask( mem, im_stats( t[3] ) )) ) 
 		return( NULL );
 
 	/* Number of non-zero pixels in mask.
@@ -1061,7 +1067,8 @@ find_image_stats( IMAGE *in, IMAGE *mask, Rect *area )
 static int
 find_overlap_stats( OverlapInfo *lap )
 {
-	IMAGE *t1 = im_open_local( lap->node->im, "find_overlap_stats:1", "p" );
+	IMAGE *mem = lap->node->st->im; 
+	IMAGE *t1 = im_open_local( mem, "find_overlap_stats:1", "p" );
 	Rect rarea, sarea;
 
 	/* Translate the overlap area into the coordinate scheme for the main
@@ -1080,15 +1087,17 @@ find_overlap_stats( OverlapInfo *lap )
 
 	/* Make a mask for the overlap.
 	 */
-	if( make_overlap_mask( lap->node->trnim, lap->other->trnim, t1,
-		&rarea, &sarea ) )
+	if( make_overlap_mask( mem, 
+		lap->node->trnim, lap->other->trnim, t1, &rarea, &sarea ) )
 		return( -1 );
 
 	/* Find stats for that area.
 	 */
-	if( !(lap->nstats = find_image_stats( lap->node->trnim, t1, &rarea )) )
+	if( !(lap->nstats = find_image_stats( mem, 
+		lap->node->trnim, t1, &rarea )) )
 		return( -1 );
-	if( !(lap->ostats = find_image_stats( lap->other->trnim, t1, &sarea )) )
+	if( !(lap->ostats = find_image_stats( mem, 
+		lap->other->trnim, t1, &sarea )) )
 		return( -1 );
 
 	return( 0 );
@@ -1144,9 +1153,10 @@ test_overlap( JoinNode *other, JoinNode *node )
 	if( !(lap = build_overlap( node, other, &overlap )) )
 		return( node );
 
-	/* Calculate overlap statistics.
+	/* Calculate overlap statistics. Open stuff relative to this, and 
+	 * free quickly.
 	 */
-	if( find_overlap_stats( lap ) )
+	if( find_overlap_stats( lap ) ) 
 		return( node );
 
 	/* If the pixel count either masked overlap is trivial, ignore this
@@ -1522,12 +1532,13 @@ generate_trn_leaves( JoinNode *node, SymbolTable *st )
 		/* Special case: if this is an untransformed leaf (there will
 		 * always be at least one), then skip the affine.
 		 */
-		if( im__transform_isidentity( &node->cumtrn ) )
+		if( vips__transform_isidentity( &node->cumtrn ) )
 			node->trnim = node->im;
 		else
 			if( !(node->trnim = 
-				im_open_local( node->im, "trnleaf:1", "p" )) ||
-				im__affine( node->im, node->trnim, 
+				im_open_local( node->st->im, 
+					"trnleaf:1", "p" )) ||
+				vips__affine( node->im, node->trnim, 
 					&node->cumtrn ) ) 
 				return( node );
 	}
