@@ -89,7 +89,8 @@
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA
 
  */
 
@@ -593,8 +594,8 @@ tb_blend_labpack( REGION *or, MergeInfo *inf, Overlapping *ovlap, Rect *oreg )
 
 		/* Unpack two bits we want.
 		 */
-		imb_LabQ2Lab( pr, r, oreg->width );
-		imb_LabQ2Lab( ps, s, oreg->width );
+		vips__LabQ2Lab_vec( r, pr, oreg->width );
+		vips__LabQ2Lab_vec( s, ps, oreg->width );
 
 		/* Blend as floats.
 		 */
@@ -602,7 +603,7 @@ tb_blend_labpack( REGION *or, MergeInfo *inf, Overlapping *ovlap, Rect *oreg )
 
 		/* Re-pack to output buffer.
 		 */
-		imb_Lab2LabQ( inf->merge, q, oreg->width );
+		vips__Lab2LabQ_vec( q, inf->merge, oreg->width );
 	}
 
 	return( 0 );
