@@ -79,6 +79,8 @@
  * 14/12/12
  * 	- redone as a class
  * 	- added input space translation
+ * 22/1/14
+ * 	- auto RAD decode
  */
 
 /*
@@ -300,6 +302,8 @@ vips_affine_gen( VipsRegion *or, void *seq, void *a, void *b, gboolean *stop )
 		clipped.height );
 #endif /*DEBUG*/
 
+	VIPS_GATE_START( "vips_affine_gen: work" ); 
+
 	/* Resample! x/y loop over pixels in the output image (5).
 	 */
 	for( y = to; y < bo; y++ ) {
@@ -366,6 +370,8 @@ vips_affine_gen( VipsRegion *or, void *seq, void *a, void *b, gboolean *stop )
 		}
 	}
 
+	VIPS_GATE_STOP( "vips_affine_gen: work" ); 
+
 	return( 0 );
 }
 
@@ -380,7 +386,7 @@ vips_affine_build( VipsObject *object )
 		vips_object_local_array( object, 4 );
 
 	VipsImage *in;
-	gboolean repack;
+	VipsDemandStyle hint; 
 	int window_size;
 	int window_offset;
 	double edge;
@@ -388,7 +394,7 @@ vips_affine_build( VipsObject *object )
 	if( VIPS_OBJECT_CLASS( vips_affine_parent_class )->build( object ) )
 		return( -1 );
 
-	if( vips_check_coding_noneorlabq( class->nickname, resample->in ) )
+	if( vips_check_coding_known( class->nickname, resample->in ) )
 		return( -1 );
 	if( vips_check_vector_length( class->nickname, 
 		affine->matrix->n, 4 ) )
@@ -459,41 +465,42 @@ vips_affine_build( VipsObject *object )
 		return( -1 );
 	}
 
-	/* Unpack labq for processing ... we repack after, see below.
+	/* Unpack for processing.
 	 */
-	repack = FALSE;
 	if( in->Coding == VIPS_CODING_LABQ ) {
 		if( vips_LabQ2LabS( in, &t[0], NULL ) )
 			return( -1 );
-		repack = TRUE;
 		in = t[0];
+	} 
+	else if( in->Coding == VIPS_CODING_RAD ) {
+		if( vips_rad2float( in, &t[1], NULL ) )
+			return( -1 );
+		in = t[1];
 	}
 
 	/* Add new pixels around the input so we can interpolate at the edges.
 	 */
-	if( vips_embed( in, &t[1], 
+	if( vips_embed( in, &t[2], 
 		window_offset, window_offset, 
 		in->Xsize + window_size, in->Ysize + window_size,
 		"extend", VIPS_EXTEND_COPY,
 		NULL ) )
 		return( -1 );
-	in = t[1];
-
-	if( vips_image_copy_fields( resample->out, in ) )
-		return( -1 );
-
-	resample->out->Xsize = affine->trn.oarea.width;
-	resample->out->Ysize = affine->trn.oarea.height;
+	in = t[2];
 
 	/* Normally SMALLTILE ... except if this is a size up/down affine.
 	 */
 	if( affine->trn.b == 0.0 && 
 		affine->trn.c == 0.0 ) 
-		vips_demand_hint( resample->out, 
-			VIPS_DEMAND_STYLE_FATSTRIP, in, NULL );
+		hint = VIPS_DEMAND_STYLE_FATSTRIP; 
 	else 
-		vips_demand_hint( resample->out, 
-			VIPS_DEMAND_STYLE_SMALLTILE, in, NULL );
+		hint = VIPS_DEMAND_STYLE_SMALLTILE;
+
+	if( vips_image_pipelinev( resample->out, hint, in, NULL ) )
+		return( -1 );
+
+	resample->out->Xsize = affine->trn.oarea.width;
+	resample->out->Ysize = affine->trn.oarea.height;
 
 	/* Generate!
 	 */
@@ -502,17 +509,6 @@ vips_affine_build( VipsObject *object )
 		in, affine ) )
 		return( -1 );
 
-	if( repack ) {
-		VipsImage *x;
-
-		if( vips_LabS2LabQ( resample->out, &x, NULL ) )
-			return( -1 );
-
-		VIPS_UNREF( resample->out );
-
-		resample->out = x;
-	}
-
 	/* Finally: can now set Xoffset/Yoffset.
 	 */
 	resample->out->Xoffset = affine->trn.odx - affine->trn.oarea.left;
