@@ -137,8 +137,6 @@ vips_shrink_start( VipsImage *out, void *a, void *b )
 	if( !(seq = IM_NEW( out, VipsShrinkSequence )) )
 		return( NULL );
 
-	/* Init!
-	 */
 	seq->ir = vips_region_new( in );
 	if( !(seq->sum = (VipsPel *) VIPS_ARRAY( out, in->Bands, double )) ) {
 		vips_shrink_stop( seq, in, shrink );
@@ -294,9 +292,13 @@ vips_shrink_gen( VipsRegion *or, void *vseq, void *a, void *b, gboolean *stop )
 		if( vips_region_prepare( ir, &s ) )
 			return( -1 );
 
+		VIPS_GATE_START( "vips_shrink_gen: work" ); 
+
 		vips_shrink_gen2( shrink, seq, 
 			or, ir, 
 			r->left, r->top + y, r->width, height );
+
+		VIPS_GATE_STOP( "vips_shrink_gen: work" ); 
 	}
 
 	return( 0 );
@@ -336,15 +338,13 @@ vips_shrink_build( VipsObject *object )
 		shrink->yshrink == 1.0 )
 		return( vips_image_write( resample->in, resample->out ) );
 
-	if( vips_image_copy_fields( resample->out, resample->in ) )
-		return( -1 );
-
 	/* THINSTRIP will work, anything else will break seq mode. If you 
 	 * combine shrink with conv you'll need to use a line cache to maintain
 	 * sequentiality.
 	 */
-	vips_demand_hint( resample->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, resample->in, NULL );
+	if( vips_image_pipelinev( resample->out, 
+		VIPS_DEMAND_STYLE_THINSTRIP, resample->in, NULL ) )
+		return( -1 );
 
 	/* Size output. Note: we round the output width down!
 	 *
